package cn.kdan.cloud.pdf.office.account.controller;

import cn.kdan.cloud.pdf.office.account.model.User;
import cn.kdan.cloud.pdf.office.api.account.vo.UserDetailVO;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.api.account.vo.UserPageVO;
import cn.kdan.cloud.pdf.office.api.account.dto.*;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.dto.UserRegisterDTO;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;

import cn.kdan.cloud.pdf.office.account.constant.UserConstant;
import cn.kdan.cloud.pdf.office.account.service.UserService;

import com.github.pagehelper.PageInfo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

@RestController
@RequestMapping("user")
public class UserController {

    @Autowired
    private UserService userService;



    /**
     * 获取指定产品的账号
     *
     * @param account 账号
     * @param platformType 平台类型
     * @return
     */
    @GetMapping("getByAppAccount")
    public ResultMap<UserVO> getByAppAccount(@RequestParam String account, @RequestParam String platformType) {
        return new ResultMap<>(CommonConstant.SUCCESS, UserConstant.USER_QUERY_SUCCESS, userService.getByAppAccount(account, platformType));
    }

    /**
     * 获取用户信息
     * @param id
     * @return
     */
    @GetMapping("getById")
    public ResultMap<UserVO> getById(@RequestParam String id) {
        return new ResultMap<>(CommonConstant.SUCCESS, UserConstant.USER_QUERY_SUCCESS, userService.getById(id));
    }

    /**
     * 用户信息包括用户的会员信息
     *
     * @param id id
     * @return UserInfoVO
     */
    @GetMapping("getInfoById")
    public ResultMap<UserInfoVO> getInfoById(@RequestParam String id) {
        return new ResultMap<>(CommonConstant.SUCCESS, UserConstant.USER_QUERY_SUCCESS, userService.getInfoById(id));
    }

    @GetMapping("getMemberInfoById")
    public ResultMap<UserInfoVO> getMemberInfoById(@RequestParam String id) {
        return new ResultMap<>(CommonConstant.SUCCESS, UserConstant.USER_QUERY_SUCCESS, userService.getMemberInfoById(id));
    }


    /**
     * 注册
     *
     * @param userRegisterDTO 注册对象
     * @return Boolean
     */
    @PostMapping("/register")
    public ResultMap<Boolean> register(@RequestBody UserRegisterDTO userRegisterDTO) {
        userService.register(userRegisterDTO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,Boolean.TRUE);
    }

    /**
     * 分页查询
     *
     * @param selectType 筛选类型 正选1 反选2
     * @param appIds 产品id
     * @param roles 账号身份
     * @param accountTypes 账号类型
     * @param accountSources 账号来源
     * @param payTypes 付费类型
     * @param queryTimeType 查询时间类型 1 创建时间 2更新时间
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param queryString 查询string id或者邮箱
     * @return PageInfo
     */
    @GetMapping("/page")
    public ResultMap<PageInfo<UserPageVO>> page(  @RequestParam(required = false, defaultValue = "1") Integer page,
                                                  @RequestParam(required = false, defaultValue = "10") Integer pageSize,
                                                  @RequestParam(required = false) String selectType,
                                                  @RequestParam(required = false)  List<String> appIds,
                                                  @RequestParam(required = false)  List<String> roles,
                                                  @RequestParam(required = false)  List<String> accountTypes,
                                                  @RequestParam(required = false)  List<String> accountSources,
                                                  @RequestParam(required = false)  List<String> payTypes,
                                                  @RequestParam(required = false)  String queryTimeType,
                                                  @RequestParam(required = false)  Date startTime,
                                                  @RequestParam(required = false)  Date endTime,
                                                  @RequestParam(required = false)  List<String> queryString) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,userService.page(page,pageSize,selectType,appIds,roles,accountTypes,accountSources,payTypes,queryTimeType,startTime,endTime,queryString));
    }

    /**
     * 运营后台查看账号详情
     *
     * @param userId 用户id
     * @return ResultMap
     */
    @GetMapping("/detail")
    public ResultMap<UserDetailVO> detail(@RequestParam String userId) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,userService.detail(userId));
    }

    /**
     * 支付成功之后修改用户状态
     *
     * @param updateUserForOrderDTO dto
     */
    @PostMapping("/updateUserForOrder")
    public ResultMap<Boolean> updateUserForOrder(@RequestBody UpdateUserForOrderDTO updateUserForOrderDTO) {
        userService.updateUserForOrder(updateUserForOrderDTO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,Boolean.TRUE);
    }

    /**
     * 后台强制注销用户
     *
     * @param logOffUserDTO
     */
    @PostMapping("/logOff")
    public ResultMap<Boolean> logOff(@RequestBody LogOffUserDTO logOffUserDTO) {
        userService.logOff(logOffUserDTO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,Boolean.TRUE);
    }


    /**
     * 更新
     *
     * @param userVO
     */
    @PostMapping("/updateUser")
    public ResultMap<Boolean> updateUser(@RequestBody UserVO userVO) {
        User user = new User();
        BeanUtils.copyProperties(userVO,user);
        userService.updateUser(user);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,Boolean.TRUE);
    }

    /**
     * 用户自己手动注销
     *
     * @param code
     * @param appId
     * @return
     */
    @PostMapping("/logOffForUser")
    public ResultMap<Boolean> logOffForUser(@RequestParam String code, @RequestParam String userId, String appId) {
        userService.logOffForUser(code,userId, appId);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,Boolean.TRUE);
    }

    /**
     * 根据email查询用户
     *
     * @param email email
     * @return UserVO
     */
    @GetMapping("/getByEmail")
    public ResultMap<UserVO> getByEmail(@RequestParam("email") String email){
        return ResultMap.success(userService.getByEmail(email));
    }

    /**
     * 判断用户邀请了多少人getInviteNum
     */
    @GetMapping("/getInviteNum")
    public ResultMap<Integer> getInviteNum(@RequestParam("userId") String userId){
        return ResultMap.success(userService.getInviteNum(userId));
    }
}
