package cn.kdan.cloud.pdf.office.account.service;

import cn.kdan.cloud.pdf.office.api.account.vo.UserDetailVO;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.api.account.vo.UserPageVO;
import cn.kdan.cloud.pdf.office.api.account.dto.*;
import cn.kdan.cloud.pdf.office.common.dto.UserRegisterDTO;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import com.github.pagehelper.PageInfo;
import cn.kdan.cloud.pdf.office.account.model.User;


import java.util.Date;
import java.util.List;

/**
 * 用户服务
 */
public interface UserService {


    UserVO getById(String userId);

    User getUser(String userId);

    /**
     * 分页查询
     *
     * @param selectType 筛选类型 正选1 反选2
     * @param appIds 产品id
     * @param roles 账号身份
     * @param accountTypes 账号类型
     * @param accountSources 账号来源
     * @param payTypes 付费类型
     * @param queryTimeType 查询时间类型 1 创建时间 2更新时间
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param queryString 查询string id或者邮箱
     * @return PageInfo
     */
    PageInfo<UserPageVO> page(Integer page, Integer pageSize,
                              String selectType,
                              List<String> appIds,
                              List<String> roles,
                              List<String> accountTypes,
                              List<String> accountSources,
                              List<String> payTypes,
                              String queryTimeType,
                              Date startTime,
                              Date endTime,
                              List<String> queryString);


    User getByAccount(String account, String platformType);

    /**
     * 登录时查询
     *
     * @param account
     * @param platformType
     * @return
     */
    UserVO getByAppAccount(String account, String platformType);


    /**
     * 获取用户（用于用户权限转移）
     * @param userId 用户id
     * @return UserPageVO
     */
    UserPageVO getUserForTransfer(String userId);

    /**
     * 运营后台获取账号详情
     *
     * @param userId 用户id
     * @return UserDetailVO
     */
    UserDetailVO detail(String userId);

    /**
     * 订单成功之后修改用户状态
     *
     * @param updateUserForOrderDTO dto
     */
    void updateUserForOrder(UpdateUserForOrderDTO updateUserForOrderDTO);


    /**
     * 注册用户
     *
     * @param userRegisterDTO userRegisterDTO
     */
    void register(UserRegisterDTO userRegisterDTO);


    /**
     * 后台强制注销用户
     * @param logOffUserDTO
     */
    void logOff(LogOffUserDTO logOffUserDTO);

    /**
     * 更新
     *
     * @param user 用户
     */
    void updateUser(User user);

    /**
     * 用户手动注销
     *
     * @param code   code 邮箱验证码
     * @param userId 用户id
     * @param appId
     */
    void logOffForUser(String code, String userId, String appId);

    /**
     * 用户信息包括用户的会员信息
     *
     * @param id id
     * @return UserInfoVO
     */
    UserInfoVO getInfoById(String id);

    UserInfoVO getMemberInfoById(String id);

    Integer getInviteNum(String userId);

    void handleLogOff(User user, String remark);


    UserVO getByEmail(String email);
}
