package cn.kdan.cloud.pdf.office.common.aop;

import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

/**
 * 处理service里方法执行超时打印日期提醒的aop
 * @author liangcai.liu
 * @version V1.0
 * Date:  2018/8/30
 * Time: 14:50
 */
@Aspect
@Component
@Order(2)
public class PerformanceAspect {
    private final Logger logger = LoggerFactory.getLogger(getClass());

    @Around("execution(* com.wondersgroup.cloud..*.service.*.*(..))")
    public Object performanceCalculate(ProceedingJoinPoint joinPoint) throws Throwable {
        long current = System.currentTimeMillis();

        Object obj;
        Object[] args = joinPoint.getArgs();
        obj = joinPoint.proceed(args);

        long after = System.currentTimeMillis();

        long timeUse = (after - current);
        if(timeUse >= 1000){
            logger.warn("----------发现性能问题方法: "+ joinPoint.getSignature().getDeclaringTypeName() + "." + joinPoint.getSignature().getName() +",时间：" + timeUse +"ms");
        }
        return obj;
    }

}
