package cn.kdan.cloud.pdf.office.common.enums;

public enum CardButtonEnum {
    // 永久会员 - 升级
    /**
     * "Mac单平台标准版永久（原dmg独有）" -> 永久会员 -> 升级
     */
    mac_standard("standard-permanent-mac", "1", "upgrade"),

    /**
     * "Windows单平台标准版永久（原exe独有）" -> 永久会员 -> 升级
     */
    windows_standard("standard-permanent-windows", "1", "upgrade"),

    /**
     * "MAC单平台高级版永久" -> 永久会员 -> 升级
     */
    mac_advanced("advanced-permanent-mac-upgrade", "1", "upgrade"),

    /**
     * "Windows单平台高级版永久" -> 永久会员 -> 升级
     */
    windows_advanced("advanced-permanent-windows", "1", "upgrade"),

    /**
     * "iOS单平台高级版永久（iOS独有）" -> 永久会员 -> 升级
     */
    ios_advanced("advanced-annual-permanent-ios", "1", "upgrade"),

    /**
     * "Android单平台高级版永久（Android独有）" -> 永久会员 -> 升级
     */
    android_advanced("advanced-annual-subscription-month-android", "1", "upgrade"),

    /**
     * "Mac&Windows双平台高级版永久" -> 永久会员 -> 升级
     */
    mac_windows_dual_advanced("advanced-permanent", "1", "upgrade"),

    // 订阅中 - 升级
    /**
     * "MAC单平台高级版年订阅（原Mac免费版独有）" -> 会员期内 -> 升级 （暂无法实现）
     */
    mac_advanced_annual_subscription("mac_advanced_annual", "2", ""),

    /**
     * "MAC单平台高级版6个月订阅（原Mac免费版独有）" -> 会员期内 -> 升级（暂无法实现）
     */
    mac_advanced_six_month_subscription("mac_advanced_6_month", "2", ""),

    /**
     * "Android单平台高级版月订阅（Android独有）" -> 会员期内 -> 升级
     */
    android_advanced_month_subscription("advanced-annual-subscription-month-android", "2", ""),

    /**
     * "全平台标准版年订阅（官网、dmg、exe独有）" -> 会员期内 -> 升级
     */
    all_platform_standard_annual_subscription("standard-annual-subscription", "2", "upgrade"),

    /**
     * "全平台高级版年订阅" -> 会员期内 -> 升级
     */
    all_platform_advanced_annual_subscription("advanced-annual-subscription", "2", ""),

    // 订阅中但取消了续订 - 提前续费
    /**
     * "MAC单平台高级版年订阅（原Mac免费版独有）" -> 会员期内（但取消了自动续订） -> 提前续费
     */
    mac_advanced_annual_cancelled_renewal("advanced-annual-subscription-mac", "3", ""),

    /**
     * "MAC单平台高级版6个月订阅（原Mac免费版独有）" -> 会员期内（但取消了自动续订） -> 提前续费
     */
    mac_advanced_six_month_cancelled_renewal("advanced-annual-subscription-six-month-mac", "3", ""),

    /**
     * "Android单平台高级版月订阅（Android独有）" -> 会员期内（但取消了自动续订） -> 提前续费
     */
    android_advanced_month_cancelled_renewal("advanced-annual-subscription-month-android", "3", "prepay"),

    /**
     * "全平台标准版年订阅（官网、dmg、exe独有）" -> 会员期内（但取消了自动续订） -> 提前续费
     */
    all_platform_standard_annual_cancelled_renewal("standard-annual-subscription", "3", "prepay"),

    /**
     * "全平台高级版年订阅" -> 会员期内（但取消了自动续订） -> 提前续费
     */
    all_platform_advanced_annual_cancelled_renewal("advanced-annual-subscription", "3", "prepay");
    /**
     * 用户账号下的权益
     */
    private final String code;
    /**
     * 用户状态 1永久会员2订阅中3订阅中但取消了续订
     */
    private final String status;
    /**
     * 操作类型 upgrade prepay
     */
    private final String button;
    CardButtonEnum(String code, String status, String button) {
        this.code = code;
        this.status = status;
        this.button = button;
    }

    public String getCode() {
        return code;
    }

    public String getStatus() {
        return status;
    }

    public String getButton() {
        return button;
    }
    //根据code获取枚举
    public static String getButton(String code,String status) {
        for (CardButtonEnum statusEnum : CardButtonEnum.values()) {
            if (statusEnum.getCode().equals(code)&& statusEnum.getStatus().equals(status)) {
                return statusEnum.getButton();
            }
        }
        return "Buy";
    }
}
