package cn.kdan.cloud.pdf.office.common.enums;

public enum ModelEnum {

    // PC Official Website
    PC_OFFICIAL("PC_OFFICIAL", "官方网站 (PC)"),
    MOBILE_OFFICIAL("MOBILE_OFFICIAL", "官方网站 (Mobile)"),

    // Mac Platform
    MAC_APP_STORE_FREE("MAC_APP_STORE_FREE", "Mac AppStore 免费版"),
    MAC_APP_STORE_PAID("MAC_APP_STORE_PAID", "Mac AppStore 付费版"),
    DMG_DOWNLOAD("DMG_DOWNLOAD", "DMG (官方网站下载)"),

    // Windows Platform
    EXE_MICROSOFT_STORE("EXE_MICROSOFT_STORE", "EXE (微软商店下载)"),
    EXE_OFFICIAL_DOWNLOAD("EXE_OFFICIAL_DOWNLOAD", "EXE (官方网站下载)"),
    UWP_FREE_MICROSOFT_STORE("UWP_FREE_MICROSOFT_STORE", "UWP 免费版 (微软商店下载)"),
    UWP_PAID_MICROSOFT_STORE("UWP_PAID_MICROSOFT_STORE", "UWP 付费版 (微软商店下载)"),

    // iOS Platform
    IOS_APP_STORE_FREE("IOS_APP_STORE_FREE", "iOS AppStore 免费版"),
    IOS_APP_STORE_PAID("IOS_APP_STORE_PAID", "iOS AppStore 付费版"),

    // Android Platform
    ANDROID_GOOGLE_PLAY("ANDROID_GOOGLE_PLAY", "Android (Google Play 商店下载)");

    private final String englishDescription;
    private final String chineseDescription;

    // 构造方法
    ModelEnum(String englishDescription, String chineseDescription) {
        this.englishDescription = englishDescription;
        this.chineseDescription = chineseDescription;
    }

    // 获取英文描述
    public String getEnglishDescription() {
        return englishDescription;
    }

    // 获取中文描述
    public String getChineseDescription() {
        return chineseDescription;
    }

    @Override
    public String toString() {
        return this.englishDescription;  // 默认返回英文描述
    }

    // 根据英文描述查找对应的枚举值
    public static ModelEnum fromEnglishString(String text) {
        for (ModelEnum platform : ModelEnum.values()) {
            if (platform.englishDescription.equalsIgnoreCase(text)) {
                return platform;
            }
        }
        throw new IllegalArgumentException("No constant with text " + text + " found");
    }

    // 根据中文描述查找对应的枚举值
    public static ModelEnum fromChineseString(String text) {
        for (ModelEnum platform : ModelEnum.values()) {
            if (platform.chineseDescription.equalsIgnoreCase(text)) {
                return platform;
            }
        }
        throw new IllegalArgumentException("No constant with text " + text + " found");
    }

}
