package cn.kdan.cloud.pdf.office.common.utils;

import cn.kdan.cloud.pdf.office.common.dto.LicenseCodeResult;
import cn.kdan.cloud.pdf.office.common.dto.PDFReaderResult;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.*;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.List;

@Slf4j
public class PDFReaderUtils {
    public static List<LicenseCodeResult> getLicenseByEmail(String pdfReaderUrl,String email) {
        String url = pdfReaderUrl + "/api/licenseCodes/getAllByEmail";
        // 创建 RestTemplate 实例
        RestTemplate restTemplate = new RestTemplate();
        // 创建请求头
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        // 创建请求体
        String jsonBody = String.format("{\"email\": \"%s\"}", email);
        HttpEntity<String> requestEntity = new HttpEntity<>(jsonBody, headers);
        ResponseEntity<PDFReaderResult<List<LicenseCodeResult>>> response;
        ParameterizedTypeReference<PDFReaderResult<List<LicenseCodeResult>>> typeRef = new ParameterizedTypeReference<PDFReaderResult<List<LicenseCodeResult>>>() {};
        try {
            response = restTemplate.exchange(
                    url,
                    HttpMethod.POST,
                    requestEntity,
                    typeRef
            );
        } catch (Exception e) {
            return new ArrayList<>();
        }
        if (response.getStatusCode() != HttpStatus.OK || org.springframework.util.ObjectUtils.isEmpty(response.getBody())){
            return new ArrayList<>();
        }
        return response.getBody().getData();
    }
    public static void invalid(String pdfReaderUrl,String code) {
        String url = pdfReaderUrl + "/api/licenseCodes/disabledLicense";
        // 创建 RestTemplate 实例
        RestTemplate restTemplate = new RestTemplate();
        // 创建请求头
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        // 创建请求体
        String jsonBody = String.format("{\"license\": \"%s\"}", code);
        HttpEntity<String> requestEntity = new HttpEntity<>(jsonBody, headers);
        ResponseEntity<PDFReaderResult<String>> response = null;
        ParameterizedTypeReference<PDFReaderResult<String>> typeRef = new ParameterizedTypeReference<PDFReaderResult<String>>() {};
        try {
            response = restTemplate.exchange(
                    url,
                    HttpMethod.POST,
                    requestEntity,
                    typeRef
            );
        } catch (Exception e) {
            log.error("失效序列码错误： code：{},msg:{}",code,e.getMessage());
        }
        if (response.getStatusCode() != HttpStatus.OK ){
            log.error("失效序列码错误： code：{}",code);
        }
    }
    public static List<LicenseCodeResult> getLicenseByDevice(String pdfReaderUrl,String deviceSign) {
        String url = pdfReaderUrl + "/api/licenseCodes/getAllByUniqueSN";
        // 创建 RestTemplate 实例
        RestTemplate restTemplate = new RestTemplate();
        // 创建请求头
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        // 创建请求体
        String jsonBody = String.format("{\"unique_sn\": \"%s\"}", deviceSign);
        HttpEntity<String> requestEntity = new HttpEntity<>(jsonBody, headers);
        ResponseEntity<PDFReaderResult<List<LicenseCodeResult>>> response;
        ParameterizedTypeReference<PDFReaderResult<List<LicenseCodeResult>>> typeRef = new ParameterizedTypeReference<PDFReaderResult<List<LicenseCodeResult>>>() {};
        try {
            response = restTemplate.exchange(
                    url,
                    HttpMethod.POST,
                    requestEntity,
                    typeRef
            );
        } catch (Exception e) {
            return new ArrayList<>();
        }
        if (response.getStatusCode() != HttpStatus.OK || org.springframework.util.ObjectUtils.isEmpty(response.getBody())){
            return new ArrayList<>();
        }
        return response.getBody().getData();
    }
}
