package cn.kdan.cloud.pdf.office.email.service.impl;

import cn.kdan.cloud.pdf.office.api.email.bo.EmailSendBO;
import cn.kdan.cloud.pdf.office.api.email.bo.TemplateHtmlBO;
import cn.kdan.cloud.pdf.office.common.constant.RabbitMqConstant;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.JsonUtils;
import cn.kdan.cloud.pdf.office.email.config.MailSenderConfig;
import cn.kdan.cloud.pdf.office.email.entity.EmailLog;
import cn.kdan.cloud.pdf.office.email.service.EmailLogService;
import cn.kdan.cloud.pdf.office.email.service.EmailTemplateService;
import cn.kdan.cloud.pdf.office.email.service.SendEmailHandlerService;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSenderImpl;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Service;

import javax.mail.internet.MimeMessage;
import java.io.File;
import java.net.URL;
import java.util.Date;
import java.util.Map;
import java.util.Objects;

/**
 * @author ComPDFKit-WPH 2023/1/12
 * <p>
 * 邮件发送中心
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class SendEmailHandlerServiceImpl implements SendEmailHandlerService {

    private final RabbitTemplate rabbitTemplate;
    private final MailSenderConfig mailSenderConfig;
    private final EmailLogService emailLogService;
    private final EmailTemplateService emailTemplateService;
    @Override
    public void sendEmail(EmailSendBO emailSendBO){
        //如果等于这些之一member_login member_reset_password member_log_off member_update_email
        if(emailSendBO.getUseEvent().equals("member_login")
                ||emailSendBO.getUseEvent().equals("member_reset_password")
                ||emailSendBO.getUseEvent().equals("member_log_off")
                ||emailSendBO.getUseEvent().equals("member_update_email")) {
            //发送会员相关邮件
            if(!send(emailSendBO,1)){
                if(!send(emailSendBO,2)){
                    send(emailSendBO,3);
                }
            };
        }else {
            rabbitTemplate.convertAndSend(RabbitMqConstant.EMAIL_EXCHANGE,
                    RabbitMqConstant.EMAIL_SEND_ROUTING_KEY,
                    emailSendBO);
        }
    }

    public boolean send(EmailSendBO emailSendBO,int type) {
        log.info("邮件发送监听内容：{}", emailSendBO);
        EmailLog emailLog = new EmailLog();
        String emailTemplateId = null;
        File file = null;
        try {
            if(StringUtils.isEmpty(emailSendBO.getToEmail())){
                log.info("邮箱为空：{}", emailSendBO);
                emailLog.setIsEnable(0);
                return true;
            }
            TemplateHtmlBO template = emailTemplateService.getTemplateByUseEvent(emailSendBO.getUseEvent());
            emailTemplateId = template.getEmailTemplateId();
            JavaMailSenderImpl mailSender;
            if(type == 1){
                mailSender = mailSenderConfig.getDefaultSender();
            }else if(type == 2){
                mailSender = mailSenderConfig.getSecondSender();
            }else {
                mailSender = mailSenderConfig.getThirdSender();
            }
            Map<String, String> sendContent = emailSendBO.getSendContent();
            String templateHtml = template.getTemplateHtml();
            for (String key : sendContent.keySet()) {
                templateHtml = templateHtml.replaceAll(key,sendContent.get(key));
            }
            String templateTitle = template.getTemplateTitle();
            Map<String, String> sendTitleContent = emailSendBO.getSendTitleContent();
            for (String key : sendTitleContent.keySet()) {
                templateTitle = templateTitle.replaceAll(key,sendTitleContent.get(key));
            }

            MimeMessage mimeMessage = mailSender.createMimeMessage();
            MimeMessageHelper helper = new MimeMessageHelper(mimeMessage, true);
            helper.setSubject(templateTitle);
            helper.setText(templateHtml, true);
            helper.setTo(emailSendBO.getToEmail());
            if(mailSender.getHost().contains("amazonaws")){
                helper.setFrom("no-reply@pdfreaderpro.com","PDF Reader Pro");
            }else{
                helper.setFrom(ObjectUtils.isEmpty(emailSendBO.getFromEmail())
                        ? Objects.requireNonNull(mailSender.getUsername())
                        : emailSendBO.getFromEmail(), "PDF Reader Pro");
            }
            if(ObjectUtils.isNotEmpty(emailSendBO.getFileUrl())&&ObjectUtils.isNotEmpty(emailSendBO.getFileName())){
                file = new File(emailSendBO.getFileName());
                FileUtils.copyURLToFile(new URL(emailSendBO.getFileUrl()), file);
                helper.addAttachment(Objects.requireNonNull(emailSendBO.getFileName()), file);
            }
            mailSender.send(mimeMessage);
            emailLog.setIsEnable(1);
            return true;
        } catch (Exception e) {
            log.error("消息消费失败:" + e.getMessage(), e);
            emailLog.setIsEnable(0);
            return false;
        } finally {
            emailLog.setSendTo(emailSendBO.getToEmail());
            emailLog.setSendFrom(emailSendBO.getFromEmail());
            emailLog.setSendTime(new Date());
            emailLog.setSendTemplate(emailTemplateId);
            emailLog.setSendContent(JsonUtils.getJsonString(emailSendBO.getSendContent()) + ";" + JsonUtils.getJsonString(emailSendBO.getSendTitleContent()));
            emailLogService.getBaseMapper().insert(emailLog);
            if (ObjectUtils.isNotEmpty(file) && file.exists()){
                file.delete();
            }
        }
    }



}
