package cn.kdan.cloud.pdf.office.generate.controller;

import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.generate.dto.GenerateCodeDTO;
import cn.kdan.cloud.pdf.office.generate.dto.TableInfoDTO;
import cn.kdan.cloud.pdf.office.generate.entity.Schemata;
import cn.kdan.cloud.pdf.office.generate.entity.TableInfo;
import cn.kdan.cloud.pdf.office.generate.service.GenerateCodeService;
import cn.kdan.cloud.pdf.office.generate.service.TableInfoService;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.toolkit.SimpleQuery;
import lombok.RequiredArgsConstructor;
import org.apache.commons.io.IOUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.List;

/**
 * @author ZhouQiang 2022/7/24
 */
@RestController
@RequestMapping("/generate")
@RequiredArgsConstructor
public class GeneratorCodeController {

    private final GenerateCodeService generateCodeService;
    private final TableInfoService tableInfoService;

    @GetMapping("/schemas")
    public ResultMap<List<String>> listSchemas() {
        return ResultMap.success(SimpleQuery.list(Wrappers.<Schemata>lambdaQuery().notIn(Schemata::getSchemaName, "mysql", "information_schema", "performance_schema"), Schemata::getSchemaName));
    }

    @GetMapping("/tables")
    public ResultMap<IPage<TableInfo>> listTableInfo(TableInfoDTO query) {
        return ResultMap.success(tableInfoService.listTableInfo(query));
    }

    @PostMapping
    public void generate(@Validated @RequestBody GenerateCodeDTO generateCod, HttpServletResponse response) throws IOException {
        byte[] data = generateCodeService.generatorCode(generateCod);

        response.reset();
        response.setHeader("Content-Disposition", "attachment; filename=" + URLEncoder.encode("code.zip", "UTF-8"));
        response.addHeader("Content-Length", "" + data.length);
        response.setContentType("application/octet-stream; charset=UTF-8");

        IOUtils.write(data, response.getOutputStream());
    }
}
