package cn.kdan.cloud.pdf.office.payment.mapper;

import cn.kdan.cloud.pdf.office.api.payment.dto.EquityVerificationDTO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.payment.entity.Subscription;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.apache.ibatis.annotations.Param;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @author ComPDFKit-WPH 2023-01-28
 */
public interface SubscriptionsMapper extends BaseMapper<Subscription> {

    /**
     * 通过用户id和产品id 获取最新的订阅信息
     *
     * @param userId   userId
     * @param platform platform
     * @return Subscriptions
     */
    SubscriptionsVO selectNewestSubscription(@Param("userId") String userId,
                                             @Param("platform") Integer platform);


    SubscriptionsVO selectBySubscriptionId(@Param("subscriptionId") String subscriptionId);

    SubscriptionsVO selectNewestSubscriptionByProductId(@Param("userId") String userId,
                                                        @Param("productId") String productId);

    /**
     * 查询最新一条实际支付的订阅信息（排除赠送的订阅）
     * @param userId 用户id
     * @param productId 产品id
     * @return SubscriptionsVO
     */
    SubscriptionsVO selectLatestSubscriptionForActualPayment(@Param("userId") String userId,
                                                        @Param("productId") String productId);

    /**
     * 查询订阅信息 通过userId
     *
     * @param userId userId
     * @return SubscriptionsVO
     */
    List<SubscriptionsVO> selectByUserId(@Param("userId")String userId,
                                         @Param("appId")String appId);

    List<EquityVerificationDTO> getSubscriptionsByEndDate(@Param("minusDays") LocalDateTime minusDays,
                                                          @Param("now") LocalDateTime now,
                                                          @Param("payment") Integer payment);

    int updateStatusByUserIdAndProductId(@Param("status") Integer status, @Param("userId") String userId, @Param("productId") String productId);
    int countAiTrail(@Param("userId") String userId);
    List<Subscription> countAdvancedSubscriptions(@Param("userId") String userId);
    int countSubscriptionTrail(@Param("userId") String userId);
    int countSubscription(@Param("userId") String userId);
    /**
     * 查询订阅信息 通过userId(有没有ai以外的订阅)ps：权益转移或者赠送的权益是没有订单的
     *
     * @param userId userId
     * @return SubscriptionsVO
     */
    int countSubscriptionInfo(@Param("userId") String userId);
}
