package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.email.bo.EmailSendBO;
import cn.kdan.cloud.pdf.office.api.email.feign.EmailApi;
import cn.kdan.cloud.pdf.office.api.payment.appstore.AppTransaction;
import cn.kdan.cloud.pdf.office.api.payment.constant.AppStoreAPIConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateOrderManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.EquityVerificationDTO;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.enums.EmailCodeTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.enums.payment.SubscriptionTypeEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.payment.client.AppStoreClient;
import cn.kdan.cloud.pdf.office.payment.entity.RightBindLog;
import cn.kdan.cloud.pdf.office.payment.enums.AppleIAPEnum;
import cn.kdan.cloud.pdf.office.payment.properties.AppStoreProperties;
import cn.kdan.cloud.pdf.office.payment.service.AppStoreService;
import cn.kdan.cloud.pdf.office.payment.service.OrderService;
import cn.kdan.cloud.pdf.office.payment.service.RightBindLogService;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import cn.kdan.cloud.pdf.office.payment.utils.TemplatesUtil;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.util.StreamUtils;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author ComPDFKit-WPH 2023/6/29
 *
 * AppStoreServiceImpl
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class AppStoreServiceImpl implements AppStoreService {

    private final AppStoreClient appStoreClient;

    private final OrderService orderService;
    private final RightBindLogService rightBindLogService;

    private final SubscriptionsService subscriptionsService;


    private final ProductApi productApi;

    private final UserApi userApi;


    private final EmailApi emailApi;
    private final AppStoreProperties properties;
    private static final RestTemplate restTemplate = new RestTemplate();
    private final RedissonClient redissonClient;

    @Value("${htmlToPdfUrl:http://139.196.160.101:3060/api/get-invoice}")
    private String htmlToPdfUrl;



    @Transactional
    @Override
    public UserInfoVO appStoreEquityVerification(EquityVerificationDTO equityVerificationDTO) {
        // 没有票据直接返回用户
        if (StringUtils.isEmpty(equityVerificationDTO.getTransactionId())) {
            UserInfoVO result = userApi.getInfoById(equityVerificationDTO.getUserId()).getResult();
            result.setDigestPassword(null);
            return result;
        }
        log.info("appStore 权益校验,用户id：{},请求体：{}",equityVerificationDTO.getUserId(),equityVerificationDTO);
        AppTransaction appTransaction = this.appStoreReceiptVerify(equityVerificationDTO.getTransactionId(),equityVerificationDTO.getApplePayProductId(),equityVerificationDTO.getAppBundleId());
        // 获取票据订单信息
        // 如果订单带折扣，但是app传的产品是不带试用的高级版年订阅，则按订单来算
        if(StringUtils.isNotEmpty(appTransaction.getOfferDiscountType())&&appTransaction.getOfferDiscountType().equals("FREE_TRIAL")
                &&equityVerificationDTO.getProductCode().equals("advanced-annual-subscription")){
            equityVerificationDTO.setProductCode("advanced-annual-subscription-trail");
        }
        String transactionId = appTransaction.getTransactionId();
        String originalTransactionId = appTransaction.getOriginalTransactionId();
        BigDecimal price = new BigDecimal(appTransaction.getPrice()).divide(new BigDecimal(1000), 2, RoundingMode.HALF_UP);
        String currency = appTransaction.getCurrency();
        String lockKey = transactionId + "-RedissonLock";
        // 获取 RLock 对象
        RLock rLock = redissonClient.getLock(lockKey);
        try {
            // 上锁
            boolean resultLock = rLock.tryLock(30, 10, TimeUnit.SECONDS);
            // 业务逻辑开始
            if (resultLock) {
                log.info("appStore 权益校验,交易id:{}",transactionId);
                OrdersVO orderByThirdOrderNo = orderService.getOrderByThirdOrderNo(transactionId);
                if (null != orderByThirdOrderNo) {
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_TICKET_HAS_BEEN_CREATED);
                }
                // 没有绑定直接建立新的绑定关系
                log.info("新购买订单，票据{}，用户绑定{}", equityVerificationDTO.getTransactionId(), equityVerificationDTO.getUserId());
                // 新增新用户的订阅
                CreateSubscriptionDTO createSubscription = new CreateSubscriptionDTO();
                ProductVO product = productApi.getProductByCode(equityVerificationDTO.getProductCode()).getResult();
                String subscriptionId = CommonUtils.generateId();
                String orderId = cn.kdan.cloud.pdf.office.payment.utils.CommonUtils.generateRightsId((long) ((Math.random() * 9999 + 1) * 10000));
                String appId = product.getAppId();
                createSubscription.setId(subscriptionId);
                createSubscription.setUserId(equityVerificationDTO.getUserId());
                // 使用购买的订单号
                createSubscription.setOrderId(orderId);
                createSubscription.setAppId(appId);
                createSubscription.setPayment(PaymentMethodEnum.APPLE_PAY.getValue());
                createSubscription.setProductId(product.getId());
                createSubscription.setThirdSubscriptionId(originalTransactionId);
                UserVO userVO  = userApi.getById(equityVerificationDTO.getUserId()).getResult();
                createSubscription.setEmail(userVO.getEmail());
                createSubscription.setBuyerType(4);//2官网购买 3后台赠送 4appstore
                createSubscription.setSubscriptionType(SubscriptionTypeEnum.MEMBER.value());//如果是0则是试用
                createSubscription.setPaymentModel(product.getPaymentModel());
                createSubscription.setPrice(price);
                createSubscription.setPayTime(price.compareTo(BigDecimal.ZERO) == 0 ? 0 : 1);
                subscriptionsService.createSubscription(createSubscription);
                String readeNo = cn.kdan.cloud.pdf.office.payment.utils.CommonUtils.generateRightsId((long) ((Math.random() * 9999 + 1) * 10000));
                // 更新订单
                CreateOrderManualDTO orderManualDTO = CreateOrderManualDTO.builder()
                        .thirdTradeNo(originalTransactionId)
                        .thirdOrderNo(transactionId)
                        .price(price)
                        .reducedPrice(price)
                        .subscriptionId(subscriptionId)
                        .userId(equityVerificationDTO.getUserId())
                        .email(userVO.getEmail())
                        .id(orderId)
                        .paymentMethod(PaymentMethodEnum.APPLE_PAY)
                        .appId(appId)
                        .productId(product.getId())
                        .paymentModel(product.getPaymentModel())
                        .subscriptionType(1)
                        .payNumber(1)
                        .tradeNo(readeNo)
                        .invoiceNo(MyDateUtils.getTimeStamp() + (int) ((Math.random() * 9 + 1) * 1000))
                        .result(appTransaction.toString())
                        .model(StringUtils.isNotEmpty(equityVerificationDTO.getModel())?"":equityVerificationDTO.getModel())
                        .currency(currency)
                        .build();
                orderService.createOrderManual(orderManualDTO);
                UserInfoVO result = userApi.getInfoById(equityVerificationDTO.getUserId()).getResult();
                result.setDigestPassword(null);
                sendBuyEmail(product, userVO,orderManualDTO);
                return result;
            }
        } catch (InterruptedException e) {
            log.info("上锁失败");
            log.error("上锁失败".concat(e.getMessage()), e);
        } finally {
            if (rLock.isLocked() && rLock.isHeldByCurrentThread()) {
                rLock.unlock();
            }
        }
        return null;
    }

    @Override
    public Boolean bindAppleTransaction(List<String> appTransactionIds, String userId, String appleBundleId,Boolean isHaveSubscription) {
        // 没有票据直接返回用户
        log.info("appStore 权益绑定,用户id：{},苹果交易id集合：{}", userId, appTransactionIds);
        boolean isBound = false;
        for (String appTransactionId : appTransactionIds) {
            //先判断交易id是否绑定过，没绑定过才绑定
            OrdersVO ordersVO = orderService.getOrderByThirdTradeNo(appTransactionId);
            if (ObjectUtils.isEmpty(ordersVO)) {
                String lockKey = appTransactionId + "-RedissonLock";
                // 获取 RLock 对象
                RLock rLock = redissonClient.getLock(lockKey);
                try {
                    AppTransaction appTransaction = this.appStoreReceiptVerify(appTransactionId, null, appleBundleId);
                    if(StringUtils.isEmpty(AppleIAPEnum.getByAppleIAP(appTransaction.getProductId()))){
                        log.info("苹果产品id没有对应的会员产品id:{}",appTransaction.getProductId());
                        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TRANSACTION_ID_VALIDATION_FAILED);
                    }
                    String transactionId = appTransaction.getTransactionId();
                    String originalTransactionId = appTransaction.getOriginalTransactionId();
                    BigDecimal price = new BigDecimal(appTransaction.getPrice()).divide(new BigDecimal(1000), 2, RoundingMode.HALF_UP);
                    String currency = appTransaction.getCurrency();
                    // 上锁
                    boolean resultLock = rLock.tryLock(30, 10, TimeUnit.SECONDS);
                    // 业务逻辑开始
                    if (resultLock) {
                        log.info("appStore 权益校验,交易id:{}", transactionId);
                        OrdersVO orderByThirdOrderNo = orderService.getOrderByThirdOrderNo(transactionId);
                        if (null != orderByThirdOrderNo) {
                            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_TICKET_HAS_BEEN_CREATED);
                        }
                        // 没有绑定直接建立新的绑定关系
                        log.info("新购买订单，票据{}，用户绑定{}",appTransactionId, userId);
                        // 新增新用户的订阅
                        CreateSubscriptionDTO createSubscription = new CreateSubscriptionDTO();
                        ProductVO product = productApi.getProductByCode(AppleIAPEnum.getByAppleIAP(appTransaction.getProductId())).getResult();
                        //如果要绑定的会员是订阅产品，并且该账号已有订阅，则不做绑定
                        if(product.getPlatform().equals(2)&&isHaveSubscription){
                            log.info("要绑定的会员是订阅产品，并且该账号已有订阅，则不做绑定产品id:{}",product.getCode());
                            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TRANSACTION_ID_VALIDATION_FAILED);
                        }
                        String subscriptionId = CommonUtils.generateId();
                        String orderId = cn.kdan.cloud.pdf.office.payment.utils.CommonUtils.generateRightsId((long) ((Math.random() * 9999 + 1) * 10000));
                        String appId = product.getAppId();
                        createSubscription.setId(subscriptionId);
                        createSubscription.setUserId(userId);
                        // 使用购买的订单号
                        createSubscription.setOrderId(orderId);
                        createSubscription.setAppId(appId);
                        createSubscription.setPayment(PaymentMethodEnum.APPLE_PAY.getValue());
                        createSubscription.setProductId(product.getId());
                        createSubscription.setThirdSubscriptionId(originalTransactionId);
                        UserVO userVO = userApi.getById(userId).getResult();
                        createSubscription.setEmail(userVO.getEmail());
                        createSubscription.setBuyerType(2);//2官网购买 3后台赠送 4appstore
                        createSubscription.setSubscriptionType(SubscriptionTypeEnum.MEMBER.value());//如果是0则是试用
                        createSubscription.setPaymentModel(product.getPaymentModel());
                        createSubscription.setPrice(price);
                        createSubscription.setPayTime(0);
                        subscriptionsService.createSubscription(createSubscription);
                        String readeNo = cn.kdan.cloud.pdf.office.payment.utils.CommonUtils.generateRightsId((long) ((Math.random() * 9999 + 1) * 10000));
                        // 更新订单
                        CreateOrderManualDTO orderManualDTO = CreateOrderManualDTO.builder()
                                .thirdTradeNo(originalTransactionId)
                                .thirdOrderNo(transactionId)
                                .price(price)
                                .reducedPrice(price)
                                .subscriptionId(subscriptionId)
                                .userId(userId)
                                .email(userVO.getEmail())
                                .id(orderId)
                                .paymentMethod(PaymentMethodEnum.APPLE_PAY)
                                .appId(appId)
                                .productId(product.getId())
                                .paymentModel(product.getPaymentModel())
                                .subscriptionType(1)
                                .payNumber(1)
                                .tradeNo(readeNo)
                                .invoiceNo(MyDateUtils.getTimeStamp() + (int) ((Math.random() * 9 + 1) * 1000))
                                .result(appTransaction.toString())
                                .model("旧权益绑定")
                                .currency(currency)
                                .type(3)
                                .build();
                        orderService.createOrderManual(orderManualDTO);
                        isBound = true;
                        RightBindLog bindLog = new RightBindLog();
                        bindLog.setAppleTransaction(appTransaction.toString());
                        bindLog.setAppleTransactionId(originalTransactionId);
                        bindLog.setCdkey(orderId);
                        bindLog.setLicenseProductCode(product.getCode());
                        bindLog.setProductId(product.getId());
                        // 苹果交易
                        bindLog.setType("2");
                        bindLog.setUserId(userId);
                        bindLog.setId(CommonUtils.generateId());
                        rightBindLogService.save(bindLog);
                    }
                } catch (Exception e) {
                    log.info("自动绑定苹果权益失败：msg：{},苹果交易id：{},用户id：{}",e.getMessage(),appTransactionId,userId);
                } finally {
                    if (rLock.isLocked() && rLock.isHeldByCurrentThread()) {
                        rLock.unlock();
                    }
                }
            }
        }
        return isBound;
    }


    private void sendBuyEmail(ProductVO product, UserVO userVO, CreateOrderManualDTO orderManualDTO) {
        if (product.getCode().contains("upgrade")) {
            EmailSendBO bo = new EmailSendBO();
            bo.setToEmail(userVO.getEmail());
            //设置事件
            bo.setUseEvent(EmailCodeTypeEnum.MEMBER_UPGRADE_SUCCESS.value());
            //设置title
            Map<String, String> titleMap = new HashMap<>();
            //设置内容
            Map<String,String> contentMap = new HashMap<>();
            contentMap.put("@plan@", product.getProductName());
            Date date = new Date();
            // 创建 SimpleDateFormat 对象，指定日期格式
            SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
            // 格式化当前日期
            String formattedDate = sdf.format(date);
            contentMap.put("@date@",formattedDate);
            String price;
//            BigDecimal renewPrice;
//            if(orderManualDTO.getCurrency().equals("USD")){
//                price = ObjectUtils.isNotEmpty(product.getDisplayPrice())? product.getDisplayPrice().toString() : product.getPrice().toString();
//                renewPrice = product.getPrice();
//            }else{
//                price = ObjectUtils.isNotEmpty(product.getCnyDisplayPrice())? product.getCnyDisplayPrice().toString() : product.getCnyPrice().toString();
//                renewPrice = product.getCnyPrice();
//            }
            contentMap.put("@payPrice@", orderManualDTO.getCurrency()+orderManualDTO.getPrice());
            contentMap.put("@renewPrice@",orderManualDTO.getCurrency()+orderManualDTO.getPrice());

            bo.setSendTitleContent(titleMap);
            //设置内容
            bo.setSendContent(contentMap);
            String file = sendInvoice(userVO.getEmail(),formattedDate,orderManualDTO.getTradeNo(),orderManualDTO.getInvoiceNo(),orderManualDTO.getPrice(),orderManualDTO.getPrice(),orderManualDTO.getCurrency());
            bo.setFileUrl(file);
            bo.setFileName("invoice.pdf");
            emailApi.sendEmail(bo);
        }else{
            EmailSendBO bo = new EmailSendBO();
            bo.setToEmail(userVO.getEmail());
            //设置事件
            bo.setUseEvent(EmailCodeTypeEnum.MEMBER_BUY_SUCCESS.value());
            //设置title
            Map<String,String> titleMap = new HashMap<>();
            //设置内容
            Map<String,String> contentMap = new HashMap<>();
            contentMap.put("@plan@", product.getProductName());
            Date date = new Date();
            // 创建 SimpleDateFormat 对象，指定日期格式
            SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
            // 格式化当前日期
            String formattedDate = sdf.format(date);
            contentMap.put("@date@",formattedDate);
            String price;
            BigDecimal renewPrice;
            if(orderManualDTO.getCurrency().equals("USD")){
                price = ObjectUtils.isNotEmpty(product.getDisplayPrice())? product.getDisplayPrice().toString() : product.getPrice().toString();
                renewPrice = product.getPrice();
            }else{
                price = ObjectUtils.isNotEmpty(product.getCnyDisplayPrice())? product.getCnyDisplayPrice().toString() : product.getCnyPrice().toString();
                renewPrice = product.getCnyPrice();
            }
            contentMap.put("@payPrice@", orderManualDTO.getCurrency()+orderManualDTO.getPrice());
            bo.setSendTitleContent(titleMap);
            //设置内容
            bo.setSendContent(contentMap);
            String file = sendInvoice(userVO.getEmail(),formattedDate,orderManualDTO.getTradeNo(),orderManualDTO.getInvoiceNo(),orderManualDTO.getPrice(),orderManualDTO.getPrice(),orderManualDTO.getCurrency());
            bo.setFileUrl(file);
            bo.setFileName("invoice.pdf");
            emailApi.sendEmail(bo);
        }
    }

    //生成发票
    private String sendInvoice(String email, String purchasedDate,String tradeNo,String invoiceNumber,BigDecimal unitPrice,BigDecimal amount,String currency) {
        String invoiceHtml = null;
        try {
            invoiceHtml = new String(StreamUtils.copyToByteArray(this.getClass().getClassLoader().getResourceAsStream("templates/invoice_member.html")), StandardCharsets.UTF_8);
        } catch (IOException e) {
            log.error(e.getMessage());
        }
        Map<String, Object> map = new HashMap<>();
        map.put("Name", email);
        map.put("email", email);
        map.put("purchasedDate", purchasedDate);
        map.put("orderNumber",tradeNo);
        map.put("invoiceDate", purchasedDate);
        map.put("invoiceNumber", invoiceNumber);
        map.put("quantity", 1);
        map.put("unitPrice", unitPrice);
        BigDecimal discountAmount = unitPrice.subtract(amount);
        map.put("discount", discountAmount);
        map.put("amount", amount);
        map.put("total", amount);
        log.info("发票html内容填充map:{}", map);
        // 生成替换内容后的发票
        String htmlStr = TemplatesUtil.replaceStringUsingFreeMarker(invoiceHtml, map);
        if (!currency.equals("USD")) {
            htmlStr = htmlStr.replace("USD", currency);
        }
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        // 封装表单数据
        MultiValueMap<String, String> paramMap = new LinkedMultiValueMap<>();
        paramMap.add("html", htmlStr);
        HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<>(paramMap, headers);
        // 发送POST请求,html转pdf
        Map postForObjectMap = restTemplate.postForObject(htmlToPdfUrl, request, Map.class);
        String invoiceFileUrl = postForObjectMap.get("url").toString();
        log.info("发票文件地址：{}", invoiceFileUrl);
        return invoiceFileUrl;
    }

    /**
     * 票据验证
     *
     * @return List<InApp>
     */
    private AppTransaction appStoreReceiptVerify(String transactionId, String applePayProductId,String appBundleId) {
        // 验证票据信息
        AppTransaction transaction = appStoreClient.fetchFirstAppTransaction(transactionId,appBundleId);
        // 如果主环境交易记录为null，尝试获取沙箱环境交易记录
        if (Objects.isNull(transaction) && "false".equals(properties.getIsSandbox())) {
            transaction = appStoreClient.fetchSandboxFirstAppTransaction(transactionId, appBundleId);
        }

        // 如果最终transaction仍为null，抛出验证失败异常
        if (Objects.isNull(transaction)) {
            log.error("验证苹果交易失败，transactionId：{}", transactionId);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TRANSACTION_ID_VALIDATION_FAILED);
        }
        log.info("苹果票据信息：{}", transaction.toString());
        // 检查支付状态等
        Date expiresDateMs = transaction.getExpiresDate();
        String inAppOwnershipType = transaction.getInAppOwnershipType();
        if (!StringUtils.isEmpty(applePayProductId)&&!transaction.getProductId().equals(applePayProductId)) {
            log.error("验证苹果交易失败,产品id不一致,AppTransaction:{} ,applePayProductId:{}", transaction,applePayProductId);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TRANSACTION_ID_VALIDATION_FAILED);
        }
        // 校验是否是支付成功的订单
        if (!AppStoreAPIConstant.INAPPOWNERSHIPTYPE_PURCHASED.equals(inAppOwnershipType)) {
            log.error("验证苹果交易失败，inAppOwnershipType：{},AppTransaction:{}", inAppOwnershipType, transaction);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TRANSACTION_ID_VALIDATION_FAILED);
        }
        // 校验订阅到期时间是否在当前时间之后
        if (expiresDateMs != null && expiresDateMs.before(new Date())) {
            log.error("验证苹果交易失败,票据已过期，AppTransaction:{} ", transaction);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TRANSACTION_ID_VALIDATION_FAILED);
        }
        return transaction;
    }

}
