package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.email.bo.EmailSendBO;
import cn.kdan.cloud.pdf.office.api.email.feign.EmailApi;
import cn.kdan.cloud.pdf.office.api.payment.constant.OrderConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateOrderManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.GooglePayDTO;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.enums.EmailCodeTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.enums.ModelEnum;
import cn.kdan.cloud.pdf.office.common.enums.payment.SubscriptionTypeEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.payment.client.GooglePayClient;
import cn.kdan.cloud.pdf.office.payment.properties.GooglePayConfig;
import cn.kdan.cloud.pdf.office.payment.service.GooglePayService;
import cn.kdan.cloud.pdf.office.payment.service.OrderService;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import cn.kdan.cloud.pdf.office.payment.utils.TemplatesUtil;
import cn.kdan.cloud.pdf.office.payment.webhook.google.SubscriptionPurchaseLineItemMS;
import cn.kdan.cloud.pdf.office.payment.webhook.google.SubscriptionPurchaseV3;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.util.StreamUtils;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author ComPDFKit-WPH 2023/6/29
 * <p>
 * AppStoreServiceImpl
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class GooglePayServiceImpl implements GooglePayService {

    private final GooglePayClient googlePayClient;

    private final OrderService orderService;

    private final SubscriptionsService subscriptionsService;

    private final WebhookServiceImpl webhookService;

    private final ProductApi productApi;

    private final UserApi userApi;

    private final EmailApi emailApi;
    private static final RestTemplate restTemplate = new RestTemplate();
    private final GooglePayConfig googlePayConfig;
    @Value("${htmlToPdfUrl:http://139.196.160.101:3060/api/get-invoice}")
    private String htmlToPdfUrl;


    @Transactional
    @Override
    public UserInfoVO googleEquityVerification(GooglePayDTO googlePayDTO) {
        // 没有票据直接返回用户
        if (StringUtils.isEmpty(googlePayDTO.getPurchaseToken())) {
            UserInfoVO result = userApi.getInfoById(googlePayDTO.getUserId()).getResult();
            result.setDigestPassword(null);
            return result;
        }
        log.info("谷歌权益校验,用户id：{}", googlePayDTO.getUserId());
        SubscriptionPurchaseV3 subscriptionPurchaseV2 = this.verify(googlePayDTO.getPurchaseToken(),googlePayDTO.getPackageName());
        // 获取票据订单信息
        if(ObjectUtils.isEmpty(subscriptionPurchaseV2.getExternalAccountIdentifiers())){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_GOOGLE_STORE_TRANSACTION_ID_VALIDATION_FAILED);
        }
        List<SubscriptionPurchaseLineItemMS> list = subscriptionPurchaseV2.getLineItems();
        ProductVO product = productApi.getProductByCode(googlePayDTO.getProductCode()).getResult();
        String productId = product.getId();
        SubscriptionPurchaseLineItemMS activeItem = list.get(0);
        //如果有首期优惠的话设置首期优惠
        BigDecimal price = product.getPrice();
        if(ObjectUtils.isNotEmpty(product.getDisplayPrice())){
            price = product.getDisplayPrice();
        }
        if(product.getCode().contains("blackFive-trail")){
            price = BigDecimal.valueOf(0.00);
        }
        OrdersVO orderByThirdOrderNo = orderService.getOrderByThirdOrderNo(subscriptionPurchaseV2.getLatestOrderId());
        // 判断是否已经处理过
        if (null != orderByThirdOrderNo) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_TICKET_HAS_BEEN_CREATED);
        }
        // 没有绑定直接建立新的绑定关系
        log.info("新购买订单，purcharseToken{}，packageName{},绑定用户{}", googlePayDTO.getPurchaseToken(), googlePayDTO.getPackageName(), googlePayDTO.getUserId());
        // 新增新用户的订阅

        CreateSubscriptionDTO createSubscription = new CreateSubscriptionDTO();
        String subscriptionId = CommonUtils.generateId();
        String orderId = cn.kdan.cloud.pdf.office.payment.utils.CommonUtils.generateRightsId((long) ((Math.random() * 9999 + 1) * 10000));
        String appId = product.getAppId().toString();
        createSubscription.setId(subscriptionId);
        createSubscription.setUserId(googlePayDTO.getUserId());
        // 使用购买的订单号
        createSubscription.setOrderId(orderId);
        createSubscription.setAppId(appId);
        createSubscription.setPayment(PaymentMethodEnum.GOOGLE_PAY.getValue());
        createSubscription.setProductId(productId);
        createSubscription.setThirdSubscriptionId(subscriptionPurchaseV2.getExternalAccountIdentifiers().getObfuscatedExternalAccountId());
        UserVO userVO  = userApi.getById(googlePayDTO.getUserId()).getResult();
        createSubscription.setEmail(userVO.getEmail());
        createSubscription.setBuyerType(5);//2官网购买 3后台赠送 4appstore
        createSubscription.setSubscriptionType(SubscriptionTypeEnum.MEMBER.value());//如果是0则是试用
        createSubscription.setPaymentModel(product.getPaymentModel());
        createSubscription.setPrice(price);
        createSubscription.setPayTime(0);
        subscriptionsService.createSubscription(createSubscription);
        String readeNo = cn.kdan.cloud.pdf.office.payment.utils.CommonUtils.generateRightsId((long) ((Math.random() * 9999 + 1) * 10000));
        CreateOrderManualDTO orderManualDTO = CreateOrderManualDTO.builder()
                .thirdTradeNo(subscriptionPurchaseV2.getExternalAccountIdentifiers().getObfuscatedExternalAccountId())
                .thirdOrderNo(subscriptionPurchaseV2.getLatestOrderId())
                .price(product.getPrice())
                .reducedPrice(price)
                .subscriptionId(subscriptionId)
                .userId(googlePayDTO.getUserId())
                .id(orderId)
                .email(userVO.getEmail())
                .paymentMethod(PaymentMethodEnum.GOOGLE_PAY)
                .appId(appId)
                .productId(productId)
                .model(ModelEnum.ANDROID_GOOGLE_PLAY.getEnglishDescription())
                //安卓全是自动续费
                .paymentModel(product.getPaymentModel())
                .subscriptionType(1)
                .payNumber(1)
                .tradeNo(readeNo)
                .invoiceNo(MyDateUtils.getTimeStamp() + (int) ((Math.random() * 9 + 1) * 1000))
                .result(googlePayDTO.getPurchaseToken())
                .tradeNo(readeNo)
                .currency("USD")
                .build();
        orderService.createOrderManual(orderManualDTO);
        sendBuyEmail(product, userVO,orderManualDTO);
        UserInfoVO result = userApi.getInfoById(googlePayDTO.getUserId()).getResult();
        result.setDigestPassword(null);
        return result;
    }
    private void sendBuyEmail(ProductVO product, UserVO userVO,CreateOrderManualDTO orderManualDTO) {
        if(product.getCode().contains("upgrade")){
            EmailSendBO bo = new EmailSendBO();
            bo.setToEmail(userVO.getEmail());
            //设置事件
            bo.setUseEvent(EmailCodeTypeEnum.MEMBER_UPGRADE_SUCCESS.value());
            //设置title
            Map<String,String> titleMap = new HashMap<>();
            //设置内容
            Map<String,String> contentMap = new HashMap<>();
            contentMap.put("@plan@", product.getProductName());
            Date date = new Date();
            // 创建 SimpleDateFormat 对象，指定日期格式
            SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
            // 格式化当前日期
            String formattedDate = sdf.format(date);
            contentMap.put("@date@",formattedDate);
            String price;
            BigDecimal renewPrice;
            if(orderManualDTO.getCurrency().equals("USD")){
                price = ObjectUtils.isNotEmpty(product.getDisplayPrice())? product.getDisplayPrice().toString() : product.getPrice().toString();
                renewPrice = product.getPrice();
            }else{
                price = ObjectUtils.isNotEmpty(product.getCnyDisplayPrice())? product.getCnyDisplayPrice().toString() : product.getCnyPrice().toString();
                renewPrice = product.getCnyPrice();
            }
            contentMap.put("@payPrice@", orderManualDTO.getCurrency()+price);
            contentMap.put("@renewPrice@",orderManualDTO.getCurrency()+price);

            bo.setSendTitleContent(titleMap);
            //设置内容
            bo.setSendContent(contentMap);
            String file = sendInvoice(userVO.getEmail(),formattedDate,orderManualDTO.getTradeNo(),orderManualDTO.getInvoiceNo(),renewPrice,orderManualDTO.getPrice(),orderManualDTO.getCurrency());
            bo.setFileUrl(file);
            bo.setFileName("invoice.pdf");
            emailApi.sendEmail(bo);
        }else{
            EmailSendBO bo = new EmailSendBO();
            bo.setToEmail(userVO.getEmail());
            //设置事件
            bo.setUseEvent(EmailCodeTypeEnum.MEMBER_BUY_SUCCESS.value());
            //设置title
            Map<String,String> titleMap = new HashMap<>();
            //设置内容
            Map<String,String> contentMap = new HashMap<>();
            contentMap.put("@plan@", product.getProductName());
            Date date = new Date();
            // 创建 SimpleDateFormat 对象，指定日期格式
            SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
            // 格式化当前日期
            String formattedDate = sdf.format(date);
            contentMap.put("@date@",formattedDate);
            String price;
            BigDecimal renewPrice;
            if(orderManualDTO.getCurrency().equals("USD")){
                price = ObjectUtils.isNotEmpty(product.getDisplayPrice())? product.getDisplayPrice().toString() : product.getPrice().toString();
                renewPrice = product.getPrice();
            }else{
                price = ObjectUtils.isNotEmpty(product.getCnyDisplayPrice())? product.getCnyDisplayPrice().toString() : product.getCnyPrice().toString();
                renewPrice = product.getCnyPrice();
            }
            contentMap.put("@payPrice@", orderManualDTO.getCurrency()+price);
            bo.setSendTitleContent(titleMap);
            //设置内容
            bo.setSendContent(contentMap);
            String file = sendInvoice(userVO.getEmail(),formattedDate,orderManualDTO.getTradeNo(),orderManualDTO.getInvoiceNo(),renewPrice,orderManualDTO.getPrice(),orderManualDTO.getCurrency());
            bo.setFileUrl(file);
            bo.setFileName("invoice.pdf");
            emailApi.sendEmail(bo);
        }
    }

    //生成发票
    private String sendInvoice(String email, String purchasedDate,String tradeNo,String invoiceNumber,BigDecimal unitPrice,BigDecimal amount,String currency) {
        String invoiceHtml = null;
        try {
            invoiceHtml = new String(StreamUtils.copyToByteArray(this.getClass().getClassLoader().getResourceAsStream("templates/invoice_member.html")), StandardCharsets.UTF_8);
        } catch (IOException e) {
            log.error(e.getMessage());
        }
        Map<String, Object> map = new HashMap<>();
        map.put("Name", email);
        map.put("email", email);
        map.put("purchasedDate", purchasedDate);
        map.put("orderNumber",tradeNo);
        map.put("invoiceDate", purchasedDate);
        map.put("invoiceNumber", invoiceNumber);
        map.put("quantity", 1);
        map.put("unitPrice", unitPrice);
        BigDecimal discountAmount = unitPrice.subtract(amount);
        map.put("discount", discountAmount);
        map.put("amount", amount);
        map.put("total", amount);
        log.info("发票html内容填充map:{}", map);
        // 生成替换内容后的发票
        String htmlStr = TemplatesUtil.replaceStringUsingFreeMarker(invoiceHtml, map);
        if (!currency.equals("USD")) {
            htmlStr = htmlStr.replace("USD", currency);
        }
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        // 封装表单数据
        MultiValueMap<String, String> paramMap = new LinkedMultiValueMap<>();
        paramMap.add("html", htmlStr);
        HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<>(paramMap, headers);
        // 发送POST请求,html转pdf
        Map postForObjectMap = restTemplate.postForObject(htmlToPdfUrl, request, Map.class);
        String invoiceFileUrl = postForObjectMap.get("url").toString();
        log.info("发票文件地址：{}", invoiceFileUrl);
        return invoiceFileUrl;
    }

    @Override
    public SubscriptionPurchaseV3 verify(String purchaseToken, String packageName) {
        SubscriptionPurchaseV3 subscriptionGoogleOrderV2 = googlePayClient.getSubscription(packageName, purchaseToken);
        //判断订单状态
        if (!googlePayClient.checkSubscriptionSuccess(subscriptionGoogleOrderV2)) {
            log.error("谷歌订单验证状态异常：{}", subscriptionGoogleOrderV2);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_GOOGLE_STORE_TRANSACTION_ID_VALIDATION_FAILED);
        }
        return subscriptionGoogleOrderV2;
    }

    @Override
    public void subscriptionPaymentSucceeded(Boolean isDiscount, String thirdTradeNo, String thirdOrderId, Integer payTime, SubscriptionPurchaseV3 subscriptionGoogleOrderV2) {
        log.info("谷歌 订阅的续费订单已成功 谷歌订单id：{},订阅id{}", thirdOrderId,thirdTradeNo);
        OrdersVO ordersVO = orderService.getOrderByThirdOrderNo(thirdOrderId);
        //如果订单处理过直接返回
        if (ObjectUtils.isNotEmpty(ordersVO) && OrderConstant.COMPLETED.equals(ordersVO.getStatus())) {
            log.info("谷歌 订阅的续费订单已处理过");
            return;
        }
        List<OrdersVO> orderByTradeNo = orderService.getOrderByTradeNo(thirdTradeNo);
        if (CollectionUtils.isEmpty(orderByTradeNo)){
            return;
        }
        OrdersVO ordersVONew = orderByTradeNo.get(0);
        ProductVO productVO = orderService.getProduct(ordersVONew.getProductId());
        BigDecimal price = productVO.getPrice();
        if(orderByTradeNo.size()==1&&productVO.getCode().equals("advanced-annual-subscription-blackFive-trail")){
            price = productVO.getDisplayPrice();
        }
        webhookService.handleSubsequentAutomaticDeduction(price, thirdTradeNo, thirdOrderId,ordersVONew, 1, subscriptionGoogleOrderV2.toString(),productVO.getPrice() );
    }

    @Override
    public void subscriptionCancel(String thirdSubscriptionId) {
        log.info("订阅取消续订，订阅id：{}", thirdSubscriptionId);
        // 根据第三放订阅id查询对应的用户id和产品id
        webhookService.handleMemberUnsubscribeFollowUpActions(thirdSubscriptionId);
    }

    @Override
    public void subscriptionRevoked(String thirdOrderId) {
        webhookService.subscriptionRevoked(thirdOrderId);
    }


}
