package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.kdan.cloud.pdf.office.api.account.dto.UpdateUserForOrderDTO;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserSubscriptionInfoApi;
import cn.kdan.cloud.pdf.office.api.account.vo.UserSubscriptionInfo;
import cn.kdan.cloud.pdf.office.api.payment.constant.SubscriptionConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.*;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.vo.*;
import cn.kdan.cloud.pdf.office.api.product.enums.PlatformEnum;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.enums.account.AccountTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionPayTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PayTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.payment.SubscriptionTypeEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.payment.client.PaddleClient;
import cn.kdan.cloud.pdf.office.payment.client.PaypalClient;
import cn.kdan.cloud.pdf.office.payment.entity.RightBindLog;
import cn.kdan.cloud.pdf.office.payment.entity.Subscription;
import cn.kdan.cloud.pdf.office.payment.error.ErrorMessage;
import cn.kdan.cloud.pdf.office.payment.mapper.SubscriptionsMapper;
import cn.kdan.cloud.pdf.office.payment.service.OrderService;
import cn.kdan.cloud.pdf.office.payment.service.RightBindLogService;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import cn.kdan.cloud.pdf.office.api.account.utils.SubscriptionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

/**
 * @author ComPDFKit-WPH 2023-01-28
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class SubscriptionsServiceImpl extends ServiceImpl<SubscriptionsMapper, Subscription> implements SubscriptionsService {

    private final ProductApi productApi;

    private final PaddleClient paddleClient;

    private final PaypalClient paypalClient;

    private final UserApi userApi;

    private final OrderService ordersService;
    private final RightBindLogService rightBindLogService;

    private final RabbitTemplate rabbitTemplate;
    private final UserSubscriptionInfoApi userSubscriptionInfoApi;
    @Override
    public IPage<Subscription> page(Subscription query) {
        return this.page(new Page<>(query.getPage(), query.getSize()), Wrappers.query(query));
    }

    @Override
    public boolean judgeTrail(String userId){
        //没有买过试用，没有买过其他订阅，
        return this.baseMapper.countSubscriptionTrail(userId) == 0 && this.baseMapper.countSubscription(userId) == 0&& this.baseMapper.countSubscriptionInfo(userId) == 0;
    }


    @Override
    public String createSubscription(CreateSubscriptionDTO subscriptionInfo) {
        log.info("createSubscription Info ：{}", subscriptionInfo);
        /* 获取产品的订阅周期时长 */
        // 当前时间
        LocalDateTime now = LocalDateTime.now();
        // 初始时间
        LocalDateTime originalTime ;
        if(subscriptionInfo.getAppId().equals("16")){
            originalTime = ObjectUtils.isNotEmpty(subscriptionInfo.getStartDate()) ? subscriptionInfo.getStartDate()
                    : getOriginalTime(subscriptionInfo.getUserId(),subscriptionInfo.getProductId(), now);
        }else{
            originalTime = ObjectUtils.isNotEmpty(subscriptionInfo.getStartDate()) ? subscriptionInfo.getStartDate()
                    : getProOriginalTime(subscriptionInfo.getUserId(),subscriptionInfo.getProductId(), now);
        }
        // 通过计算获取结束时间
        //todo 怎么判断是不是第一次支付
        Integer payTime = ObjectUtils.isNotEmpty(subscriptionInfo.getPayTime())?subscriptionInfo.getPayTime():1;
        LocalDateTime endTime = ObjectUtils.isNotEmpty(subscriptionInfo.getEndDate()) ? subscriptionInfo.getEndDate()
                : getEndTime(subscriptionInfo.getProductId(), originalTime, payTime);

        // 产生一条最新的订阅信息
        Subscription subscription = new Subscription();
        if(StringUtils.isEmpty(subscriptionInfo.getId())){
            subscriptionInfo.setId(CommonUtils.generateId());
        }
        subscription.setId(subscriptionInfo.getId());
        subscription.setProductId(subscriptionInfo.getProductId());
        subscription.setStatus(SubscriptionConstant.actived);
        subscription.setUserId(subscriptionInfo.getUserId());
        subscription.setStartDate(MyDateUtils.localDateTimeToDate(originalTime));
        subscription.setEndDate(MyDateUtils.localDateTimeToDate(endTime));
        if(!StringUtils.isEmpty(subscriptionInfo.getThirdSubscriptionId())){
            subscription.setThirdSubscriptionId(subscriptionInfo.getThirdSubscriptionId());
        }
        if(!StringUtils.isEmpty(subscriptionInfo.getOrderId())){
            subscription.setOrderId(subscriptionInfo.getOrderId());
        }
        subscription.setAppId(subscriptionInfo.getAppId());
        subscription.setPayment(subscriptionInfo.getPayment());
        subscription.setEmail(subscriptionInfo.getEmail());
        subscription.setSubscriptionType(subscriptionInfo.getSubscriptionType());
        if (null != subscriptionInfo.getIsLock()) {
            subscription.setIsLock(subscriptionInfo.getIsLock());
        }
        //2官网购买 3后台赠送 4appstore
        if (ObjectUtils.isNotEmpty(subscriptionInfo.getBuyerType())){
            subscription.setBuyerType(subscriptionInfo.getBuyerType());
        }else {
            subscription.setBuyerType(2);
        }
        if (ObjectUtils.isNotEmpty(subscriptionInfo.getPrice())){
            subscription.setPrice(subscriptionInfo.getPrice());
        }
        if (ObjectUtils.isNotEmpty(subscriptionInfo.getPaymentModel())){
            subscription.setPaymentModel(subscriptionInfo.getPaymentModel());
        }

        this.baseMapper.insert(subscription);
        //  更新 userSubscription
        updateUserSubscriptionStatus(subscription);
        // 发送消息
        SubscriptionUtil.subscriptionExpiredSendNews(rabbitTemplate, subscription.getId(), endTime);
        return subscription.getId();
    }


    private void updateUserSubscriptionStatus(Subscription subscription) {
        // 根据产品id查询产品信息
        ProductVO productVO = productApi.getProduct(subscription.getProductId()).getResult();
        // 获取产品平台
        Integer platform = productVO.getPlatform();
        UpdateUserForOrderDTO successDTO = new UpdateUserForOrderDTO();
        successDTO.setUserId(subscription.getUserId());
        if(productVO.getPlatform()==1&&productVO.getAppId().equals("1")){
            successDTO.setPayTpe(PayTypeEnum.NO_AUTO.value());
        }else{
            successDTO.setPayTpe(PayTypeEnum.AUTO.value());
        }

        successDTO.setPlatform(platform);
        if(subscription.getSubscriptionType().equals(SubscriptionTypeEnum.TRAIL.value())){
            successDTO.setUserStatus(PDFOfficeUserSubscriptionStatusEnum.TRIAL.value());
        }else{
            successDTO.setUserStatus(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value());
        }
        successDTO.setEndDate(subscription.getEndDate());
        successDTO.setAppId(subscription.getAppId());
        successDTO.setProductId(subscription.getProductId());
        if(productVO.getPlatform().equals(3)&&productVO.getAppId().equals(CommonConstant.PDF_READER_PRO_APP_ID)){
            successDTO.setPoint(Integer.valueOf(productVO.getPoints()));
        }
        if(subscription.getPrice().compareTo(BigDecimal.valueOf(0))==0){
            successDTO.setAccountType(AccountTypeEnum.FREE_ACCOUNT);
        }
        userApi.updateUserForOrder(successDTO);
    }

    @Override
    public SubscriptionsVO getSubscriptions(String userId, PlatformEnum platform) {
        return this.baseMapper.selectNewestSubscription(userId, platform.getValue());
    }

    @Override
    public SubscriptionsVO getSubscriptionsByProductId(String userId, String productId) {
        return this.baseMapper.selectNewestSubscriptionByProductId(userId, productId);
    }

    @Override
    public SubscriptionsVO getLatestSubscriptionForActualPayment(String userId, String productId) {
        return this.baseMapper.selectLatestSubscriptionForActualPayment(userId, productId);
    }

    @Override
    public LocalDateTime getNextPayTime(String userId, String productId) {
        Subscription latestSubscription = this.baseMapper.selectList(new LambdaQueryWrapper<Subscription>().eq(Subscription::getUserId,userId)
                .eq(Subscription::getProductId,productId)
                .ne(Subscription::getPayment,PaymentMethodEnum.NO_NEED_TO_PAY.getValue())
                .orderByDesc(Subscription::getCreatedAt)
                .last("LIMIT 1")).stream().findFirst().orElse(null);
        return getEndTime(productId, latestSubscription.getCreatedAt(), 1);
    }

    @Override
    public String createSubscriptionManual(CreateSubscriptionManualDTO subscriptionManual) {
        CreateSubscriptionDTO createSubscriptionDTO = new CreateSubscriptionDTO();
        BeanUtils.copyProperties(subscriptionManual, createSubscriptionDTO);
        return this.createSubscription(createSubscriptionDTO);
    }

    @Override
    public void chargebackSubscription(ChargebackOrderDTO chargebackOrderDTO) {
        CancelSubscriptionDTO cancelSubscriptionDTO = new CancelSubscriptionDTO();
        cancelSubscriptionDTO.setSubscriptionId(chargebackOrderDTO.getSubscriptionId());
        this.chargebackSubscription(cancelSubscriptionDTO);
    }

    @Override
    public void chargebackSubscription(CancelSubscriptionDTO cancelSubscriptionDTO) {
        log.info("取消订阅开始：{}",cancelSubscriptionDTO.toString());
        Subscription subscription = this.baseMapper.selectById(cancelSubscriptionDTO.getSubscriptionId());
        String thirdSubscriptionId = subscription.getThirdSubscriptionId();
        switch (PaymentMethodEnum.fromValue(subscription.getPayment())){
            case PAYPAL:
                // 取消续订 webhook实际处理状态字段
                paypalClient.cancelSubscription(thirdSubscriptionId,"Not satisfied with the service");
                break;
            case PADDLE:
                // 更新paddle的订阅信息
                paddleClient.cancelSubscription(thirdSubscriptionId);
                break;
            case ALIPAY: break;
            case WXPAY: break;
            case NO_NEED_TO_PAY: // TODO 是否需要根据别的字段取消续订？看后续后台规划
                break;
            default:
                throw new BackendRuntimeException(ErrorMessage.PAYMENT_METHOD_INVALID);
        }
        UserSubscriptionInfo userSubscriptionInfo = userSubscriptionInfoApi.getByUserIdAndProductId(subscription.getUserId(), subscription.getProductId()).getResult();
        log.info("userSubscriptionInfo id: {} ",userSubscriptionInfo.getId());
        UserSubscriptionInfo info = new UserSubscriptionInfo();
        info.setUserId(userSubscriptionInfo.getUserId());
        info.setProductId(userSubscriptionInfo.getProductId());
        info.setPayType(PDFOfficeUserSubscriptionPayTypeEnum.NON_AUTO_SUBSCRIPTION.value());
        info.setId(userSubscriptionInfo.getId());
        userSubscriptionInfoApi.update(info);
    }
    @Override
    public void revokeSubscription(String userId,Integer platform) {
        log.info("取消订阅开始：{}", userId,platform);

        UserSubscriptionInfo userSubscriptionInfo = userSubscriptionInfoApi.getByUserIdAndPlatform(userId, platform).getResult();

        if (userSubscriptionInfo == null) {
            return; // 早期返回，避免空指针异常
        }

        log.info("userSubscriptionInfo id: {}", userSubscriptionInfo.getId());

        // 根据平台类型进行处理
        switch (userSubscriptionInfo.getPlatform()) {
            case 1:
                handlePermanentSubscription(userSubscriptionInfo);
                break;
            case 2:
                handleSubscription(userSubscriptionInfo);
                break;
            case 3:
                handleAiSubscription(userSubscriptionInfo);
                break;
            default:
                log.error("未知平台类型: {}", userSubscriptionInfo.getPlatform());
        }
    }

    private void handlePermanentSubscription(UserSubscriptionInfo userSubscriptionInfo) {
        log.info("永久版订阅，删除用户订阅信息，UserSubscriptionInfoId: {}", userSubscriptionInfo.getId());
        userSubscriptionInfoApi.delete(userSubscriptionInfo.getId());
    }

    private void handleSubscription(UserSubscriptionInfo userSubscriptionInfo) {
        // 获取更新后的结束时间
        LocalDateTime updatedEndDate = getMinEndTime(userSubscriptionInfo.getProductId(), MyDateUtils.dateToLocalDateTime(userSubscriptionInfo.getEndDate()));
        LocalDateTime currentDate = LocalDateTime.now();
        // 更新订阅状态和结束时间
        updateSubscription(userSubscriptionInfo, updatedEndDate, currentDate);
        // 如果是高级版年订阅，则判断是否有0元领取的一年ai要回收
        ProductVO productVO = productApi.getProduct(userSubscriptionInfo.getProductId()).getResult();
        if (!productVO.getCycle().equals(4) || !productVO.getCode().contains("advanced-annual-subscription")) {
            return;
        }
        ProductVO aiProduct = productApi.getProductByCode("ai-subscription-year-trail").getResult();
        UserSubscriptionInfo us = userSubscriptionInfoApi.getByUserIdAndProductId(userSubscriptionInfo.getUserId(), aiProduct.getId()).getResult();
        if (us != null) {
            handleAiSubscription(us);
        }
    }

    private void handleAiSubscription(UserSubscriptionInfo userSubscriptionInfo) {
        // 获取更新后的结束时间
        LocalDateTime updatedEndDate = getMinEndTime(userSubscriptionInfo.getProductId(), MyDateUtils.dateToLocalDateTime(userSubscriptionInfo.getEndDate()));
        LocalDateTime currentDate = LocalDateTime.now();

        // 更新订阅状态和结束时间
        updateSubscription(userSubscriptionInfo, updatedEndDate, currentDate);

        // 扣除点数
        Integer point = Integer.valueOf(productApi.getProduct(userSubscriptionInfo.getProductId()).getResult().getPoints());
        Integer remainingPoints = Math.max(userSubscriptionInfo.getPoint() - point, 0);
        userSubscriptionInfo.setPoint(remainingPoints);
    }

    private void updateSubscription(UserSubscriptionInfo userSubscriptionInfo, LocalDateTime updatedEndDate, LocalDateTime currentDate) {
        // 如果更新后的结束时间早于当前时间，设置为已过期
        if (updatedEndDate.isBefore(currentDate)) {
            userSubscriptionInfo.setStatus(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_EXPIRED.value());
            userSubscriptionInfo.setPayType(0);
        }
        // 更新结束时间
        userSubscriptionInfo.setEndDate(MyDateUtils.localDateTimeToDate(updatedEndDate));
        userSubscriptionInfo.setPayType(PayTypeEnum.NO_AUTO.value());
        // 更新数据库
        userSubscriptionInfoApi.update(userSubscriptionInfo);
    }

    @Override
    public Map<String,SubscriptionSchemeVO> getSubscriptionSchemeInfo(String userId,String appId) {
        List<SubscriptionsVO> subscriptions = this.baseMapper.selectByUserId(userId,appId);
        Map<String, List<SubscriptionsVO>> collect = subscriptions.stream().collect(Collectors.groupingBy(SubscriptionsVO::getProductId));
        Map<String,SubscriptionSchemeVO> map = new HashMap<>();
        for (String productId : collect.keySet()) {
            SubscriptionSchemeVO subscriptionSchemeVO = new SubscriptionSchemeVO();
            subscriptionSchemeVO.setSubscriptionInfo(getSubscriptionInfo(userId,productId,appId));
            map.put(productId,subscriptionSchemeVO);
        }
        Map<String, List<OrderInfoVO>> orderCollect =
                getOrderInfo(userId, appId).stream().collect(Collectors.groupingBy(OrderInfoVO::getProductId));
        for (String productId : orderCollect.keySet()) {
            SubscriptionSchemeVO subscriptionSchemeVO = map.get(productId);
            if (ObjectUtils.isNotEmpty(subscriptionSchemeVO)){
                subscriptionSchemeVO.setOrdersInfo(orderCollect.get(productId));
            }else {
                subscriptionSchemeVO = new SubscriptionSchemeVO();
                subscriptionSchemeVO.setOrdersInfo(orderCollect.get(productId));
                map.put(productId,subscriptionSchemeVO);
            }

        }

//        subscriptionSchemeVO.setOrdersInfo();
        return map;
    }

    @Override
    public SubscriptionsVO getSubscriptionBySubscriptionId(String subscriptionId) {
        return this.baseMapper.selectBySubscriptionId(subscriptionId);
    }

    /**
     * 修改订阅状态
     *
     * @param subscriptionId id
     * @param subscriptionStatus SubscriptionConstant
     */
    @Override
    public void updateSubscriptionStatus(String subscriptionId, Integer subscriptionStatus){
        // 修改订阅状态
        Subscription subscriptions = new Subscription();
        subscriptions.setId(subscriptionId);
        subscriptions.setStatus(subscriptionStatus);
        this.baseMapper.updateById(subscriptions);
    }

    @Override
    public List<SubscriptionsVO> getSubscriptionsByUserIdAndProductId(String userId, String productId) {
        return this.baseMapper.selectList(new LambdaQueryWrapper<Subscription>().eq(Subscription::getUserId,userId)
                .eq(Subscription::getProductId,productId).ne(Subscription::getPayment,PaymentMethodEnum.NO_NEED_TO_PAY.getValue())).stream().map(subscription -> {
            SubscriptionsVO subscriptionsVO = new SubscriptionsVO();
            BeanUtils.copyProperties(subscription,subscriptionsVO);
            return subscriptionsVO;
        }).collect(Collectors.toList());
    }

    //获取endDate在当前时间以前的所有活跃的订阅
    @Override
    public List<SubscriptionsVO> getSubscriptionsByEndDate() {
        return this.baseMapper.selectList(new LambdaQueryWrapper<Subscription>().eq(Subscription::getStatus,3)
                .le(Subscription::getEndDate,new Date())).stream().map(subscription -> {
            SubscriptionsVO subscriptionsVO = new SubscriptionsVO();
            BeanUtils.copyProperties(subscription,subscriptionsVO);
            return subscriptionsVO;
        }).collect(Collectors.toList());
    }

    @Override
    public SubscriptionsVO getSubscriptionByThirdSubscriptionId(String thirdSubscriptionId) {
        SubscriptionsVO subscriptionsVO = new SubscriptionsVO();
        Subscription subscription = this.baseMapper.selectOne(new LambdaQueryWrapper<Subscription>().eq(Subscription::getThirdSubscriptionId,thirdSubscriptionId)
                .orderByDesc(Subscription::getCreatedAt).last("Limit 1"));
        if(subscription == null){
            return subscriptionsVO;
        }
        BeanUtils.copyProperties(subscription
                ,subscriptionsVO);
        return subscriptionsVO;
    }

    @Override
    public List<SubscriptionsVO> getSubscriptionsByPayment(String userId, Integer payment) {
        return this.baseMapper.selectList(new LambdaQueryWrapper<Subscription>()
                .eq(Subscription::getUserId, userId).eq(Subscription::getPayment, payment))
                .stream().map(subscription -> {
                    SubscriptionsVO subscriptionsVO = new SubscriptionsVO();
                    BeanUtils.copyProperties(subscription, subscriptionsVO);
                    return subscriptionsVO;
                }).collect(Collectors.toList());
    }

    @Override
    public List<EquityVerificationDTO> getSubscriptionsByEndDate(LocalDateTime minusDays, PaymentMethodEnum paymentMethodEnum) {
        return this.baseMapper.getSubscriptionsByEndDate(minusDays,LocalDateTime.now(),paymentMethodEnum.getValue());
    }

    @Override
    public int updateStatusByUserIdAndProductId(Integer status, String userId, String productId) {
        return this.baseMapper.updateStatusByUserIdAndProductId(status, userId, productId);
    }


    @Override
    public Boolean judgeAiDiscount(String userId) {
        //免费用户购买了全平台高级版年订阅
        AtomicBoolean haveAdvancedSubscriptions = new AtomicBoolean(false);
        //除标准版年订阅升级以外的高级年订阅购买
        List<Subscription> list= this.baseMapper.countAdvancedSubscriptions(userId);
        list.forEach(item->{
            //有一条不为0的的高级订阅
            if(!ObjectUtils.isEmpty(item.getPrice())&&item.getPrice().compareTo(BigDecimal.valueOf(0))!=0){
                haveAdvancedSubscriptions.set(true);
            }
        });
        if(haveAdvancedSubscriptions.get() &&this.baseMapper.countAiTrail(userId)==0){
            return true;
        }
        //转移了权益的用户，如果在权益转移表有用户的记录(转移了序列码或者app端绑定了权益)
        return rightBindLogService.count(Wrappers.<RightBindLog>lambdaQuery().eq(RightBindLog::getUserId, userId)) > 0 && this.baseMapper.countAiTrail(userId) == 0;
    }

    @Override
    public Boolean judgeAiTrail(String userId) {
        return this.baseMapper.countAiTrail(userId)>0;
    }

    /**
     * 获取订阅结束时间
     *
     * @param productId productId
     * @param originalTime     开始时间
     * @param payTimes 第几次付款 如果有多个不同的续订周期时
     * @return 结束时间
     */
    private LocalDateTime getEndTime(String productId, LocalDateTime originalTime, Integer payTimes) {
        ProductVO product = productApi.getProduct(productId).getResult();
        //7.days:0,1.years:20,1.years:40
        if(product.getForPeriod().contains(",")){
            List<String> dateAndPriceList = Arrays.asList(product.getForPeriod().split(","));
            int times =  dateAndPriceList.size() > payTimes ? payTimes : dateAndPriceList.size()-1;
            String forPeriod = dateAndPriceList.get(times);
            return MyDateUtils.TimeAddition(originalTime, forPeriod);
        }
        String forPeriod = product.getForPeriod();
        return MyDateUtils.TimeAddition(originalTime, forPeriod);
    }
    private LocalDateTime getMinEndTime(String productId, LocalDateTime originalTime) {
        ProductVO product = productApi.getProduct(productId).getResult();
        //7.days:0,1.years:20,1.years:40
        if(product.getForPeriod().contains(",")){
            List<String> dateAndPriceList = Arrays.asList(product.getForPeriod().split(","));
            int times =  dateAndPriceList.size()-1;
            String forPeriod = dateAndPriceList.get(times);
            return MyDateUtils.timeMinus(originalTime, forPeriod);
        }
        String forPeriod = product.getForPeriod();
        return MyDateUtils.timeMinus(originalTime, forPeriod);
    }

    /**
     * 获取初始开始时间
     *
     * @param userId userId
     * @param now              now
     * @return LocalDateTime
     */
    private LocalDateTime getOriginalTime(String userId,String productId, LocalDateTime now) {
        // 通过用户id和产品id获取最新订阅信息
        SubscriptionsVO newestSubscriptions = getSubscriptionsByProductId(userId, productId);
        LocalDateTime originalTime;
        // 没有订阅数据
        if (ObjectUtils.isEmpty(newestSubscriptions)) {
            originalTime = now;
        } else {
            LocalDateTime localDateTime = MyDateUtils.dateToLocalDateTime(newestSubscriptions.getEndDate());
            // 当前时间大于最后结束时间
            if (now.isAfter(localDateTime)) {
                originalTime = now;
            } else {
                originalTime = localDateTime;
            }
        }
        return originalTime;
    }
    private LocalDateTime getProOriginalTime(String userId,String productId, LocalDateTime now) {
        // 通过用户id和产品id获取最新订阅信息
        UserSubscriptionInfo userSubscriptionInfo = userSubscriptionInfoApi.getByUserIdAndPlatform(userId, productApi.getProduct(productId).getResult().getPlatform()).getResult();
        LocalDateTime originalTime;
        // 没有订阅数据
        if (ObjectUtils.isEmpty(userSubscriptionInfo)) {
            originalTime = now;
        } else {
            Date endDate = ObjectUtils.isNotEmpty(userSubscriptionInfo.getEndDate()) ? userSubscriptionInfo.getEndDate() : new Date();
            LocalDateTime localDateTime = MyDateUtils.dateToLocalDateTime(endDate);
            // 当前时间大于最后结束时间
            if (now.isAfter(localDateTime)) {
                originalTime = now;
            } else {
                originalTime = localDateTime;
            }
        }
        return originalTime;
    }

    private SubscriptionInfoVO getSubscriptionInfo(String userId,String productId,String appId) {
        ProductVO productVO = productApi.getProduct(productId).getResult();
        boolean isYearSubscription = productVO.getForPeriod().contains("years");
        UserSubscriptionInfo userSubscriptionInfo = userSubscriptionInfoApi.getByUserIdAndProductId(userId, productId).getResult();
        List<SubscriptionsVO> subscriptions = this.baseMapper.selectByUserId(userId,appId);
        if (CollectionUtils.isEmpty(subscriptions)) {
            // 没有有效订阅
            return null;
        }else {
            BigDecimal price = null;
            PaymentMethodEnum paymentMethodEnum = null;
            for (SubscriptionsVO subscription : subscriptions) {
                if (!PaymentMethodEnum.NO_NEED_TO_PAY.getValue().equals(subscription.getPayment())){
                    price = ordersService.getOrderById(subscription.getOrderId()).getPrice();
                    paymentMethodEnum = PaymentMethodEnum.fromValue(subscription.getPayment());
                    break;
                }else {
                    paymentMethodEnum = PaymentMethodEnum.fromValue(subscription.getPayment());
                    price = subscription.getPrice();
                }
            }
            return SubscriptionInfoVO.builder()
                    .price(price)
                    .endDate(MyDateUtils.dateToLocalDateTime(userSubscriptionInfo.getEndDate()))
                    .paymentMethod(paymentMethodEnum.getValue())
                    .productName(productVO.getProductName())
                    .subscriptionPlan(isYearSubscription ? 2 : 1)
                    .productId(productId)
                    .subscriptionStatus(userSubscriptionInfo.getStatus())
                    .autoSubscriptionStatus(userSubscriptionInfo.getPayType())
                    .build();
        }
    }

    private List<OrderInfoVO> getOrderInfo(String userId,String appId) {
        List<OrdersVO> ordersVOS = ordersService.getOrderByUserId(userId,appId);
        return ordersVOS.stream().map(ordersVO -> {
            ProductVO productVO = productApi.getProduct(ordersVO.getProductId()).getResult();
            int isYearSubscription = productVO.getForPeriod().contains("years") ? 2 : 1;
            if (ordersVO.getPayment().equals(PaymentMethodEnum.NO_NEED_TO_PAY.getValue())) {
                SubscriptionsVO subscription = this.getSubscriptionBySubscriptionId(ordersVO.getSubscriptionId());
                Date endDate = subscription.getEndDate();
                Date startDate = subscription.getStartDate();
                isYearSubscription = MyDateUtils.daysBetween(startDate, endDate);
//                isYearSubscription = endDate.compareTo(startDate);
            }
            return OrderInfoVO.builder()
                    .date(ordersVO.getCreatedAt())
                    .price(ordersVO.getPrice())
                    .status(ordersVO.getStatus())
                    .subscriptionPlan(isYearSubscription)
                    .paymentMethod(ordersVO.getPayment())
                    .productId(ordersVO.getProductId())
                    .build();
        }).collect(Collectors.toList());
    }
}
