package cn.kdan.cloud.pdf.office.payment.task;

import cn.kdan.cloud.pdf.office.api.account.dto.UpdateUserForOrderDTO;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.utils.SubscriptionUtil;
import cn.kdan.cloud.pdf.office.api.payment.constant.SubscriptionConstant;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PayTypeEnum;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;

import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.stream.Subscription;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import rx.subscriptions.Subscriptions;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

@Slf4j
@Component
@RequiredArgsConstructor
public class SubscriptionTask {
    private final SubscriptionsService subscriptionsService;
    private final UserApi userApi;

    @Scheduled(cron = "0 0/10 * * * ?")
    public void handleSubscriptionsExpire() {
        List<SubscriptionsVO> list = subscriptionsService.getSubscriptionsByEndDate();
        //已处理过过期的订阅id
        list.forEach(subscriptionsVO-> {
            subscriptionsService.updateSubscriptionStatus(subscriptionsVO.getId(), SubscriptionConstant.expired);
            LocalDateTime now = LocalDateTime.now();
            UserInfoVO userInfoVO = userApi.getInfoById(subscriptionsVO.getUserId()).getResult();
            if (ObjectUtils.isEmpty(userInfoVO)) {
                log.info("用户不存在 处理成功 :{}", subscriptionsVO.getId());
                return;
            }
            List<UserSubscriptionInfoVO> subscriptionInfoList = userInfoVO.getSubscriptionInfoList();
            UserSubscriptionInfoVO userSubscriptionInfoVO = SubscriptionUtil.getUserSubscriptionInfoByProductId(subscriptionInfoList, subscriptionsVO.getProductId());
            if (!ObjectUtils.isEmpty(userSubscriptionInfoVO)&&!ObjectUtils.isEmpty(userSubscriptionInfoVO.getEndDate())){
                LocalDateTime endDate = MyDateUtils.dateToLocalDateTime(userSubscriptionInfoVO.getEndDate());
                LocalDateTime oldEndDate = MyDateUtils.dateToLocalDateTime(subscriptionsVO.getEndDate());
                // 修改订阅表订阅状态 过期
                log.info("订阅过期 当前时间 :{} 到期时间 :{}", now, oldEndDate);
                subscriptionsService.updateSubscriptionStatus(subscriptionsVO.getId(), SubscriptionConstant.expired);
                if (now.isAfter(endDate)) {
                    log.info("订阅过期 修改用户 :{}", subscriptionsVO.getUserId());
                    // 修改用户状态
                    // 获取当前用户的订阅状态
                    Integer userStatus = Integer.MIN_VALUE;
                    if (!ObjectUtils.isEmpty(userSubscriptionInfoVO)) {
                        userStatus = userSubscriptionInfoVO.getStatus();
                        PDFOfficeUserSubscriptionStatusEnum userStatusEnum = PDFOfficeUserSubscriptionStatusEnum.getEnumByValue(userSubscriptionInfoVO.getStatus());
                        if (userStatusEnum == PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS) {
                            userStatus = PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_EXPIRED.value();
                        } else if (userStatusEnum == PDFOfficeUserSubscriptionStatusEnum.TRIAL) {
                            userStatus = PDFOfficeUserSubscriptionStatusEnum.TRIAL_EXPIRED.value();
                        }
                    }
                    // 修改用户状态
                    UpdateUserForOrderDTO paySuccessDTO = new UpdateUserForOrderDTO();
                    paySuccessDTO.setUserId(subscriptionsVO.getUserId());
                    paySuccessDTO.setPlatform(userSubscriptionInfoVO.getPlatform());
                    paySuccessDTO.setUserStatus(userStatus);
                    paySuccessDTO.setPayTpe(PayTypeEnum.NO_AUTO.value());
                    paySuccessDTO.setAppId(subscriptionsVO.getAppId());
                    userApi.updateUserForOrder(paySuccessDTO);
                }
                log.info("订阅过期 处理成功 :{}", subscriptionsVO.getId());
            }
        });
    }

}
