package cn.kdan.cloud.pdf.office.payment.utils;


import com.auth0.jwt.JWT;
import com.auth0.jwt.algorithms.Algorithm;

import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.interfaces.ECPrivateKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.time.Instant;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

public class BearerTokenAuthenticator {
    private static final String APP_STORE_CONNECT_AUDIENCE = "appstoreconnect-v1";
    private static final String BUNDLE_ID_KEY = "bid";
    private final ECPrivateKey signingKey;
    private final String keyId;
    private final String issuerId;
    private final String bundleId;

    public BearerTokenAuthenticator(String signingKey, String keyId, String issuerId, String bundleId) {
        try {
            if(signingKey.contains("-----BEGIN PRIVATE KEY-----")){
                signingKey = signingKey.replace("-----BEGIN PRIVATE KEY-----", "")
                        .replaceAll("\\R+", "")
                        .replace("-----END PRIVATE KEY-----", "");
            }
            byte[] derEncodedSigningKey = Base64.getDecoder().decode(signingKey);
            KeyFactory keyFactory = KeyFactory.getInstance("EC");
            PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(derEncodedSigningKey);
            this.signingKey = (ECPrivateKey) keyFactory.generatePrivate(keySpec);
        } catch (NoSuchAlgorithmException e) {
            throw new AssertionError(e);
        } catch (InvalidKeySpecException e) {
            throw new RuntimeException(e);
        }

        this.keyId = keyId;
        this.issuerId = issuerId;
        this.bundleId = bundleId;
    }

    public String generateToken() {
        // Create a mutable map for the payload
        Map<String, Object> payload = new HashMap<>();
        payload.put(BUNDLE_ID_KEY, this.bundleId);

        // Generate the token
        return JWT.create()
                .withAudience(APP_STORE_CONNECT_AUDIENCE)
                .withExpiresAt(Instant.now().plusSeconds(300)) // 5 minutes in seconds
                .withIssuer(this.issuerId)
                .withKeyId(this.keyId)
                .withPayload(payload)
                .sign(Algorithm.ECDSA256(this.signingKey));
    }
}
