package cn.kdan.cloud.pdf.office.payment.utils;


import org.springframework.util.CollectionUtils;

import javax.servlet.http.HttpServletRequest;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.security.MessageDigest;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author tangxiangan
 */
public class CommonUtils {

    public final static String DEFAULT_DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";

    private static final String NAME_REGEX="^[a-zA-Z\\d_][a-zA-Z\\d-_]*$";

    private static final int DEFAULT_ID_LENGTH = 9;


    /**
     * 校验名称
     */
    public static Boolean validateName(String name){
        Pattern pattern= Pattern.compile(NAME_REGEX);
        Matcher matcher = pattern.matcher(name);
        return matcher.matches();
    }

    /**
     * 功能描述：获取当前日期时间格式化字符串，默认yyyy-MM-dd HH:mm:ss格式
     */
    public static String getCurrentDateTime() {
        return getDateTime(DEFAULT_DATE_FORMAT);
    }

    /**
     * 功能描述：获取当前日期时间格式化字符串
     *
     * @param dateFormat 日期时间格式
     */
    public static String getDateTime(String dateFormat) {
        return new SimpleDateFormat(dateFormat).format(Calendar.getInstance()
                .getTime());
    }

    public static String getDateTimeAfterTenYears() {
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(System.currentTimeMillis());
        int currenYear = calendar.get(Calendar.YEAR);
        calendar.set(Calendar.YEAR, currenYear + 100);
        return new SimpleDateFormat(DEFAULT_DATE_FORMAT).format(calendar.getTime());
    }

    public static String getDateTimeBeforeTenYears() {
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(System.currentTimeMillis());
        int currenYear = calendar.get(Calendar.YEAR);
        calendar.set(Calendar.YEAR, currenYear - 1);
        return new SimpleDateFormat(DEFAULT_DATE_FORMAT).format(calendar.getTime());
    }


    /**
     * 功能描述：获取指定日期时间格式化字符串
     *
     * @param dateFormat 日期时间格式
     */
    public static String getDateTime(String dateFormat, Date date) {
        return new SimpleDateFormat(dateFormat).format(date);
    }

    /*
     * 功能描述：生成对象的主键
     *
     * @return String
     */
    public static String generateId() {
        UUID uuid = UUID.randomUUID();
        String id = uuid.toString();
        //17PDF主键存在-符号，故暂不考虑替换 by songfuqiang 2022年10月19日14:17:14
//        id = id.replaceAll("-", "");
        return id;
    }


    public static String generateRightsId(long incrementalNumber) {
        String datePart = getDateTime("yyyyMMdd"); // 获取日期部分
        String numberPart = String.format("%08d", incrementalNumber); // 格式化数字为5位
        return datePart + numberPart;
    }

    public static String generatePrefixId(long incrementalNumber, String prefix) {
        String datePart = getDateTime("yyyyMMdd"); // 获取日期部分
        String numberPart = String.format("%05d", incrementalNumber); // 格式化数字为5位
        return prefix + datePart + numberPart;
    }


    public static String generateDateId() {
        Random random = new Random();
        int i = random.nextInt(9999) + 10000;
        String num = String.valueOf(i).substring(1);
        return getDateTime("yyyyMMddHHmmss") + num;
    }

    public static String generateVerifyCode() {
        return String.valueOf(new Random().nextInt(899999) + 100000);
    }

    public static Date parseDate(String dateFormat, String date) throws ParseException {
        return new SimpleDateFormat(dateFormat).parse(date);
    }

    public static String listToString(List<String> list) {

        if (list == null) {
            return null;
        }

        StringBuilder result = new StringBuilder();
        boolean first = true;

        //第一个前面不拼接","
        for (String string : list) {
            if (first) {
                first = false;
            } else {
                result.append(",");
            }
            result.append(string);
        }
        return result.toString();
    }


    /**
     * 用*覆盖内容
     *
     * @param content
     * @return
     */
    private static String coverContent(String content) {
        int length = content.length();
        int coverLength = Double.valueOf(Math.floor(length / 2)).intValue();
        StringBuilder stringBuilder = new StringBuilder();
        if (coverLength == 0) {
            return content;
        } else {
            int halfLength = Double.valueOf(Math.floor((length - coverLength) / 2)).intValue();
            for (int i = 0; i < length; i++) {
                if (i >= halfLength && (i < (halfLength + coverLength))) {
                    stringBuilder.append("*");
                } else {
                    stringBuilder.append(content.charAt(i));
                }
            }
        }
        return stringBuilder.toString();
    }

    public static String generateUriVariables(Map<String, Object> param) {
        StringBuilder variables = new StringBuilder();
        if (!CollectionUtils.isEmpty(param)) {
            variables.append("?");
            for (String key : param.keySet()) {
                if (param.get(key) != null) {
                    variables.append(key).append("=").append(param.get(key).toString()).append("&");
                }
            }
        }
        return variables.toString();
    }

    public static String getIpAddr(HttpServletRequest request) {
        String ipAddress = null;
        try {
            ipAddress = request.getHeader("x-forwarded-for");
            if (ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
                ipAddress = request.getHeader("Proxy-Client-IP");
            }
            if (ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
                ipAddress = request.getHeader("WL-Proxy-Client-IP");
            }
            if (ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
                ipAddress = request.getRemoteAddr();
                if (ipAddress.equals("127.0.0.1")) {
                    // 根据网卡取本机配置的IP
                    InetAddress inet = null;
                    try {
                        inet = InetAddress.getLocalHost();
                    } catch (UnknownHostException e) {
                        e.printStackTrace();
                    }
                    ipAddress = inet.getHostAddress();
                }
            }
            // 对于通过多个代理的情况，第一个IP为客户端真实IP,多个IP按照','分割
            if (ipAddress != null && ipAddress.length() > 15) {
                // "***.***.***.***".length()
                // = 15
                if (ipAddress.indexOf(",") > 0) {
                    ipAddress = ipAddress.substring(0, ipAddress.indexOf(","));
                }
            }
        } catch (Exception e) {
            ipAddress = "";
        }
        return ipAddress;
    }

    public static String MD5(String data) {
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            byte[] array = md.digest(data.getBytes("UTF-8"));
            StringBuilder sb = new StringBuilder();
            for (byte item : array) {
                sb.append(Integer.toHexString((item & 0xFF) | 0x100).substring(1, 3));
            }
            return sb.toString().toUpperCase();
        } catch (Exception exception) {
        }
        return null;
    }

    private static final int MIN_NUMBER = 33;
    private static final int MAX_NUMBER = 126;
    private static final int BOUND = MAX_NUMBER - MIN_NUMBER + 1;

    /**
     * 随机生成指定位数的密码
     *
     * char[33,126]，可表示数字、大小写字母、特殊字符
     *
     * @param length 密码长度
     * @return
     */
    private static final String CHARACTERS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

    public static String randomPassword(int length) {
        StringBuilder password = new StringBuilder();
        Random random = new Random();

        for (int i = 0; i < length; i++) {
            int index = random.nextInt(CHARACTERS.length());
            password.append(CHARACTERS.charAt(index));
        }

        return password.toString();
    }


}
