package cn.kdan.cloud.pdf.office.payment.webhook;

import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.payment.client.GooglePayClient;
import cn.kdan.cloud.pdf.office.payment.properties.GooglePayConfig;
import cn.kdan.cloud.pdf.office.payment.service.GooglePayService;
import cn.kdan.cloud.pdf.office.payment.webhook.google.DeveloperNotification;
import cn.kdan.cloud.pdf.office.payment.webhook.google.SubscriptionNotification;
import cn.kdan.cloud.pdf.office.payment.webhook.google.SubscriptionNotifyTypeEnum;
import cn.kdan.cloud.pdf.office.payment.webhook.google.SubscriptionPurchaseV3;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import jodd.net.URLDecoder;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.http.*;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.client.RestTemplate;

import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.concurrent.TimeUnit;

/**
 * @author ComPDFKit-WPH 2023/6/20
 */
@Controller
@RequestMapping("/google")
@RequiredArgsConstructor
@Slf4j
public class GoogleWebhookMonitor {

    private final RedissonClient redissonClient;

    private final GooglePayService googlePayService;
    private final GooglePayClient googlePayClient;
    private final GooglePayConfig googlePayConfig;
    private final RestTemplate restTemplate = new RestTemplate();

    @RequestMapping(value = "/webhook/subscription", method = RequestMethod.POST)
    public ResponseEntity<Void> receiveSubscriptionCallback(@RequestBody(required = false) byte[] body) {
        log.info("谷歌 接收到回调");
//        return ResponseEntity.status(HttpStatus.OK).build(); // 返回 200 OK
        try {
            String bodyStr = null;
            bodyStr = new String(body, StandardCharsets.UTF_8);

            // 获取 Base64 编码的字符串
            JSONObject bodyJson = JSONObject.parseObject(URLDecoder.decode(bodyStr, "utf-8"));
            String base64Data = bodyJson.getJSONObject("message").getString("data");

            // 使用 Java 标准库进行 Base64 解码
            byte[] decodedBytes = Base64.getDecoder().decode(base64Data);
            String data = new String(decodedBytes);
            //        String data = "{\"version\":\"1.0\",\"packageName\":\"com.pdftechnologies.pdfreaderpro\",\"eventTimeMillis\":\"1730450275958\",\"subscriptionNotification\":{\"version\":\"1.0\",\"notificationType\":2,\"purchaseToken\":\"nbohocncnjkkfejblaegdjcn.AO-J1OzWC-iyg4Je_5ibj2dpsby49UShnZBn0D5jEXO_9OdB0M2E4aUDahXAYflW4udRFMY1VvKvjWmhRK-QDNOe6KCG8LooRPZKQ_suYWRoSXHJiLrJBz4\",\"subscriptionId\":\"new_all_access_pack_monthly\"}}";
            log.info("谷歌 接收回调通知值：{}", data);
            DeveloperNotification developerNotification = JSONObject.parseObject(data, DeveloperNotification.class);
            if (ObjectUtils.isEmpty(developerNotification.getSubscriptionNotification())) {
                log.info("谷歌订阅回调：缺少SubscriptionNotification()");
                return ResponseEntity.status(HttpStatus.OK).build(); // 返回 200 OK
            }
            SubscriptionNotification subscriptionNotification = developerNotification.getSubscriptionNotification();
            int notificationType = subscriptionNotification.getNotificationType();
            String purchaseToken = subscriptionNotification.getPurchaseToken();
            log.info("谷歌 token：{}", purchaseToken);
            // 获取订阅订单信息
            SubscriptionPurchaseV3 subscriptionGoogleOrderV2 = googlePayClient.getSubscription(developerNotification.getPackageName(), purchaseToken);
            //测试订单
            log.info("谷歌 订阅：{}", subscriptionGoogleOrderV2.toString());
            if(subscriptionGoogleOrderV2.getExternalAccountIdentifiers().getObfuscatedExternalAccountId().contains("test")){
                if (googlePayConfig.getIsSandbox().equals("false")) {
                    log.info("正式环境转发测试回调", purchaseToken);
                    sandboxCallBack(body);
                    return ResponseEntity.status(HttpStatus.OK).build(); // 返回 200 OK
                }
            }
            //判断订单状态
            if (!googlePayClient.checkSubscriptionSuccess(subscriptionGoogleOrderV2)) {
                log.error("谷歌订单验证状态异常：{}", subscriptionGoogleOrderV2);
                return ResponseEntity.status(HttpStatus.OK).build(); // 返回 200 OK
            }
            String lockKey = Integer.toString(notificationType).concat(subscriptionGoogleOrderV2.getExternalAccountIdentifiers().getObfuscatedExternalAccountId()) + "-RedissonLock";
            RLock rLock = redissonClient.getLock(lockKey);

            try {
                // 上锁
                boolean resultLock = rLock.tryLock(30, 10, TimeUnit.SECONDS);
                if (resultLock) {
                    log.info("上锁成功, 当前事件谷歌订阅回调：{}", notificationType);
                    String thirdSubId;
                    if (ObjectUtils.isEmpty(subscriptionGoogleOrderV2.getExternalAccountIdentifiers())) {
                        //如果是旧的用户，提取订阅id
                        thirdSubId = CommonUtils.extractSubscriptionID(subscriptionGoogleOrderV2.getLatestOrderId());
                    } else {
                        thirdSubId = subscriptionGoogleOrderV2.getExternalAccountIdentifiers().getObfuscatedExternalAccountId();
                    }
                    if (SubscriptionNotifyTypeEnum.SUBSCRIPTION_PURCHASED.getType() == notificationType) {
                        // 处理购买新订阅
                    } else if (SubscriptionNotifyTypeEnum.SUBSCRIPTION_CANCELED.getType() == notificationType) {
                        googlePayService.subscriptionCancel(thirdSubId);
                        return ResponseEntity.status(HttpStatus.OK).build(); // 返回 200 OK
                    } else if (SubscriptionNotifyTypeEnum.SUBSCRIPTION_RENEWED.getType() == notificationType) {
                        // 续订处理
                        googlePayService.subscriptionPaymentSucceeded(false, thirdSubId, subscriptionGoogleOrderV2.getLatestOrderId(), 2, subscriptionGoogleOrderV2);
                        return ResponseEntity.status(HttpStatus.OK).build(); // 返回 200 OK
                    } else if (SubscriptionNotifyTypeEnum.SUBSCRIPTION_REVOKED.getType() == notificationType) {
                        // 退款
                        googlePayService.subscriptionRevoked(subscriptionGoogleOrderV2.getLatestOrderId());
                        return ResponseEntity.status(HttpStatus.OK).build(); // 返回 200 OK
                    }
                }
            } catch (InterruptedException e) {
                log.error("上锁失败", e);
                return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).build();
            } finally {
                if (rLock.isLocked() && rLock.isHeldByCurrentThread()) {
                    rLock.unlock();
                }
            }
            return ResponseEntity.status(HttpStatus.OK).build(); // 返回 200 OK

        } catch (Exception e) {
            log.error("谷歌 回调失败：" + e.getMessage() + "purchaseToken:{}");
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).build(); // 返回 500 错误
        }
    }
    public void sandboxCallBack(byte[] body) {
        byte[] requestBody = body;  // 使用传入的字节数组作为请求体

        // 创建 HttpHeaders，设置 Content-Type 为 application/json
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);

        // 创建 HttpEntity，包含字节数组请求体和请求头
        HttpEntity<byte[]> entity = new HttpEntity<>(requestBody, headers);

        // 使用 restTemplate 发送 POST 请求并检查响应状态码
        ResponseEntity<Void> response = restTemplate.exchange(
                "https://test-pdf-pro.kdan.cn/member-system-payment/google/webhook/subscription",  // URL
                HttpMethod.POST,               // 请求方法
                entity,                        // 请求体和头信息
                Void.class                     // 响应体类型，这里不需要实际的响应内容
        );

        // 检查响应状态码是否为 200 OK
        if (response.getStatusCode() != HttpStatus.OK) {
            log.error("谷歌 回调测试环境失败：响应状态码不为 200");
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TRANSACTION_ID_VALIDATION_FAILED);
        }
    }

}
