package cn.kdan.cloud.pdf.office.website.config;

import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.security.oauth2.common.exceptions.InvalidTokenException;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.token.RemoteTokenServices;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;

public class CustomRemoteTokenServices extends RemoteTokenServices {
    @Override
    public OAuth2Authentication loadAuthentication(String accessToken) throws InvalidTokenException {
        try {
            return super.loadAuthentication(accessToken);
        } catch (Exception ex) {
            Map<String, Object> map = new HashMap<>(7);
            map.put("code", ExceptionEnum.EXCEPTION_MSG_TOKEN_IS_INVALID.value());
            map.put("msg", ExceptionEnum.EXCEPTION_MSG_TOKEN_IS_INVALID.msg());
            map.put("data", "invalid_token");
            map.put("path", "path");
            map.put("timestamp", String.valueOf(System.currentTimeMillis()));
            // Token失效时的异常处理
            // 设置HTTP状态码为401
            HttpServletResponse response = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getResponse();
//            if (response != null) {
//                // 设置HTTP状态码为401
//                response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
//            }
            response.setStatus(HttpServletResponse.SC_OK);
            response.setContentType("application/json");
            try {
                ObjectMapper mapper = new ObjectMapper();
                mapper.writeValue(response.getOutputStream(), map);
            } catch (Exception e) {
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_TOKEN_IS_INVALID);
            }
            // Token失效时的异常处理
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_TOKEN_IS_INVALID);
        }
    }
}
