package cn.kdan.cloud.pdf.office.website.controller;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserSubscriptionInfoApi;
import cn.kdan.cloud.pdf.office.api.account.vo.UserSubscriptionInfo;
import cn.kdan.cloud.pdf.office.api.payment.feign.SubscriptionApi;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.api.product.feign.LoginLogApi;
import cn.kdan.cloud.pdf.office.api.product.feign.PrizeApi;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.LoginInVO;
import cn.kdan.cloud.pdf.office.api.product.vo.PrizeVO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.enums.PDFReaderProPlatformEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PayTypeEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.website.dto.SignInDTO;
import cn.kdan.cloud.pdf.office.website.service.LotteryService;
import cn.kdan.cloud.pdf.office.website.utils.SecurityUtils;
import com.alibaba.nacos.common.utils.CollectionUtils;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.security.Principal;
import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 * 活动模块
 */
@RestController
@RequestMapping("/activity")
@RequiredArgsConstructor
public class ActivityController {
    private final PrizeApi prizeApi;
    private final LoginLogApi loginLogApi;
    private final LotteryService lotteryService;
    private final SubscriptionApi subscriptionApi;
    private final UserSubscriptionInfoApi userSubscriptionInfoApi;
    private final UserApi userApi;
    private final ProductApi productApi;

    /**
     * 黑五判断用户有没有抽过奖
     * @return
     */
    @PostMapping("/canDoLottery")
    public ResultMap<Boolean> canDoLottery() {
        String userId = SecurityUtils.getUserId();
        //判断用户有没有抽过奖
        PrizeVO prizeVO = new PrizeVO();
        prizeVO.setUserId(userId);
        //1是黑五抽奖
        prizeVO.setActivityType("1");
        List<PrizeVO> list= prizeApi.list(prizeVO).getResult();
        if(CollectionUtils.isEmpty(list)){
            return ResultMap.success(true);
        }
        return ResultMap.success(false);
    }

    /**
     * 抽奖
     * @param principal
     * @return
     */
    @PostMapping("/lottery")
    public ResultMap<Integer> lottery(Principal principal) {
        String userId = SecurityUtils.getUserId();
        //判断用户有没有抽过奖
        PrizeVO prizeVO = new PrizeVO();
        prizeVO.setUserId(userId);
        //1是黑五抽奖
        prizeVO.setActivityType("1");
        List<PrizeVO> list = prizeApi.list(prizeVO).getResult();
        if (!CollectionUtils.isEmpty(list)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_ALREADY_LOTTERY);
        }
        return ResultMap.success(lotteryService.lottery(SecurityUtils.getUserId()));
    }

    //签到
    @PostMapping("/signIn")
    public ResultMap<Boolean> signIn(@RequestBody SignInDTO signInDTO) {
        String userId = SecurityUtils.getUserId();
        // 获取用户的基本信息
        UserInfoVO user = userApi.getMemberInfoById(userId).getResult();
        UserSubscriptionInfoVO activeSubscription = CommonUtils.findActiveSubscription(user.getSubscriptionInfoList(), null);

        if (activeSubscription != null && activeSubscription.getPayType().equals(1) && activeSubscription.getStatus().equals(1) && activeSubscription.getCode().contains("trail")) {
            SubscriptionsVO subscriptionsVO = subscriptionApi.getSubscriptions(userId, 2).getResult();

            // 如果结束时间在今天之后，并且距离开始时间不超过8天，设置为试用
            if (isWithinTrialPeriod(subscriptionsVO)) {
                // 存到login_log表里
                LoginInVO vo = new LoginInVO();
                BeanUtils.copyProperties(signInDTO, vo);
                vo.setUserId(userId);
                vo.setType("2");
                loginLogApi.insert(vo);

                // 获取用户签到的天数
                int loginCountForToday = loginLogApi.getLoginCountForToday(userId).getResult();
                if (loginCountForToday > 1) {
                    return ResultMap.success(true);
                }

                int loginDays = loginLogApi.getLoginDaysByUserId(userId).getResult();
                giveAiBasedOnLoginDays(userId, loginDays);

                return ResultMap.success(true);
            }
        }
        return ResultMap.success(false);
    }

    private boolean isWithinTrialPeriod(SubscriptionsVO subscriptionsVO) {
        Date now = new Date();
        return subscriptionsVO.getEndDate().after(now) && MyDateUtils.daysBetween(subscriptionsVO.getStartDate(), subscriptionsVO.getEndDate()) <= 8;
    }

    private void giveAiBasedOnLoginDays(String userId, int loginDays) {
        switch (loginDays) {
            case 1:
            case 3:
                giveAi(userId, 10);
                break;
            case 7:
                giveAi(userId, 20);
                break;
            default:
                // Do nothing
                break;
        }
    }

    private void giveAi(String userId, Integer point) {
        ProductVO product = productApi.getProductByCode("ai-giveaway").getResult();
        UserInfoVO infoVO = userApi.getMemberInfoById(userId).getResult();
        // 检查是否存在 isAI 为 1 的项
        Optional<UserSubscriptionInfoVO> aiSubscription = infoVO.getSubscriptionInfoList().stream()
                .filter(subscription -> StringUtils.isNotEmpty(subscription.getIsAi()) && "1".equals(subscription.getIsAi())) // 筛选出 isAI 为 1 的项
                .findFirst(); // 找到第一个符合条件的项
        if (aiSubscription.isPresent()) {
            // 如果存在，则修改 point
            UserSubscriptionInfoVO subscriptionInfo = aiSubscription.get();
            UserSubscriptionInfo update = new UserSubscriptionInfo();
            update.setId(subscriptionInfo.getId());
            update.setPoint(subscriptionInfo.getPoint()+point);
            userSubscriptionInfoApi.update(update);
        } else {
            // 其他必要的设置，例如用户ID，应用ID等
            UserSubscriptionInfo newInfo = new UserSubscriptionInfo().withId(CommonUtils.generateId())
                    //status 已注册
                    .withUserId(userId).withStatus(1).withAppId("1")
                    .withPlatform(PDFReaderProPlatformEnum.AI.getCode()).withPayType(PayTypeEnum.NO_AUTO.value()).withPoint(point).withProductId(product.getId());
            userSubscriptionInfoApi.insert(newInfo);
        }
    }

    private String getUserId(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            return ((CustomUserDetails) (((OAuth2Authentication) principal).getPrincipal())).getId();
        }
        throw new BackendRuntimeException(CommonConstant.EXCEPTION_MSG_LOGIN_USER_NOT_FOUND);
    }
}
