package cn.kdan.cloud.pdf.office.website.controller;

import cn.kdan.cloud.pdf.office.api.product.vo.ProductListVO;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.website.service.PDFWebV2Service;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @author ComPDFKit-Bob 2024/12/21
 * <p>
 * PDF OFFICE web端 Controller
 */
@RestController
@RequestMapping("/web/v2")
@RequiredArgsConstructor
public class PDFWebV2Controller {

    private final PDFWebV2Service pdfWebV2Service;

    /**
     * 获取上架产品，返回前端和app购买比较表
     * @param isEducation
     * @param platformId 1 官网，2 dmg，3 Mac store免费版，4e xe，5 Mac store付费版，6i OS免费版，7 iOS付费版，8 Android
     */
    @GetMapping("/getProductsList")
    public ResultMap<List<ProductListVO>> getProductsList(@Validated @NotNull(message = "是否教育优惠不能为空") @RequestParam("isEducation") Integer isEducation,
                                                          @Validated @NotNull(message = "是否试用不能为空") @RequestParam("isTrial") Integer isTrial,
                                                          @Validated @NotNull(message = "是否活动不能为空") @RequestParam("isActivity") Integer isActivity,
                                                          @Validated @NotNull(message = "平台id不能为空") @RequestParam("platformId") String platformId,
                                                          @RequestParam(value = "userId", required = false) String userId,
                                                          @RequestParam(value = "code", required = false) String code,
                                                          @RequestParam(value = "deviceSign", required = false) String deviceSign){
        return pdfWebV2Service.getProductsList(isEducation, isTrial, isActivity, platformId, userId, code, deviceSign);
    }

    /**
     * 获取购买商品价格，验证用户教育购买权限
     */
    @GetMapping("/getProductPriceByCode")
    public ResultMap<ProductListVO> getProductPriceByCode(@Validated @NotNull(message = "产品code不能为空") @RequestParam("productCode") String productCode,
                                                             @RequestParam(value = "userId", required = false) String userId,
                                                             @Validated @NotNull(message = "是否教育优惠不能为空") @RequestParam("isEducation") Integer isEducation,
                                                             @Validated @NotNull(message = "是否试用不能为空") @RequestParam("isTrial") Integer isTrial,
                                                             @Validated @NotNull(message = "是否活动不能为空") @RequestParam("isActivity") Integer isActivity,
                                                             @RequestParam(value = "platformId", required = false) String platformId,
                                                             @RequestParam(value = "deviceSign", required = false) String deviceSign
    ) {
        return ResultMap.success(pdfWebV2Service.getProductPriceByCode(productCode, userId, isEducation, isTrial, isActivity, platformId, deviceSign));
    }

}
