package cn.kdan.cloud.pdf.office.website.controller;

import cn.kdan.cloud.pdf.office.api.payment.dto.AppStoreBuyCompleteDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.AppStoreOrderSucceedDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.EquityVerificationDTO;
import cn.kdan.cloud.pdf.office.api.payment.feign.OrderApi;
import cn.kdan.cloud.pdf.office.api.payment.vo.CreateOrderResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.website.service.PaymentService;
import cn.kdan.cloud.pdf.office.website.service.ProductInfoService;
import cn.kdan.cloud.pdf.office.website.utils.SecurityUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;


@RestController
@RequestMapping("/pay")
@RequiredArgsConstructor
public class PayController {

    private final PaymentService paymentService;

    private final ProductInfoService productInfoService;

    private final OrderApi orderApi;

    /**
     * 创建订单
     *
     * @param  createOrderDTO  createOrderDTO
     * @return ResultMap
     */
    @PostMapping("/createOrder")
    public ResultMap<CreateOrderResultVO> createOrder(@RequestBody CreateOrderDTO createOrderDTO) {
        return ResultMap.success(paymentService.createOrder(createOrderDTO));
    }

    /**
     * 根据订单id获取订单详情
     * @param orderId 订单id
     * @return OrdersVO
     */
    @GetMapping("/getOrderById")
    public ResultMap<OrdersVO> getOrderById(@RequestParam String orderId) {
        return ResultMap.success(orderApi.getOrderWithUpdate(orderId).getResult());
    }


    /**
     * 订单创建（applePay专用）
     *
     * @param createOrderDTO createOrderDTO
     * @return CreateOrderResultVO
     */
    @PostMapping("/appStoreCreateOrder")
    public ResultMap<CreateOrderResultVO> appStoreCreateOrder(@RequestBody CreateOrderDTO createOrderDTO){
        return ResultMap.success(paymentService.appStoreCreateOrder(createOrderDTO));
    }


    /**
     * appStore 权益校验
     *
     * @param equityVerificationDTO equityVerificationDTO
     * @return UserInfoVO
     */
    @PostMapping("/appStoreEquityVerification")
    public ResultMap<UserInfoVO> appStoreEquityVerification(@RequestBody EquityVerificationDTO equityVerificationDTO){
        equityVerificationDTO.setUserId(SecurityUtils.getUserId());
        return ResultMap.success(paymentService.appStoreEquityVerification(equityVerificationDTO));
    }

    /**
     * 取消订阅
     * @param productId productId
     * @return String
     */
    @PostMapping("/cancelSubscription")
    public ResultMap<String> cancelSubscription(@RequestParam String productId) {
        paymentService.cancelSubscription(productId);
        return ResultMap.success(CommonConstant.RESULT_SUCCESS);
    }
}
