package cn.kdan.cloud.pdf.office.website.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserSubscriptionInfoApi;
import cn.kdan.cloud.pdf.office.api.account.vo.UserSubscriptionInfo;
import cn.kdan.cloud.pdf.office.api.email.bo.EmailSendBO;
import cn.kdan.cloud.pdf.office.api.email.feign.EmailApi;
import cn.kdan.cloud.pdf.office.api.payment.feign.OrderApi;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrderGiftLogVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.dto.UserRegisterDTO;
import cn.kdan.cloud.pdf.office.common.dto.ValidUserDTO;
import cn.kdan.cloud.pdf.office.common.enums.*;
import cn.kdan.cloud.pdf.office.common.enums.account.AccountTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PayTypeEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.website.service.OrderGiftService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;


@Service
@Slf4j
@RequiredArgsConstructor
@Transactional
public class OrderGiftServiceImpl implements OrderGiftService {
    private final UserApi userApi;
    private final EmailApi emailApi;
    private final OrderApi orderApi;
    private final ProductApi productApi;
    @Resource
    private UserSubscriptionInfoApi userSubscriptionInfoApi;
    @Value("${orderGiftUrl:}")
    private String orderGiftUrl;

    @Override
    public Integer getGiftNum(String userId) {
        return Math.toIntExact(orderApi.getGiftNum(userId).getResult());
    }

    @Override
    public void giveAway(String userId, String name, String email, boolean isGift) {
        //赠送会员
        //判断接受人是否已经有账号
        UserVO userVO = userApi.getByEmail(email).getResult();
        String giftEmail = userApi.getById(userId).getResult().getEmail();
        AtomicBoolean gifted = new AtomicBoolean(false);
        //如果没有账号直接发送赠送邮件,并且记录赠送日志
        if (ObjectUtils.isEmpty(userVO)) {
            OrderGiftLogVO orderGiftLogVO = orderApi.giveAway(userId, email, name).getResult();
            String orderGiftId = orderGiftLogVO.getId();
            EmailSendBO bo = new EmailSendBO();
            bo.setToEmail(email);
            handleEmailSendBO(EmailCodeTypeEnum.GIVE_AWAY, bo, giftEmail,email,orderGiftId);
            //修改orderGift中的一条记录为赠送中,且记录赠送日志为未接收
        } else {
            UserInfoVO userInfoVO = userApi.getMemberInfoById(userVO.getId()).getResult();
            userInfoVO.getSubscriptionInfoList().forEach(userSubscriptionInfoVO -> {
                //判断是否是免费用户
                //是自动订阅
                //是永久会员并且在订阅中
                if (userSubscriptionInfoVO.getPaymentModel().equals("2")
                        && userSubscriptionInfoVO.getStatus().equals(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value())) {
                    //2.如果是永久会员，提示：该用户已经是PDF Reader Pro永久版会员，无需再赠送哦~
                    throw new BackendRuntimeException(ExceptionEnum.USER_ALREADY_SUBSCRIBED_PACKAGE);
                }
                //是订阅中的订阅会员, //3.如果是订阅会员,如果有isGift且为ture,则直接赠送，如果没有则提醒你已经是订阅会员了
                if (userSubscriptionInfoVO.getPaymentModel().equals("1")&&userSubscriptionInfoVO.getPlatform().equals(2)) {
                    if (isGift) {
                        OrderGiftLogVO orderGiftLogVO = orderApi.giveAway(userId, email, name).getResult();
                        String orderGiftId = orderGiftLogVO.getId();
                        //修改orderGift中的一条记录为赠送中,且记录赠送日志为未接收
                        EmailSendBO bo = new EmailSendBO();
                        bo.setToEmail(email);
                        handleEmailSendBO(EmailCodeTypeEnum.GIVE_AWAY, bo, giftEmail,email,orderGiftId);
                        gifted.set(true);
                    } else {
                        throw new BackendRuntimeException(ExceptionEnum.USER_ALREADY_SUBSCRIBED_PLAN);
                    }
                }
            });
            if(!gifted.get()){
                //发送赠送邮件
                //1.如果是免费用户，接收人是免费用户，则直接发送邮件 （赠送邮件24小时有效，超过24小时未确认，邮件失效，【接收会员】按钮点击提示已失效，需要用户重新发起赠送邀请）
                //修改orderGift中的一条记录为赠送中,且记录赠送日志为未接收
                OrderGiftLogVO orderGiftLogVO = orderApi.giveAway(userId, email, name).getResult();
                String orderGiftId = orderGiftLogVO.getId();
                //存到redis里
                EmailSendBO bo = new EmailSendBO();
                bo.setToEmail(email);
                handleEmailSendBO(EmailCodeTypeEnum.GIVE_AWAY, bo, giftEmail,email,orderGiftId);
            }

        }
    }


    @Override
    public void cancelGiveAway(String orderGiftLogId) {
        OrderGiftLogVO vo = orderApi.getOrderGiftLogById(orderGiftLogId).getResult();
        if(vo.getValidFlag().equals(OrderGiftLogVaildFlagEnum.RECEIVE_GIFT.getCode())){
            orderApi.cancelGiveAway(orderGiftLogId);
        }
    }

    void handleEmailSendBO(EmailCodeTypeEnum action, EmailSendBO bo, String userEmail,String receiveEmail,String orderGiftLogId) {
        //设置事件
        bo.setUseEvent(action.value());
        //设置title
        Map<String, String> titleMap = new HashMap<>();
        //设置内容
        Map<String, String> contentMap = new HashMap<>();
        contentMap.put("@userEmail@", userEmail);
        contentMap.put("@receviceEmail@", receiveEmail);
        contentMap.put("@url@", orderGiftUrl+orderGiftLogId);
        bo.setSendTitleContent(titleMap);
        //设置内容
        bo.setSendContent(contentMap);
        emailApi.sendEmail(bo);
    }

    @Override
    //链接过期的逻辑(已完成)
    //接受会员接口
    public void acceptUser(String orderGiftLogId) {
        OrderGiftLogVO vo = orderApi.getOrderGiftLogById(orderGiftLogId).getResult();
        String email = vo.getEmail();
        if(vo.getValidFlag().equals(OrderGiftLogVaildFlagEnum.RECEIVE_CANCEL.getCode())){
            throw new BackendRuntimeException(ExceptionEnum.ORDER_GIFT_LOG_ALREADY_CANCEL);
        }
        if(!vo.getValidFlag().equals(OrderGiftLogVaildFlagEnum.RECEIVE_GIFT.getCode())){
            throw new BackendRuntimeException(ExceptionEnum.ORDER_GIFT_LOG_ALREADY_RECEIVE);
        }
        UserVO userInfoVO = userApi.getByEmail(email).getResult();
        //如果没有账号创建账号，并且直接赠送会员
        if (ObjectUtils.isEmpty(userInfoVO)) {
            ValidUserDTO dto = new ValidUserDTO();
            dto.setValidType("2");
            dto.setUsername(email);
            dto.setPlatformType(ValidStatusEnum.VALID.value());
            UserRegisterDTO userRegisterDTO = new UserRegisterDTO();
            userRegisterDTO.setUsername(email);
            userRegisterDTO.setAppId("1");
            //默认为0官网账号
            userRegisterDTO.setPlatformType("0");
            userRegisterDTO.setAccountTypeEnum(AccountTypeEnum.PAID_ACCOUNT);
            ResultMap<Boolean> resultMap = userApi.register(userRegisterDTO);
            if(resultMap.getCode() == CommonConstant.SUCCESS){
                userInfoVO = userApi.getByEmail(email).getResult();
            //插永久版的订阅info
            }
        }else {
            UserInfoVO user = userApi.getMemberInfoById(userInfoVO.getId()).getResult();
            user.getSubscriptionInfoList().forEach(userSubscriptionInfoVO -> {
                if (userSubscriptionInfoVO.getPaymentModel().equals("2")
                        && userSubscriptionInfoVO.getStatus().equals(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value())) {
                    //2.如果是永久会员，提示：该用户已经是PDF Reader Pro永久版会员，无需再赠送哦~
                    throw new BackendRuntimeException(ExceptionEnum.USER_ALREADY_SUBSCRIBED_PACKAGE);
                }
            });
        }
        ProductVO product = productApi.getProductByCode("advanced-permanent").getResult();
        UserSubscriptionInfo newInfo = new UserSubscriptionInfo().withId(CommonUtils.generateId())
                //status 已注册
                .withUserId(userInfoVO.getId()).withStatus(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value()).withAppId("1").withPlatform(PDFReaderProPlatformEnum.PERMANENT.getCode()).withPayType(PayTypeEnum.NO_AUTO.value()).withProductId(product.getId().toString());
        userSubscriptionInfoApi.insert(newInfo);
        //如果有账号，则直接赠送，（如果永久版是不能赠送的，所以用户一定没有永久版）
        orderApi.acceptUser(orderGiftLogId,userInfoVO.getId());
    }

}
