package cn.kdan.cloud.pdf.office.website.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserSubscriptionInfoApi;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ListingProductVO;
import cn.kdan.cloud.pdf.office.api.product.vo.ParentProductVO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductListVO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.dto.LicenseCodeResult;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionStatusEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.utils.PDFReaderUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.website.service.PDFWebV2Service;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author ComPDFKit-Bob 2023/2/6
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class PDFWebV2ServiceImpl implements PDFWebV2Service {

    private final ProductApi productApi;

    private final UserApi userAPi;
    private final UserSubscriptionInfoApi userSubscriptionInfoApi;
    @Value("${pdfReaderUrl:123/}")
    private String pdfReaderUrl;

//    private final OrderApi orderApi;

    @Override
    public ResultMap<List<ProductListVO>> getProductsList(Integer isEducation, Integer isTrial, Integer isActivity, String platformId, String userId, String code, String deviceSign) {
        // 筛选订单购买的产品中有没有vip订阅类型
        boolean hasVip = false;
        UserSubscriptionInfoVO vipSubscriptionInfoVO = null;
        boolean hasSVip = false;
        // 买断版支持平台数
        int permanentPlatform = 0;
        // 判断用户是否登录，登录的用户获取已购买的权益
        UserInfoVO userVO = null;
        if (StringUtils.isNotEmpty(userId)) {
            userVO = userAPi.getMemberInfoById(userId).getResult();
            handOldRight(platformId, deviceSign, userVO);
            List<UserSubscriptionInfoVO> userSubscriptionInfoVOList = userVO.getSubscriptionInfoList().stream().filter(Objects::nonNull).filter(s -> s.getPlatform() != 0 && s.getPlatform() != 3 && s.getStatus() != 2 && s.getStatus() != 4).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(userSubscriptionInfoVOList)) {
                for (UserSubscriptionInfoVO userSubscriptionInfoVO : userSubscriptionInfoVOList) {
                    // 买断的产品，判断是否多平台
                    if (userSubscriptionInfoVO.getPaymentModel().equals("2") && StringUtils.isNotEmpty(userSubscriptionInfoVO.getPlatforms())) {
                        if (userSubscriptionInfoVO.getPlatforms().contains("Mac")) permanentPlatform++;
                        if (userSubscriptionInfoVO.getPlatforms().contains("Windows")) permanentPlatform++;
                        if (userSubscriptionInfoVO.getPlatforms().contains("iOS")) permanentPlatform++;
                        if (userSubscriptionInfoVO.getPlatforms().contains("Android")) permanentPlatform++;
                    } else {
                        // 优先判断高级版
                        if (StringUtils.isNotEmpty(userSubscriptionInfoVO.getLevels())&&userSubscriptionInfoVO.getLevels().contains("3")) {
                            hasSVip = true;
                        } else if (StringUtils.isNotEmpty(userSubscriptionInfoVO.getLevels())&&userSubscriptionInfoVO.getLevels().contains("2")) {
                            hasVip = true;
                            vipSubscriptionInfoVO = userSubscriptionInfoVO;
                        }
                    }
                }
            }
        }
        // 根据平台id获取所有父商品
        List<ParentProductVO> productList = productApi.getProductListByPlatformId(platformId).getResult();
        List<ProductListVO> productListVOList = new ArrayList<>();
        for (ParentProductVO parentProduct : productList) {
            ProductListVO productListVO = new ProductListVO();
            productListVO.setParentId(parentProduct.getId());
            productListVO.setParentProductCode(parentProduct.getCode());
            productListVO.setIsPossibleToPurchase(1);
            productListVO.setTypeOfPurchase(1);
            productListVO.setIsEducation(0);
            // 4单平台买断升级，5双平台买断升级，6标准订阅升级,教育买断升级
            String upgradeProductCode = null;
            // 根据用户的权益和需要购买的商品，先判断能不能升级
            String productCode = parentProduct.getCode();
            if (productCode.contains("高级版年订阅")) {
                if (hasVip) {
                    // 标准订阅升级
                    upgradeProductCode = "advanced-annual-subscription-standard-year-upgrade";
                    productListVO.setTypeOfPurchase(4);
                } else if (hasSVip) {
                    // 判断用户是否还在订阅中,订阅中的不能购买，取消订阅可以提前续费
                    if (userVO.getSubscriptionInfoList().stream().filter(Objects::nonNull)
                            .anyMatch(userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatform().equals(2) && userSubscriptionInfoVO.getPayType() == 1)) {
                        productListVO.setIsPossibleToPurchase(0);
                        productListVOList.add(productListVO);
                        continue;
                    } else {
                        // 提前续费同等级的，判断续费商品是不是全平台
                        if (userVO.getSubscriptionInfoList().stream().filter(Objects::nonNull)
                                .anyMatch(userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatforms().split(",").length != 4)) {
                            productListVO.setIsPossibleToPurchase(0);
                            productListVOList.add(productListVO);
                            continue;
                        }
                        productListVO.setTypeOfPurchase(2);
                    }
                } else if (permanentPlatform > 0) {
                    // 单平台买断升级
                    upgradeProductCode = "advanced-annual-subscription-single-upgrade";
                    if (permanentPlatform > 1) {
                        // 双平台买断升级
                        upgradeProductCode = "advanced-annual-subscription-multi-upgrade";
                    }
                    productListVO.setTypeOfPurchase(3);
                }
            } else if (productCode.contains("标准版年订阅")) {
                if (hasSVip) {
                    // svip无法购买vip
                    productListVO.setIsPossibleToPurchase(0);
                    productListVOList.add(productListVO);
                    continue;
                } else if (hasVip) {
                    // vip提前续费
                    // 判断用户是否还在订阅中,
                    if (userVO.getSubscriptionInfoList().stream().filter(Objects::nonNull)
                            .anyMatch(userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatform().equals(2) && userSubscriptionInfoVO.getPayType() == 1)) {
                        productListVO.setIsPossibleToPurchase(0);
                        productListVOList.add(productListVO);
                        continue;
                    } else {
                        // 提前续费同等级的，判断续费商品是不是全平台
                        if (userVO.getSubscriptionInfoList().stream().filter(Objects::nonNull)
                                .anyMatch(userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatforms().split(",").length != 4)) {
                            productListVO.setIsPossibleToPurchase(0);
                            productListVOList.add(productListVO);
                            continue;
                        }
                        productListVO.setTypeOfPurchase(2);
                    }
                } else if (permanentPlatform > 0) {
                    // 买断无法购买vip
                    productListVO.setIsPossibleToPurchase(0);
                    productListVOList.add(productListVO);
                    continue;
                }
            }
            // 判断子产品优惠类型，0正价，1教育优惠，2活动，3试用，4升级，10试用+活动
            Integer discountType = 0;
            if (StringUtils.isEmpty(upgradeProductCode)) {
                // 如果不能升级，根据请求参数获取对应的优惠类型
                if (isActivity == 1 && isTrial == 0 && !productCode.contains("标准版年订阅")) {
                    discountType = 2;
                } else if (isActivity == 0 && isTrial == 1) {
                    discountType = 3;
                } else if (isActivity == 1 && isTrial == 1) {
                    discountType = 10;
                }
            } else {
                discountType = 4;
            }
            // 根据父商品id和优惠类型获取子商品
            List<ProductVO> productVOList = productApi.getProductByParentIdAndDiscountType(parentProduct.getId(), discountType).getResult();
            if (ObjectUtils.isEmpty(productVOList)) {
                productListVO.setIsPossibleToPurchase(0);
            } else {
                if (StringUtils.isEmpty(upgradeProductCode)) {
                    // 不是升级类型的子商品，应该只有一个
                    if (productVOList.size() > 1) {
                        log.error("父商品id为 {} 的 {} 类型子商品列表长度大于1", parentProduct.getId(), discountType);
                    }
                    BeanUtils.copyProperties(productVOList.get(0), productListVO);
                } else {
                    // 根据升级code找子商品
                    String finalUpgradeProductCode = upgradeProductCode;
                    ProductVO productVO = productVOList.stream().filter(item -> item.getCode().equals(finalUpgradeProductCode)).findFirst().orElse(null);
                    if (ObjectUtils.isEmpty(productVO)) {
                        productListVO.setIsPossibleToPurchase(0);
                        productListVOList.add(productListVO);
                        continue;
                    }
                    BeanUtils.copyProperties(productVO, productListVO);
                    // 标准订阅升级，计算差价
                    if (upgradeProductCode.equals("advanced-annual-subscription-standard-year-upgrade")) {
                        if (ObjectUtils.isEmpty(vipSubscriptionInfoVO)) {
                            log.error("标准订阅升级，用户标准权益错误");
                        }
                        // 计算用户订阅的endDate到现在的天数
                        long diff = ChronoUnit.DAYS.between(LocalDate.now(), vipSubscriptionInfoVO.getEndDate().toInstant().atZone(ZoneId.systemDefault()).toLocalDate());
                        log.info("标准订阅升级，用户标准权益剩余天数为 {} 天", diff);
                        if (diff > 0) {
                            // 计算美元差价, (99.99 - 79.99 ) / 365 * 天数
                            productListVO.setDisplayPrice(productVO.getPrice().subtract(new BigDecimal("79.99"))
                                    .multiply(new BigDecimal(diff)).divide(new BigDecimal("365"), 2, RoundingMode.HALF_UP));
                            // 计算人名币差价, (649.00 - 519.00 ) / 365 * 天数
                            productListVO.setCnyDisplayPrice(productVO.getCnyPrice().subtract(new BigDecimal("519.00"))
                                    .multiply(new BigDecimal(diff)).divide(new BigDecimal("365"), 2, RoundingMode.HALF_UP));
                        }
                    }
                }
            }
            productListVOList.add(productListVO);
        }
        if (isEducation == 1) {
            // 额外获取每个父商品下的教育优惠商品
            for (ParentProductVO parentProduct : productList) {
                ProductListVO productListVO = new ProductListVO();
                productListVO.setParentId(parentProduct.getId());
                productListVO.setParentProductCode(parentProduct.getCode());
                productListVO.setIsPossibleToPurchase(1);
                productListVO.setTypeOfPurchase(1);
                productListVO.setIsEducation(1);
                List<ProductVO> productVOList = productApi.getProductByParentIdAndDiscountType(parentProduct.getId(), 1).getResult();
                if (ObjectUtils.isEmpty(productVOList)) {
                    productListVO.setIsPossibleToPurchase(0);
                } else {
                    BeanUtils.copyProperties(productVOList.get(0), productListVO);
                }
                productListVOList.add(productListVO);
            }
        }
        if (StringUtils.isNotEmpty(code) && "B89EF-166C-4AD".equals(code)) {
            // 运营 4.7.6 试用引导&退订挽回活动
            // 标准版年订阅 Coupon: $10 -> 69.99
            // 高级版年订阅 Coupon: $30 -> 69.99
            // 永久版      Coupon: $20 -> 99.99
            productListVOList.forEach(listingProductVO -> {
                BigDecimal couponPrice;
                BigDecimal cnyCouponPrice;
                if ("standard-annual-subscription".equals(listingProductVO.getCode())) {
                    couponPrice = new BigDecimal("10");
                    cnyCouponPrice = new BigDecimal("60");
                } else if ("advanced-annual-subscription".equals(listingProductVO.getCode())) {
                    couponPrice = new BigDecimal("30");
                    cnyCouponPrice = new BigDecimal("180");
                } else if ("advanced-permanent".equals(listingProductVO.getCode())) {
                    couponPrice = new BigDecimal("20");
                    cnyCouponPrice = new BigDecimal("120");
                } else {
                    return;
                }
//                listingProductVO.setTypeOfPurchase(5);
                listingProductVO.setDisplayAmount(couponPrice);
                listingProductVO.setCnyDisplayAmount(cnyCouponPrice);
                listingProductVO.setDisplayPrice(null != listingProductVO.getDisplayPrice() ? listingProductVO.getDisplayPrice().subtract(couponPrice) : listingProductVO.getPrice().subtract(couponPrice));
                listingProductVO.setCnyDisplayPrice(null != listingProductVO.getCnyDisplayPrice() ? listingProductVO.getCnyDisplayPrice().subtract(cnyCouponPrice) : listingProductVO.getCnyPrice().subtract(cnyCouponPrice));
            });
        }
        return ResultMap.success(productListVOList);
    }

    @Override
    public ProductListVO getProductPriceByCode(String productCode, String userId, Integer isEducation, Integer isTrial, Integer isActivity, String platformId, String deviceSign) {
        // 查询购买的商品信息
        ListingProductVO listingProductVO = productApi.getProductsByCode(productCode).getResult();
        if (ObjectUtils.isEmpty(listingProductVO)) {
            log.error("商品不存在，id：{}", productCode);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRODUCT_NOT_EXIST);
        }
        // 筛选订单购买的产品中有没有vip订阅类型
        boolean hasVip = false;
        UserSubscriptionInfoVO vipSubscriptionInfoVO = null;
        boolean hasSVip = false;
        // 买断版  平台数
        int permanentPlatform = 0;
        // 判断用户是否登录，登录的用户获取已购买的权益
        UserInfoVO userVO = null;
        if (ObjectUtils.isNotEmpty(userId)) {
            userVO = userAPi.getMemberInfoById(userId).getResult();
            handOldRight(platformId, deviceSign, userVO);
            List<UserSubscriptionInfoVO> userSubscriptionInfoVOList = userVO.getSubscriptionInfoList().stream().filter(Objects::nonNull).filter(s -> s.getPlatform() != 0 &&  s.getPlatform() != 3 && s.getStatus() != 2 && s.getStatus() != 4).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(userSubscriptionInfoVOList)) {
                for (UserSubscriptionInfoVO userSubscriptionInfoVO : userSubscriptionInfoVOList) {
                    // 买断的产品，判断是否多平台
                    if (userSubscriptionInfoVO.getPaymentModel().equals("2") && StringUtils.isNotEmpty(userSubscriptionInfoVO.getPlatforms())) {
                        if (userSubscriptionInfoVO.getPlatforms().contains("Mac")) permanentPlatform++;
                        if (userSubscriptionInfoVO.getPlatforms().contains("Windows")) permanentPlatform++;
                        if (userSubscriptionInfoVO.getPlatforms().contains("iOS")) permanentPlatform++;
                        if (userSubscriptionInfoVO.getPlatforms().contains("Android")) permanentPlatform++;
                    } else {
                        // 优先判断高级版
                        if (StringUtils.isNotEmpty(userSubscriptionInfoVO.getLevels())&&userSubscriptionInfoVO.getLevels().contains("3")) {
                            hasSVip = true;
                        } else if (StringUtils.isNotEmpty(userSubscriptionInfoVO.getLevels())&&userSubscriptionInfoVO.getLevels().contains("2")) {
                            hasVip = true;
                            vipSubscriptionInfoVO = userSubscriptionInfoVO;
                        }
                    }
                }
            }
        }
        ProductListVO productListVO = new ProductListVO();
        productListVO.setIsPossibleToPurchase(1);
        productListVO.setTypeOfPurchase(1);
        String upgradeProductCode = null;
        if (listingProductVO.getPaymentModel() == 1) {
            // 根据用户的权益和需要购买的商品，先判断能不能升级
            if (!StringUtils.isEmpty(listingProductVO.getLevels())&&listingProductVO.getLevels().contains("3")) {
                if (hasVip) {
                    // 标准订阅升级
                    upgradeProductCode = "advanced-annual-subscription-standard-year-upgrade";
                    productListVO.setTypeOfPurchase(4);
                } else if (hasSVip) {
                    // 判断用户是否还在订阅中,订阅中的不能购买，取消订阅可以提前续费
                    if (userVO.getSubscriptionInfoList().stream().filter(Objects::nonNull)
                            .anyMatch(userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatform().equals(2) && userSubscriptionInfoVO.getPayType() == 1)) {
                        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_ALREADY_SUBSCRIBED);
                    } else {
                        // 提前续费同等级的，判断续费商品平台
                        // 判断字符串逗号分割后的数组长度是不是一致
                        if (userVO.getSubscriptionInfoList().stream().filter(Objects::nonNull)
                                .anyMatch(userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatforms().split(",").length != listingProductVO.getPlatforms().split(",").length)) {
                            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_ALREADY_SUBSCRIBED);
                        }
                        productListVO.setTypeOfPurchase(2);
                    }
                } else if (permanentPlatform > 0) {
                    if (isEducation != 1) {
                        // 单平台买断升级
                        upgradeProductCode = "advanced-annual-subscription-single-upgrade";
                        if (permanentPlatform > 1) {
                            // 双平台买断升级
                            upgradeProductCode = "advanced-annual-subscription-multi-upgrade";
                        }
                    } else {
                        // 买断版的教育升级
                        upgradeProductCode = "advanced-annual-subscription-edu-upgrade";
                        productListVO.setIsEducation(1);
                    }
                    productListVO.setTypeOfPurchase(3);
                }
            } else if (!StringUtils.isEmpty(listingProductVO.getLevels())&&listingProductVO.getLevels().contains("2")) {
                // 购买vip
                if (hasSVip) {
                    // svip无法购买vip
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_CANNOT_BUY_VIP);
                } else if (hasVip) {
                    // vip提前续费
                    // 判断用户是否还在订阅中,
                    if (userVO.getSubscriptionInfoList().stream().filter(Objects::nonNull)
                            .anyMatch(userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatform().equals(2) && userSubscriptionInfoVO.getPayType() == 1)) {
                        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_ALREADY_SUBSCRIBED);
                    } else {
                        // 提前续费同等级的，判断续费商品平台
                        // 判断字符串逗号分割后的数组长度是不是一致
                        if (userVO.getSubscriptionInfoList().stream().filter(Objects::nonNull)
                                .anyMatch(userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatforms().split(",").length != listingProductVO.getPlatforms().split(",").length)) {
                            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_ALREADY_SUBSCRIBED);
                        }
                        productListVO.setTypeOfPurchase(2);
                    }
                } else if (permanentPlatform > 0) {
                    // 买断无法购买vip
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_CANNOT_UPGRADE_VIP);
                }
            }
        }

        // 判断子产品优惠类型，0正价，1教育优惠，2活动，3试用，4升级，10试用+活动
        Integer discountType = 0;
        // 如果不能升级，根据请求参数获取对应的优惠类型
        if (StringUtils.isEmpty(upgradeProductCode)) {
            if (isEducation == 1) {
                discountType = 1;
            } else if (isActivity == 1 && isTrial == 0) {
                discountType = 2;
            } else if (isActivity == 0 && isTrial == 1) {
                discountType = 3;
            } else if (isActivity == 1 && isTrial == 1) {
                discountType = 10;
            }
        } else {
            discountType = 4;
        }
        // 根据父商品id和优惠类型获取子商品
        List<ProductVO> productVOList = productApi.getProductByParentIdAndDiscountType(listingProductVO.getParentId(), discountType).getResult();
        if (ObjectUtils.isEmpty(productVOList)) {
            productListVO.setIsPossibleToPurchase(0);
        } else {
            if (StringUtils.isEmpty(upgradeProductCode)) {
                // 不是升级类型的子商品，应该只有一个
                if (productVOList.size() > 1) {
                    log.error("父商品id为 {} 的 {} 类型子商品列表长度大于1", listingProductVO.getParentId(), discountType);
                }
                BeanUtils.copyProperties(productVOList.get(0), productListVO);
            } else {
                // 根据升级code找子商品
                String finalUpgradeProductCode = upgradeProductCode;
                ProductVO productVO = productVOList.stream().filter(item -> item.getCode().equals(finalUpgradeProductCode)).findFirst().orElse(null);
                if (ObjectUtils.isEmpty(productVO)) {
                    log.error("升级商品错误");
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_ALREADY_SUBSCRIBED);
                }
                BeanUtils.copyProperties(productVO, productListVO);
                // 标准订阅升级，计算差价
                if (upgradeProductCode.equals("advanced-annual-subscription-standard-year-upgrade")) {
                    if (ObjectUtils.isEmpty(vipSubscriptionInfoVO)) {
                        log.error("标准订阅升级，用户标准权益错误");
                    }
                    // 计算用户订阅的endDate到现在的天数
                    long diff = ChronoUnit.DAYS.between(LocalDate.now(), vipSubscriptionInfoVO.getEndDate().toInstant().atZone(ZoneId.systemDefault()).toLocalDate());
                    log.info("标准订阅升级，用户标准权益剩余天数为 {} 天", diff);
                    if (diff > 0) {
                        // 计算美元差价, (99.99 - 79.99 ) / 365 * 天数
                        productListVO.setDisplayPrice(productVO.getPrice().subtract(new BigDecimal("79.99"))
                                .multiply(new BigDecimal(diff)).divide(new BigDecimal("365"), 2, RoundingMode.HALF_UP));
                        // 计算人名币差价, (649.00 - 519.00 ) / 365 * 天数
                        productListVO.setCnyDisplayPrice(productVO.getCnyPrice().subtract(new BigDecimal("519.00"))
                                .multiply(new BigDecimal(diff)).divide(new BigDecimal("365"), 2, RoundingMode.HALF_UP));
                    }
                }
            }
        }
        return productListVO;
    }

    private void handOldRight(String platformId, String deviceSign, UserInfoVO userVO) {
        //官网：查询邮箱下的序列码权益并且转换
        if(platformId.equals("1")){
            List<LicenseCodeResult> licenseCodes = PDFReaderUtils.getLicenseByEmail(pdfReaderUrl, userVO.getEmail());
            //只有一个序列码的时候才转化
            if(licenseCodes.size()==1){
                UserSubscriptionInfoVO vo = userSubscriptionInfoApi.getByOldProductCode(licenseCodes.get(0).getCode()).getResult();
                //转化来的订阅没有状态，设置为1
                vo.setStatus(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value());
                userVO.getSubscriptionInfoList().add(vo);
            }
        }
        //dmg和exe，设备上的序列码购买邮箱和账号邮箱一致时才计入
        if(platformId.equals("2")|| platformId.equals("4")){
            List<LicenseCodeResult> licenseCodes = PDFReaderUtils.getLicenseByDevice(pdfReaderUrl, deviceSign);
            if(licenseCodes.size()==1&&licenseCodes.get(0).getEmail().equals(userVO.getEmail())){
                UserSubscriptionInfoVO vo = userSubscriptionInfoApi.getByOldProductCode(licenseCodes.get(0).getCode()).getResult();
                //转化来的订阅没有状态，设置为1
                vo.setStatus(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value());
                userVO.getSubscriptionInfoList().add(vo);
            }
        }
    }

}
