package cn.kdan.cloud.pdf.office.website.utils;

import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;

import java.util.Optional;

/**
 * @author wph 2022/7/5
 */
public class SecurityUtils {

    /**
     * 获取认证信息
     *
     * @return 认证信息
     */
    public static Authentication getAuthentication() {
        return SecurityContextHolder.getContext().getAuthentication();
    }

    /**
     * 获取用户信息
     *
     * @return 用户信息
     */
    public static CustomUserDetails getUser() {
        Authentication authentication = getAuthentication();
        Object principal = authentication.getPrincipal();
        if (principal instanceof CustomUserDetails) {
            return (CustomUserDetails) principal;
        }else if (principal instanceof String){
            return new CustomUserDetails((String) principal);
        }
        throw new BackendRuntimeException("未知用户类型！");
    }

    /**
     * 获取用户id
     *
     * @return 用户id
     */
    public static String getUserId() {
        return Optional.of(getUser()).map(CustomUserDetails::getId).orElse(null);
    }
}
