package cn.kdan.cloud.pdf.office.product.service.impl;

import cn.kdan.cloud.pdf.office.api.product.vo.ActivityInfoVO;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.product.entity.ActivityInformation;
import cn.kdan.cloud.pdf.office.product.error.ErrorMessage;
import cn.kdan.cloud.pdf.office.product.mapper.ActivityInformationMapper;
import cn.kdan.cloud.pdf.office.product.service.ActivityInformationService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

/**
 * @author ComPDFKit-WPH 2023-01-10
 */
@Service
public class ActivityInformationServiceImpl extends ServiceImpl<ActivityInformationMapper, ActivityInformation> implements ActivityInformationService {


    @Override
    public ActivityInfoVO getActivityInfoByType(String type) {
        ActivityInfoVO activityInfoVO = this.baseMapper.selectActivityInfoByType(type);
        if (ObjectUtils.isNull(activityInfoVO)){
            // 查询数据为空抛出异常 没有当前活动类型信息
            throw new BackendRuntimeException(ErrorMessage.NO_CURRENT_ACTIVITY);
        }
        return activityInfoVO;
    }

    @Override
    public ActivityInfoVO getActivityInfoByActivityId(String activityId){
        ActivityInformation activityInformation = this.baseMapper.selectOne(new LambdaQueryWrapper<ActivityInformation>()
                .eq(ActivityInformation::getId, activityId)
                .eq(ActivityInformation::getIsEnable, 1));
        ActivityInfoVO activityInfoVO = new ActivityInfoVO();
        activityInfoVO.setActivityInfo(activityInformation.getContentInformation());
        BeanUtils.copyProperties(activityInformation,activityInfoVO);
        return activityInfoVO;
    }

}
