package cn.kdan.cloud.pdf.office.product.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.utils.SubscriptionUtil;
import cn.kdan.cloud.pdf.office.api.product.vo.*;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.api.product.dto.ProductListDTO;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.JsonUtils;
import cn.kdan.cloud.pdf.office.product.entity.ParentProduct;
import cn.kdan.cloud.pdf.office.product.entity.Prize;
import cn.kdan.cloud.pdf.office.product.entity.Product;
import cn.kdan.cloud.pdf.office.product.entity.RProductPay;
import cn.kdan.cloud.pdf.office.product.error.ErrorMessage;
import cn.kdan.cloud.pdf.office.product.mapper.ProductMapper;
import cn.kdan.cloud.pdf.office.product.service.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author ComPDFKit-WPH 2023-01-28
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class ProductServiceImpl extends ServiceImpl<ProductMapper, Product> implements ProductService {

    private final ActivityInformationService activityService;

    private final UserApi userAPi;

//    private final OrderApi orderApi;

    private final RProductPayService rProductPayService;

    private final EduEmailsService eduEmailsService;

    private final PrizeService prizeService;

    private final ParentProductService parentProductService;

    @Value("${blackFiveProduct:advanced-permanent-blackFive,advanced-annual-subscription-blackFive}")
    private List<String> blackFiveProduct;



    @Override
    public IPage<ProductVO> page(ProductListDTO query) {
        Page<Product> productPage = this.page(new Page<>(query.getPage(), query.getSize()), new LambdaQueryWrapper<Product>()
                .eq(Product::getAppId, query.getAppId()));
        List<ProductVO> productVOS = JsonUtils.jsonStringToList(JsonUtils.getJsonString(productPage.getRecords()), ProductVO.class);
        productVOS.forEach(productVO -> {
            if (!ObjectUtils.isEmpty(productVO.getActivityId())) {
                productVO.setActivityInfo(activityService.getActivityInfoByActivityId(productVO.getActivityId()));
            }
        });
        Page<ProductVO> page = new Page<>(query.getPage(), query.getSize());
        page.setRecords(productVOS);
        return page;
    }

    @Override
    public ProductPriceVo getProductPrice(String productId, String userId) {
        Product product = this.baseMapper.selectOne(new LambdaQueryWrapper<Product>()
                .select(Product::getPrice, Product::getCnyPrice, Product::getPaddleProductId, Product::getActivityId)
                .eq(Product::getId, productId));
        ProductPriceVo productPriceVo = new ProductPriceVo();
        productPriceVo.setPaddleProductId(product.getPaddleProductId());
        // 判断当前产品是否有活动 获取活动的最终价格
        if (!ObjectUtils.isEmpty(product.getActivityId())) {
            ActivityInfoVO activityInfo = activityService.getActivityInfoByActivityId(product.getActivityId());
            // 判断是否是新用户专用
            List<UserSubscriptionInfoVO> subscriptionInfoList = userAPi.getInfoById(userId).getResult().getSubscriptionInfoList();
            if (activityInfo.getActivityType() == 1) {
                UserSubscriptionInfoVO userSubscriptionInfoVO = SubscriptionUtil.getUserSubscriptionInfoByPlatform(subscriptionInfoList, product.getPlatform());
                if (ObjectUtils.isEmpty(userSubscriptionInfoVO)) {
                    productPriceVo.setPrice(activityInfo.getFinalPrice());
                    productPriceVo.setCnyPrice(activityInfo.getFinalCnyPrice());
                } else {
                    throw new BackendRuntimeException(ErrorMessage.USER_IS_NOT_NEW_USER);
                }
            } else {
                productPriceVo.setPrice(activityInfo.getFinalPrice());
                productPriceVo.setCnyPrice(activityInfo.getFinalCnyPrice());
            }
        }
        return productPriceVo;
    }

    @Override
    public ProductVO getProduct(String productId) {
        ProductVO productVO = new ProductVO();
        BeanUtils.copyProperties(this.baseMapper.selectOne(new LambdaQueryWrapper<Product>().eq(Product::getId, productId)), productVO);
        return productVO;
    }

    @Override
    public ProductVO getProductByAppIdAndModel(String appId, Integer model) {
        List<Product> list = this.baseMapper.selectList(new LambdaQueryWrapper<Product>().eq(Product::getAppId, appId).eq(Product::getPlatform, model));
        ProductVO productVO = new ProductVO();
        BeanUtils.copyProperties(list.get(0),productVO);
        return productVO;
    }

    @Override
    public ProductVO getProductByCode(String code) {
        List<Product> list = this.baseMapper.selectList(new LambdaQueryWrapper<Product>().eq(Product::getCode, code));
        ProductVO productVO = new ProductVO();
        BeanUtils.copyProperties(list.get(0),productVO);
        return productVO;
    }

    @Override
    public List<ProductVO> getNewUserProduct(String appId) {
        List<ProductVO> productVOS = this.baseMapper.selectNewUserProductByAppId(appId);
        for (ProductVO productVO : productVOS) {
            productVO.setActivityInfo(activityService.getActivityInfoByActivityId(productVO.getActivityId()));
        }
        return productVOS;
    }

    @Override
    public ProductVO getProductForPay(String productId, Integer payment, Integer isDiscount) {
        ProductVO productVO = new ProductVO();
        BeanUtils.copyProperties(this.baseMapper.selectOne(new LambdaQueryWrapper<Product>().eq(Product::getId, productId)), productVO);
        RProductPay productPay = new RProductPay();
        productPay.setProductId(productId);
        productPay.setPayment(payment);
        productPay.setIsDiscount(isDiscount);
        List<RProductPay>  list = rProductPayService.list(productPay);
        productVO.setThirdPayId(list.get(0).getThirdPayId());
        return productVO;
    }

    @Override
    public ProductVO getProductByThirdPayId(String thirdPayId) {
        ProductVO productVO = this.baseMapper.getProductByThirdPayId(thirdPayId);
        if (ObjectUtils.isEmpty(productVO)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_THIRD_PAY_ID_ILLEGAL);
        }
        return productVO;
    }

    @Override
    public List<ParentProductVO> getProductListByPlatformId(String platformId) {
        return parentProductService.getProductListByPlatformId(platformId);
    }

    /*@Override
    public List<ProductListVO> getProductsList(Integer isEducation, Integer isTrial, Integer isActivity, String platformId, String userId, String code) {
        // 筛选订单购买的产品中有没有vip订阅类型
        boolean hasVip = false;
//        UserSubscriptionInfoVO vipSubscriptionInfoVO = null;
        boolean hasSVip = false;
//        UserSubscriptionInfoVO sVipSubscriptionInfoVO = null;
        // 买断版支持平台数
        int permanentPlatform = 0;
        // 判断用户是否登录，登录的用户获取已购买的权益
        UserInfoVO userVO = null;
        if (StringUtils.isNotEmpty(userId)) {
            userVO = userAPi.getMemberInfoById(userId).getResult();
            List<UserSubscriptionInfoVO> userSubscriptionInfoVOList = userVO.getSubscriptionInfoList().stream().filter(Objects::nonNull).filter(s -> s.getPlatform() != 0 && s.getStatus() != 2 && s.getStatus() != 4).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(userSubscriptionInfoVOList)) {
                for (UserSubscriptionInfoVO userSubscriptionInfoVO : userSubscriptionInfoVOList) {
                    // 买断的产品，判断是否多平台
                    if (userSubscriptionInfoVO.getPaymentModel().equals("2") && StringUtils.isNotEmpty(userSubscriptionInfoVO.getPlatforms())) {
                        if (userSubscriptionInfoVO.getPlatforms().contains("Mac")) permanentPlatform++;
                        if (userSubscriptionInfoVO.getPlatforms().contains("Windows")) permanentPlatform++;
                        if (userSubscriptionInfoVO.getPlatforms().contains("iOS")) permanentPlatform++;
                        if (userSubscriptionInfoVO.getPlatforms().contains("Android")) permanentPlatform++;
                    } else {
                        if (StringUtils.isNotEmpty(userSubscriptionInfoVO.getLevels())&&userSubscriptionInfoVO.getLevels().contains("2")) {
                            hasVip = true;
//                            vipSubscriptionInfoVO = userSubscriptionInfoVO;
                        } else if (StringUtils.isNotEmpty(userSubscriptionInfoVO.getLevels())&&userSubscriptionInfoVO.getLevels().contains("3")) {
                            hasSVip = true;
//                            sVipSubscriptionInfoVO = userSubscriptionInfoVO;
                        }
                    }
                }
            }
        }
        // 根据平台id获取所有父商品
        List<ParentProductVO> productList = parentProductService.getProductListByPlatformId(platformId);
        List<ProductListVO> productListVOList = new ArrayList<>();
        for (ParentProductVO parentProduct : productList) {
            ProductListVO productListVO = new ProductListVO();
            productListVO.setParentId(parentProduct.getId());
            productListVO.setParentProductCode(parentProduct.getCode());
            productListVO.setIsPossibleToPurchase(1);
            productListVO.setTypeOfPurchase(1);
            // 4单平台买断升级，5双平台买断升级，6标准订阅升级,教育买断升级
            String upgradeProductCode = null;
            // 根据用户的权益和需要购买的商品，先判断能不能升级
            String productCode = parentProduct.getCode();
            if (productCode.contains("高级版年订阅")) {
                if (hasVip) {
                    // 标准订阅升级
                    upgradeProductCode = "advanced-annual-subscription-standard-year-upgrade";
                    productListVO.setTypeOfPurchase(4);
                } else if (hasSVip) {
                    // 判断用户是否还在订阅中,订阅中的不能购买，取消订阅可以提前续费
                    if (userVO.getSubscriptionInfoList().stream().filter(Objects::nonNull)
                            .anyMatch(userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatform().equals(2) && userSubscriptionInfoVO.getPayType() == 1)) {
                        productListVO.setIsPossibleToPurchase(0);
                        productListVOList.add(productListVO);
                        continue;
                    } else {
                        productListVO.setTypeOfPurchase(2);
                    }
                } else if (permanentPlatform > 0) {
                    // 单平台买断升级
                    upgradeProductCode = "advanced-annual-subscription-single-upgrade";
                    if (permanentPlatform > 1) {
                        // 双平台买断升级
                        upgradeProductCode = "advanced-annual-subscription-multi-upgrade";
                    }
                    productListVO.setTypeOfPurchase(3);
                }
            } else if (productCode.contains("标准版年订阅")) {
                // vip提前续费
                if (hasVip) {
                    // 判断用户是否还在订阅中,
                    if (userVO.getSubscriptionInfoList().stream().filter(Objects::nonNull)
                            .anyMatch(userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatform().equals(2) && userSubscriptionInfoVO.getPayType() == 1)) {
                        productListVO.setIsPossibleToPurchase(0);
                        productListVOList.add(productListVO);
                        continue;
                    } else {
                        productListVO.setTypeOfPurchase(2);
                    }
                } else if (hasSVip) {
                    // svip无法购买vip
                    productListVO.setIsPossibleToPurchase(0);
                    continue;
                } else if (permanentPlatform > 0) {
                    // 买断无法购买vip
                    productListVO.setIsPossibleToPurchase(0);
                    continue;
                }
            }
            // 判断子产品优惠类型，0正价，1教育优惠，2活动，3试用，4升级，10试用+活动
            Integer discountType = 0;
            if (StringUtils.isEmpty(upgradeProductCode)) {
                // 如果不能升级，根据请求参数获取对应的优惠类型
                if (isActivity == 1 && isTrial == 0) {
                    discountType = 2;
                } else if (isActivity == 0 && isTrial == 1) {
                    discountType = 3;
                } else if (isActivity == 1 && isTrial == 1) {
                    discountType = 10;
                }
            } else {
                discountType = 4;
            }
            // 根据父商品id和优惠类型获取子商品
            List<ProductVO> productVOList = baseMapper.getProductByParentIdAndDiscountType(parentProduct.getId(), discountType);
            if (ObjectUtils.isEmpty(productVOList)) {
                productListVO.setIsPossibleToPurchase(0);
            } else {
                if (StringUtils.isEmpty(upgradeProductCode)) {
                    // 不是升级类型的子商品，应该只有一个
                    if (productVOList.size() > 1) {
                        log.error("父商品id为 {} 的 {} 类型子商品列表长度大于1", parentProduct.getId(), discountType);
                    }
                    BeanUtils.copyProperties(productVOList.get(0), productListVO);
                } else {
                    // 根据升级code找子商品
                    String finalUpgradeProductCode = upgradeProductCode;
                    productVOList.stream().filter(productVO -> productVO.getCode().equals(finalUpgradeProductCode))
                            .findFirst().ifPresent(productVO -> BeanUtils.copyProperties(productVO, productListVO));
                }
            }
            productListVOList.add(productListVO);
        }
        if (isEducation == 1) {
            // 额外获取每个父商品下的教育优惠商品
            for (ParentProductVO parentProduct : productList) {
                ProductListVO productListVO = new ProductListVO();
                productListVO.setParentId(parentProduct.getId());
                productListVO.setParentProductCode(parentProduct.getCode());
                productListVO.setIsPossibleToPurchase(1);
                productListVO.setTypeOfPurchase(1);
                List<ProductVO> productVOList = baseMapper.getProductByParentIdAndDiscountType(parentProduct.getId(), 1);
                if (ObjectUtils.isEmpty(productVOList)) {
                    productListVO.setIsPossibleToPurchase(0);
                } else {
                    BeanUtils.copyProperties(productVOList.get(0), productListVO);
                }
                productListVOList.add(productListVO);
            }
        }
        if (StringUtils.isNotEmpty(code) && "B89EF-166C-4AD".equals(code)) {
            // 运营 4.7.6 试用引导&退订挽回活动
            // 标准版年订阅 Coupon: $10 -> 69.99
            // 高级版年订阅 Coupon: $30 -> 69.99
            // 永久版      Coupon: $20 -> 99.99
            productListVOList.forEach(listingProductVO -> {
                BigDecimal couponPrice;
                BigDecimal cnyCouponPrice;
                if ("standard-annual-subscription".equals(listingProductVO.getCode())) {
                    couponPrice = new BigDecimal("10");
                    cnyCouponPrice = new BigDecimal("60");
                } else if ("advanced-annual-subscription".equals(listingProductVO.getCode())) {
                    couponPrice = new BigDecimal("30");
                    cnyCouponPrice = new BigDecimal("180");
                } else if ("advanced-permanent".equals(listingProductVO.getCode())) {
                    couponPrice = new BigDecimal("20");
                    cnyCouponPrice = new BigDecimal("120");
                } else {
                    return;
                }
                listingProductVO.setTypeOfPurchase(5);
                listingProductVO.setDisplayAmount(couponPrice);
                listingProductVO.setCnyDisplayAmount(cnyCouponPrice);
                listingProductVO.setDisplayPrice(null != listingProductVO.getDisplayPrice() ? listingProductVO.getDisplayPrice().subtract(couponPrice) : listingProductVO.getPrice().subtract(couponPrice));
                listingProductVO.setCnyDisplayPrice(null != listingProductVO.getCnyDisplayPrice() ? listingProductVO.getCnyDisplayPrice().subtract(cnyCouponPrice) : listingProductVO.getCnyPrice().subtract(cnyCouponPrice));
            });
        }
        return productListVOList;
    }*/

    @Override
    public List<ListingProductVO> getListingProducts(Integer isEducation, String platformId, String code) {
        // 获取上架中的产品，默认不查教育商品
        List<Integer> discountTypeList = new ArrayList<>();
        discountTypeList.add(0);
        if (isEducation == 1) {
            // 教育
            discountTypeList.add(1);
        }
        // 黑五
        discountTypeList.add(2);
        List<ListingProductVO> listingProducts = this.baseMapper.getProductsByPlatformIdAndDiscountType(platformId, discountTypeList);

        if (null != isEducation && isEducation == 2) {
            // 获取商品code在blackFiveProduct里的产品
            this.baseMapper.selectList(new LambdaQueryWrapper<Product>().in(Product::getCode, blackFiveProduct)).forEach(product -> {
                ListingProductVO listingProductVO = new ListingProductVO();
                BeanUtils.copyProperties(product, listingProductVO);

                String productCode = listingProductVO.getCode();
                if (productCode.endsWith("blackFive")) {
                    // 获取截取后的 code
                    String trimmedCode = productCode.replace("-blackFive", "");
                    // 替换 listingProducts 中对应的产品
                    listingProducts.removeIf(existingProduct -> existingProduct.getCode().equals(trimmedCode));
                    listingProducts.add(listingProductVO);
                } else {
                    listingProducts.add(listingProductVO);
                }
            });
        }
        if (StringUtils.isNotEmpty(code) && "B89EF-166C-4AD".equals(code)) {
            // 运营 4.7.6 试用引导&退订挽回活动
            // 标准版年订阅 Coupon: $10 -> 69.99
            // 高级版年订阅 Coupon: $30 -> 69.99
            // 永久版      Coupon: $20 -> 99.99
            listingProducts.forEach(listingProductVO -> {
                BigDecimal couponPrice;
                BigDecimal cnyCouponPrice;
                if ("standard-annual-subscription".equals(listingProductVO.getCode())) {
                    couponPrice = new BigDecimal("10");
                    cnyCouponPrice = new BigDecimal("60");
                } else if ("advanced-annual-subscription".equals(listingProductVO.getCode())) {
                    couponPrice = new BigDecimal("30");
                    cnyCouponPrice = new BigDecimal("180");
                } else if ("advanced-permanent".equals(listingProductVO.getCode())) {
                    couponPrice = new BigDecimal("20");
                    cnyCouponPrice = new BigDecimal("120");
                } else {
                    return;
                }
                listingProductVO.setDisplayAmount(couponPrice);
                listingProductVO.setCnyDisplayAmount(cnyCouponPrice);
                listingProductVO.setDisplayPrice(null != listingProductVO.getDisplayPrice() ? listingProductVO.getDisplayPrice().subtract(couponPrice) : listingProductVO.getPrice().subtract(couponPrice));
                listingProductVO.setCnyDisplayPrice(null != listingProductVO.getCnyDisplayPrice() ? listingProductVO.getCnyDisplayPrice().subtract(cnyCouponPrice) : listingProductVO.getCnyPrice().subtract(cnyCouponPrice));
            });
        }
        return listingProducts;
    }


    @Override
    public List<ListingProductVO> getProductsByIds(List<String> productIds) {
        return this.baseMapper.getProductsByIds(productIds);
    }

    @Override
    public ListingProductVO getProductsByCode(String code) {
        return this.baseMapper.getProductsByCode(code);
    }

    @Override
    public List<ProductVO> getProductByParentIdAndDiscountType(String parentId, Integer discountType) {
        return this.baseMapper.getProductByParentIdAndDiscountType(parentId, discountType);
    }

    /**
     * 验证用户邮箱教育优惠资格
     * @param email 用户邮箱
     * @return
     */
    @Override
    public Integer checkEmail(String email) {
        return eduEmailsService.checkEmail(email);
    }

    /*public ListingProductVO getProductPriceForBuy(String productId, String userId, Integer isEducation) {
        // 查询购买的商品信息
        List<ListingProductVO> productVOs = this.baseMapper.getProductsWithPlatformId(null, Collections.singletonList(productId));
        ListingProductVO listingProductVO = productVOs.get(0);
        // 查询用户+所有订单
        if (StringUtils.isEmpty(userId)) {
            return listingProductVO;
        }
        UserVO userVO = userAPi.getById(userId).getResult();
        if (null == userVO) {
            log.error("用户不存在，id：{}", userId);
            return listingProductVO;
        }
        List<OrdersVO> orders = orderApi.getOrderByUserId(userId).getResult();
        if (null != isEducation && isEducation == 1) {
            // 判断用户教育优惠资格
            // 根据用户邮箱
            Integer checkEmail = eduEmailsService.checkEmail(userVO.getEmail());
            if (checkEmail == 0) {
                throw new BackendRuntimeException(ErrorMessage.USER_IS_NOT_EDUCATION_USER);
            }
            // 查询用户所有订单。判断近一年有没有教育优惠订单
            if (orders.stream().anyMatch(ordersVO -> ordersVO.getIsEducation() == 1 && ordersVO.getCreatedAt().isAfter(LocalDateTime.now().minusYears(1)))) {
                throw new BackendRuntimeException(ErrorMessage.USER_HAS_EDUCATION);
            }
            // 校验教育用户资格通过
            listingProductVO.setDisplayPrice(ProductEducationPriceEnum.getByPrice(listingProductVO.getPrice()).getDisplayPrice());
            // 没有购买过的用户直接返回
            if (CollectionUtils.isEmpty(orders)) {
                return listingProductVO;
            }
            // 获取已购买的订单商品及会员权益
            List<String> orderProductIds = orders.stream().map(OrdersVO::getProductId).collect(Collectors.toList());
            List<ListingProductVO> orderProducts = this.baseMapper.getProductsWithPlatformId(null, orderProductIds);
            if (CollectionUtils.isEmpty(orderProducts)) {
                return listingProductVO;
            }
            // 筛选订单购买的产品中有没有vip订阅类型
            boolean hasVip = false;
            boolean hasSVip = false;
            boolean hasPermanent = false;
            for (ListingProductVO orderProduct : orderProducts) {
                if (orderProduct.getPaymentModel() == 1) {
                    hasPermanent = true;
                } else {
                    if (orderProduct.getLevels().contains("2")) {
                        hasVip = true;
                    } else if (orderProduct.getLevels().contains("3")) {
                        hasSVip = true;
                    }
                }
            }
            // 购买svip，判断是否买断升级订阅（买且只买了永久版，购买高级版订阅）
            if (listingProductVO.getLevels().contains("3")) {
                // vip无法购买svip
                if (hasVip) {
                    throw new BackendRuntimeException(ErrorMessage.USER_CANNOT_BUY_PREMIUM);
                }
                // svip原价提前续费
                if (hasSVip) {
                    return listingProductVO;
                }
                // 买断升级订阅价格
                if (hasPermanent) {
                    listingProductVO.setUpgradePrice(new BigDecimal("9.99"));
                    return listingProductVO;
                }
            }
            // 购买vip
            if (listingProductVO.getLevels().contains("2")) {
                // vip提前续费
                if (hasVip) {
                    return listingProductVO;
                }
                // svip无法购买vip
                if (hasSVip) {
                    throw new BackendRuntimeException(ErrorMessage.USER_CANNOT_BUY_VIP);
                }
                // 买断无法购买vip
                if (hasPermanent) {
                    throw new BackendRuntimeException(ErrorMessage.USER_CANNOT_UPGRADE_VIP);
                }
            }
            return listingProductVO;
        } else {
            // 没有购买过的用户直接返回
            if (CollectionUtils.isEmpty(orders)) {
                return listingProductVO;
            }
            // 获取已购买的订单商品及会员权益
            List<String> orderProductIds = orders.stream().map(OrdersVO::getProductId).collect(Collectors.toList());
            List<ListingProductVO> orderProducts = this.baseMapper.getProductsWithPlatformId(null, orderProductIds);
            if (CollectionUtils.isEmpty(orderProducts)) {
                return listingProductVO;
            }
            // 筛选订单购买的产品中有没有vip订阅类型
            boolean hasVip = false;
            boolean hasSVip = false;
            boolean hasPermanent = false;
            for (ListingProductVO orderProduct : orderProducts) {
                if (orderProduct.getPaymentModel() == 1) {
                    hasPermanent = true;
                } else {
                    if (orderProduct.getLevels().contains("2")) {
                        hasVip = true;
                    } else if (orderProduct.getLevels().contains("3")) {
                        hasSVip = true;
                    }
                }
            }
            // 购买svip，判断是否买断升级订阅（买且只买了永久版，购买高级版订阅）
            if (listingProductVO.getLevels().contains("3")) {
                // vip无法购买svip
                if (hasVip) {
                    throw new BackendRuntimeException(ErrorMessage.USER_CANNOT_BUY_PREMIUM);
                }
                // svip原价提前续费
                if (hasSVip) {
                    return listingProductVO;
                }
                // 买断升级订阅价格
                if (hasPermanent) {
                    listingProductVO.setUpgradePrice(new BigDecimal("19.99"));
                    return listingProductVO;
                }
            }
            // 购买vip
            if (listingProductVO.getLevels().contains("2")) {
                // vip原价提前续费
                if (hasVip) {
                    return listingProductVO;
                }
                // svip无法购买vip
                if (hasSVip) {
                    throw new BackendRuntimeException(ErrorMessage.USER_CANNOT_BUY_VIP);
                }
                // 买断无法购买vip
                if (hasPermanent) {
                    throw new BackendRuntimeException(ErrorMessage.USER_CANNOT_UPGRADE_VIP);
                }
            }
            return listingProductVO;
        }
    }*/

    /**
     * 批量获取产品价格
     * @return
     */
    @Override
    public BigDecimal getBatchProductPrice(String productId, Integer num) {
        Product productVO = baseMapper.selectById(productId);
        if (productVO == null) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRODUCT_NOT_EXIST);
        }
        // 2-5	10% OFF	$107.99/个
        //6-20	20% OFF	$95.99/个
        //21-50	30% OFF	$83.99/个
        //50+	联系我们获取最新折扣
        if (num <= 1) {
            return productVO.getPrice();
        } else if (num <= 5) {
            return new BigDecimal("107.99").multiply(new BigDecimal(num));
        } else if (num <= 20) {
            return new BigDecimal("95.99").multiply(new BigDecimal(num));
        } else if (num <= 50) {
            return new BigDecimal("83.99").multiply(new BigDecimal(num));
        } else {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_CONTACT_US);
        }
    }

    @Override
    public ListingProductVO checkCoupon(String productId, String userId, String code) {
        List<ListingProductVO> productVOs = this.baseMapper.getProductsByIds(Collections.singletonList(productId));
        ListingProductVO listingProductVO = productVOs.get(0);
        if (ObjectUtils.isEmpty(listingProductVO)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRODUCT_NOT_EXIST);
        }
        log.info("检查优惠券，产品code:{}", listingProductVO.getCode());
        if ("4DDF2-6860-49C".equals(code)) {
            // 运营官网活动
            // 全平台高级版年订阅，40美元
            // Mac&Windows双平台高级版永久 和 标准版 30 美元
            List<String> activityProduct = new ArrayList<>();
            activityProduct.add("advanced-annual-subscription");
            List<String> activityProduct1 = new ArrayList<>();
            activityProduct1.add("advanced-permanent");
            activityProduct1.add("standard-annual-subscription");
            if (activityProduct.contains(listingProductVO.getCode()) || activityProduct1.contains(listingProductVO.getCode())) {
                if (activityProduct.contains(listingProductVO.getCode())) {
                    listingProductVO.setDisplayPrice(null != listingProductVO.getDisplayPrice() ? listingProductVO.getDisplayPrice().subtract(new BigDecimal("40")) : listingProductVO.getPrice().subtract(new BigDecimal("40")));
                    listingProductVO.setCnyDisplayPrice(null != listingProductVO.getCnyDisplayPrice() ? listingProductVO.getCnyDisplayPrice().subtract(new BigDecimal("261")) : listingProductVO.getCnyPrice().subtract(new BigDecimal("261")));
                } else {
                    listingProductVO.setDisplayPrice(null != listingProductVO.getDisplayPrice() ? listingProductVO.getDisplayPrice().subtract(new BigDecimal("30")) : listingProductVO.getPrice().subtract(new BigDecimal("30")));
                    listingProductVO.setCnyDisplayPrice(null != listingProductVO.getCnyDisplayPrice() ? listingProductVO.getCnyDisplayPrice().subtract(new BigDecimal("190")) : listingProductVO.getCnyPrice().subtract(new BigDecimal("190")));
                }
            } else {
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_PRODUCT_ERROR);
            }
        } else if ("B89EF-166C-4AD".equals(code)) {
            // 运营 4.7.6 试用引导&退订挽回活动
            // 标准版年订阅 Coupon: $10 -> 69.99
            // 高级版年订阅 Coupon: $30 -> 69.99
            // 永久版      Coupon: $20 -> 99.99
            BigDecimal couponPrice;
            BigDecimal cnyCouponPrice;
            if ("standard-annual-subscription".equals(listingProductVO.getCode())) {
                couponPrice = new BigDecimal("10");
                cnyCouponPrice = new BigDecimal("60");
            } else if ("advanced-annual-subscription".equals(listingProductVO.getCode())) {
                couponPrice = new BigDecimal("30");
                cnyCouponPrice = new BigDecimal("180");
            } else if ("advanced-permanent".equals(listingProductVO.getCode())) {
                couponPrice = new BigDecimal("20");
                cnyCouponPrice = new BigDecimal("120");
            } else {
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_PRODUCT_ERROR);
            }
            listingProductVO.setDisplayPrice(null != listingProductVO.getDisplayPrice() ? listingProductVO.getDisplayPrice().subtract(couponPrice) : listingProductVO.getPrice().subtract(couponPrice));
            listingProductVO.setCnyDisplayPrice(null != listingProductVO.getCnyDisplayPrice() ? listingProductVO.getCnyDisplayPrice().subtract(cnyCouponPrice) : listingProductVO.getCnyPrice().subtract(cnyCouponPrice));
        } else {
            Prize prize = prizeService.getOne(new LambdaQueryWrapper<Prize>()
                    .eq(Prize::getCode, code).eq(Prize::getValidFlag, 1));
            if (prize == null) {
                log.error("优惠券不存在:{}", code);
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_NOT_EXISTS);
            }
            if (prize.getType() != 1) {
                log.error("优惠券类型错误:{}", code);
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_TYPE_ERROR);
            }
            if (StringUtils.isNotEmpty(prize.getProductId()) && !prize.getProductId().contains(productId)) {
                log.error("优惠券关联产品错误，productId:{}", productId);
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_PRODUCT_ERROR);
            }
            if (prize.getValidFlag() != 1) {
                log.error("优惠券不可用:{}", code);
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_INVALID);
            }
            if (prize.getEndDate().before(new Date()) || prize.getStartDate().after(new Date())) {
                log.error("优惠券时间不可用:{}", code);
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_INVALID);
            }
            if(ObjectUtils.isNotEmpty(listingProductVO.getIsAi())&&listingProductVO.getIsAi().equals(1)){
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_PRODUCT_ERROR);
            }
            if(ObjectUtils.isNotEmpty(listingProductVO.getDisplayPrice())){
                if (prize.getActivityType().equals("1")){
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_PRODUCT_ERROR);
                }
                listingProductVO.setDisplayPrice(listingProductVO.getDisplayPrice().subtract(prize.getUsdPrice()));
                listingProductVO.setCnyDisplayPrice(listingProductVO.getCnyDisplayPrice().subtract(prize.getCnyPrice()));
            }else{
                if ("1".equals(prize.getActivityType())) {
                    if (listingProductVO.getDiscountType()==1) {
                        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_PRODUCT_ERROR);
                    }
                    if (listingProductVO.getPrice().compareTo(BigDecimal.valueOf(79.99)) < 0) {
                        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_PRODUCT_ERROR);
                    }
                }
                listingProductVO.setDisplayPrice(listingProductVO.getPrice().subtract(prize.getUsdPrice()));
                listingProductVO.setCnyDisplayPrice(listingProductVO.getCnyPrice().subtract(prize.getCnyPrice()));
            }
        }
        return listingProductVO;
    }

    @Override
    public Boolean updateCouponState(String code, Integer state, String orderId) {
        return prizeService.update(new Prize(), new LambdaUpdateWrapper<Prize>()
                .set(Prize::getValidFlag, state)
                .set(Prize::getOrderId, orderId)
                .eq(Prize::getCode, code));
    }

    @Override
    public Boolean refundCouponByOrderId(String orderId) {
        return prizeService.update(new Prize(), new LambdaUpdateWrapper<Prize>()
                .set(Prize::getValidFlag, 1).eq(Prize::getOrderId, orderId));
    }
}
