package cn.kdan.cloud.pdf.office.sso.controller;

import cn.kdan.cloud.pdf.office.api.account.feign.LoginDeviceApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserSubscriptionInfoApi;
import cn.kdan.cloud.pdf.office.api.account.vo.LoginDevice;
import cn.kdan.cloud.pdf.office.api.account.vo.UserSubscriptionInfo;
import cn.kdan.cloud.pdf.office.api.email.bo.EmailSendBO;
import cn.kdan.cloud.pdf.office.api.email.feign.EmailApi;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.RightBindLogDTO;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.feign.OrderApi;
import cn.kdan.cloud.pdf.office.api.payment.feign.RightBindLogApi;
import cn.kdan.cloud.pdf.office.api.payment.feign.SubscriptionApi;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.api.product.feign.LoginLogApi;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.dto.LicenseCodeResult;
import cn.kdan.cloud.pdf.office.common.enums.*;
import cn.kdan.cloud.pdf.office.common.enums.account.AccountTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PayTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.payment.SubscriptionTypeEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.common.utils.PDFReaderUtils;
import cn.kdan.cloud.pdf.office.common.utils.RedisUtils;
import cn.kdan.cloud.pdf.office.common.vo.*;
import cn.kdan.cloud.pdf.office.sso.constant.AuthConstant;
import cn.kdan.cloud.pdf.office.sso.enums.FeedbackType;
import cn.kdan.cloud.pdf.office.sso.pojo.CancelReasonDTO;
import cn.kdan.cloud.pdf.office.sso.pojo.InfoParam;
import cn.kdan.cloud.pdf.office.sso.pojo.LogOutSystemDTO;
import cn.kdan.cloud.pdf.office.sso.service.AuthService;
import cn.kdan.cloud.pdf.office.sso.utils.RSAUtils;
import com.alibaba.nacos.common.utils.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.authentication.OAuth2AuthenticationDetails;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

/**
 * 个人信息模块
 */
@RestController
@RequestMapping("/user")
public class UserController {

    private final RSAUtils rsaUtils = new RSAUtils();
    @Autowired
    private UserApi userApi;
    @Autowired
    private ProductApi productApi;
    @Autowired
    private LoginLogApi loginLogApi;
    @Autowired
    private SubscriptionApi subscriptionApi;
    @Autowired
    private RightBindLogApi rightBindLogApi;
    @Autowired
    private UserSubscriptionInfoApi userSubscriptionInfoApi;
    @Autowired
    private OrderApi orderApi;
    @Autowired
    private AuthService authService;
    @Value("${avatarUrl}")
    private String avatarUrl;
    @Autowired
    private EmailApi emailApi;
    @Value("${pdfReaderUrl:123/}")
    private String pdfReaderUrl;
    private static final ThreadPoolExecutor PAY_UPDATE_THREAD_POOL = new ThreadPoolExecutor(Runtime.getRuntime().availableProcessors(),
            Runtime.getRuntime().availableProcessors(),
            1L,
            TimeUnit.MINUTES,
            new LinkedBlockingQueue<>(300), new ThreadFactoryBuilder()
            .setNameFormat("PAY_NOTIFY_THREAD_POOL").build(), new ThreadPoolExecutor.AbortPolicy());
    @Autowired
    private RedisUtils<String, String> redisUtils;
    @Autowired
    private LoginDeviceApi loginDeviceApi;

    public static String convertToLowerCase(String input) {
        if (input == null || input.isEmpty()) {
            return ""; // 如果输入为空，返回空字符串
        }

        // 分割字符串
        String[] platforms = input.split(",");

        // 将平台名称转换为小写
        StringBuilder lowerCasePlatforms = new StringBuilder();
        for (int i = 0; i < platforms.length; i++) {
            lowerCasePlatforms.append(platforms[i].toLowerCase()); // 转换为小写
            if (i < platforms.length - 1) {
                lowerCasePlatforms.append(","); // 添加逗号分隔符
            }
        }

        return lowerCasePlatforms.toString(); // 返回结果字符串
    }

    @GetMapping("/me")
    public UserInfoVO me(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            {
                String userId = ((CustomUserDetails) ((OAuth2Authentication) principal).getPrincipal()).getId();
                authService.refreshToken(principal);
                // 获取用户的基本信息
                UserInfoVO user = userApi.getInfoById(userId).getResult();
                user.getSubscriptionInfoList().forEach(item -> {
                    ProductVO vo = productApi.getProduct(item.getProductId()).getResult();
                    item.setProductCode(vo.getCode());
                });
                List<UserSubscriptionInfoVO> list = user.getSubscriptionInfoList().stream()
                        .sorted(Comparator.comparing(UserSubscriptionInfoVO::getPlatform, Comparator.reverseOrder()))
                        .collect(Collectors.toList());
                list.forEach(item -> {
                    //自动续订，显示续订日期
                    if (!ObjectUtils.isEmpty(item.getEndDate()) && PayTypeEnum.AUTO.value().equals(item.getPayType())) {
                        LocalDateTime payTime = subscriptionApi.getNextPayTime(item.getUserId(), item.getProductId()).getResult();
                        Date date = MyDateUtils.localDateTimeToDate(payTime);
                        item.setEndDate(date);
                    }
                    // 没有购买显示当前时间
                    if (ObjectUtils.isEmpty(item.getEndDate())) {
                        item.setEndDate(new Date());
                    }
                });
                user.setSubscriptionInfoList(list);
                user.setDigestPassword(null);
                return user;
            }
        }
        return null;
    }

    /**
     * 获取用户个人信息
     *
     * @return info
     */
    @GetMapping("/info")
    public ResultMap<MemberInfoVO> info(Principal principal,@RequestParam(required = false) String model) {
        if (principal instanceof OAuth2Authentication) {
            {
                String userId = ((CustomUserDetails) ((OAuth2Authentication) principal).getPrincipal()).getId();
                authService.refreshToken(principal);
                // 获取用户的基本信息
                UserInfoVO user = userApi.getMemberInfoById(userId).getResult();
                // 没有购买显示当前时间
                if (ObjectUtils.isEmpty(user.getFullName())) {
                    user.setFullName("");
                }
                MemberInfoVO result = new MemberInfoVO();
                BeanUtils.copyProperties(user, result);
                if (!StringUtils.isEmpty(user.getDigestPassword())) {
                    result.setIsHavePwd(ValidStatusEnum.VALID.value());
                }
                if (subscriptionApi.judgeAiDiscount(user.getId()).getResult()) {
                    result.setIsHaveAIDiscount(ValidStatusEnum.VALID.value());
                } else {
                    result.setIsHaveAIDiscount(ValidStatusEnum.INVALID.value());
                }
                if (subscriptionApi.judgeAiTrail(user.getId()).getResult()) {
                    result.setIsAlreadyAITrail(ValidStatusEnum.VALID.value());
                } else {
                    result.setIsAlreadyAITrail(ValidStatusEnum.INVALID.value());
                }
                if (subscriptionApi.judgeTrail(user.getId()).getResult()) {
                    result.setCanTrail(ValidStatusEnum.VALID.value());
                } else {
                    result.setCanTrail(ValidStatusEnum.INVALID.value());
                }
                UserSubscriptionInfoVO activeSubscription = authService.findActiveSubscription(user.getSubscriptionInfoList(), null);
                SimpleUserSubscriptionInfoVO vo = new SimpleUserSubscriptionInfoVO();
                 // 没有购买显示当前时间
                if (ObjectUtils.isEmpty(activeSubscription.getEndDate())) {
                    activeSubscription.setEndDate(new Date());
                }
                if (ObjectUtils.isEmpty(activeSubscription.getPoint())) {
                    activeSubscription.setPoint(0);
                }
                //默认不是ai
                activeSubscription.setIsAi(StringUtils.isEmpty(activeSubscription.getIsAi()) ? "0" : activeSubscription.getIsAi());
                BeanUtils.copyProperties(activeSubscription,vo);
                vo.setIsTrail(ValidStatusEnum.INVALID.value());
                vo.setTrailDays(0);
                if(vo.getStatus().equals(1)&&vo.getCode().contains("trail")){
                    SubscriptionsVO subscriptionsVO =  subscriptionApi.getSubscriptions(userId,2).getResult();
                    //如果结束时间在今天之后，并且距离开始时间不超过8天，设置为试用
                    if(subscriptionsVO.getEndDate().after(new Date())&&MyDateUtils.daysBetween(subscriptionsVO.getStartDate(),subscriptionsVO.getEndDate())<=8){
                        vo.setIsTrail(ValidStatusEnum.VALID.value());
                        vo.setTrailDays(loginLogApi.getLoginDaysByUserId(userId).getResult());
                    }
                }
                result.setActiveVIP(vo);
                vo.setPlatforms(convertToLowerCase(vo.getPlatforms()));
                if(activeSubscription.getStatus().equals(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value())
                        &&activeSubscription.getPlatform().equals(2)&&activeSubscription.getPayType().equals(PayTypeEnum.AUTO.value())) {
                    PAY_UPDATE_THREAD_POOL.submit(() -> {
                        orderApi.updateStatusPayment(userId, activeSubscription.getProductId());
                    });
                }
                result.setAiPoint(authService.getAiPoints(user.getSubscriptionInfoList()));
                SimpleUserSubscriptionInfoVO aiInfo = new SimpleUserSubscriptionInfoVO();
                UserSubscriptionInfoVO infoVO = authService.getActiveAi(user.getSubscriptionInfoList());
                BeanUtils.copyProperties(infoVO, aiInfo);
                result.setActiveAI(aiInfo);
                result.setAvatarUrl(avatarUrl);
                result.setIsHaveActiveSubscription(authService.judgeActiveSubscription(user.getSubscriptionInfoList()));
                //如果只有一条订阅并且该条订阅是免费，则为免费账户
                result.setIsFreeAccount(user.getSubscriptionInfoList().size()==1&&user.getSubscriptionInfoList().get(0).getLevels().equals("1") ? "1":"0");
                if (StringUtils.isNotEmpty(model)&&(model.equals(ModelEnum.PC_OFFICIAL.getEnglishDescription())||
                        model.equals(ModelEnum.MOBILE_OFFICIAL.getEnglishDescription()))) {
                    result.setIsShowGetAiButton(getIsShowGetAiButton(activeSubscription,user.getSubscriptionInfoList(),
                            user.getEmail(),result.getIsFreeAccount(),result.getIsHaveAIDiscount()));
                }

                Integer deviceNum = activeSubscription.getMaxDeviceNum();
                List<LoginDevice> devices = loginDeviceApi.getByUserId(userId, "1").getResult();
                // 如果设备数超过最大限制，处理设备登录情况
                if (CollectionUtils.isNotEmpty(devices) && devices.size() > deviceNum) {
                    handleMaxDevice(principal, userId, devices);
                }
                result.setUserSubscriptionStatus(getUserSubscriptionStatus(activeSubscription));
                result.setActiveButton(getActiveButton(user.getEmail(),result.getIsFreeAccount(),activeSubscription));
                //如果是订阅的会员则显示自动扣费平台

                return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, result);
            }
        }

        throw new BackendRuntimeException("net error");
    }

    @GetMapping("/judgeTrailCancel")
    public ResultMap<Boolean> judgeTrailCancel(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            {
                String userId = ((CustomUserDetails) ((OAuth2Authentication) principal).getPrincipal()).getId();
                authService.refreshToken(principal);
                // 获取用户的基本信息
                UserInfoVO user = userApi.getMemberInfoById(userId).getResult();
                UserSubscriptionInfoVO activeSubscription = authService.findActiveSubscription(user.getSubscriptionInfoList(), null);
                Boolean judgePremium = CommonUtils.judgeSubscription(user.getSubscriptionInfoList(),1);
                //没有永久版没有买ai只试用退订了高级版年订阅
                if(!judgePremium&&activeSubscription.getPayType().equals(PayTypeEnum.NO_AUTO.value())&&activeSubscription.getLevels().equals("3")&&activeSubscription.getCycle().equals(4)&&activeSubscription.getPaymentModel().equals("1")
                        &&activeSubscription.getCode().contains("trail")){
                    SubscriptionsVO subscriptionsVO =  subscriptionApi.getLatestSubscriptionForActualPayment(userId,activeSubscription.getProductId()).getResult();
                    //如果结束时间在今天之后，并且距离开始时间不超过8天，设置为试用
                    if(MyDateUtils.daysBetween(subscriptionsVO.getStartDate(),subscriptionsVO.getEndDate())<=8){
                        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
                    }
                }
                return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.FALSE);
            }
        }

        throw new BackendRuntimeException("net error");
    }

    /**
     * 登录后判断是否权益转移
     * @param param param
     * @return String 1登录完成：没有购买过license的老用户，购买过多个license的用户（app端解绑时对应解绑成功）
     * 2跳转到pricing页面（官网）/比较表（app端）：没有购买过license的新用户（app端解绑时对应解绑成功）
     * 3弹出license权益转移询问弹窗： 只购买过1个license的用户，并且license未转移
     * （官网登录根据邮箱判断，dmg或者exe登录根据设备id和邮箱来判断，mac付费版不用判断）
     */
    @PostMapping("judgeUserLicenseStatus")
    public ResultMap<String> judgeUserLicenseStatus(Principal principal,@RequestBody(required = false) InfoParam param) {
        String userId = ((CustomUserDetails) ((OAuth2Authentication) principal).getPrincipal()).getId();
        // 获取用户的基本信息
        UserInfoVO user = userApi.getMemberInfoById(userId).getResult();
        String isFreeAccount = user.getSubscriptionInfoList().size()==1&&user.getSubscriptionInfoList().get(0).getLevels().equals("1") ? "1":"0";
        boolean isHaveActiveSubscription = authService.judgeActiveSubscription(user.getSubscriptionInfoList());
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, getUserLicenseStatus(param.getDeviceSign(), user.getEmail(), user.getId(), param.getModelDetail(),isFreeAccount,param.getAppleTransactionIds(),param.getAppleBundleId(),isHaveActiveSubscription));
    }


    private String getActiveButton(String email,String isFreeAccount,UserSubscriptionInfoVO activeSubscription){
        /*
         购买和试用（仅账号下未试用过的用户有试用按钮）
         */
        List<LicenseCodeResult> licenseCodes = PDFReaderUtils.getLicenseByEmail(pdfReaderUrl,email);
        //无历史购买
        if (licenseCodes.isEmpty()) {
            //如果是免费账户
            if(isFreeAccount.equals(ValidStatusEnum.VALID.value())){
               return "buy";
            }else if(activeSubscription.getStatus().equals(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_EXPIRED.value())){
                return "prepay";
            }
            else {
                return CardButtonEnum.getButton(activeSubscription.getCode(),convertStatusForCard(activeSubscription));
            }
        }
        //有且仅买过一个标准或高级license
        if(licenseCodes.size()==1){
            if(isFreeAccount.equals(ValidStatusEnum.VALID.value())||activeSubscription.getStatus().equals(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_EXPIRED.value())){
                //如果是免费账户根据code换算成会员系统的产品权益
                //根据权益判断卡片按钮类型
                //永久版status是1
                return CardButtonEnum.getButton(CodeConvertEnum.getNewCodeByOldCode(licenseCodes.get(0).getCode()),"1");
            } else {
                //转换旧产品的权益
                //根据权益对比得出有效权益
                //根据有效权益判断按钮
                UserSubscriptionInfoVO  infoVO = convertProduct(licenseCodes.get(0).getCode(),activeSubscription);
                //如果是永久版，status为1
                if(infoVO.getPlatform().equals(1)){
                    return CardButtonEnum.getButton(infoVO.getCode(),"1");
                }else {
                    return CardButtonEnum.getButton(infoVO.getCode(),convertStatusForCard(infoVO));
                }
            }
        }
        return "buy";
    }

    private UserSubscriptionInfoVO convertProduct(String code,UserSubscriptionInfoVO activeSubscription) {
        UserSubscriptionInfoVO vo = userSubscriptionInfoApi.getByOldProductCode(code).getResult();
        List<UserSubscriptionInfoVO> voList = new ArrayList<>();
        voList.add(vo);
        voList.add(activeSubscription);
        return authService.findActiveSubscription(voList, null);
    }

    private String getUserSubscriptionStatus(UserSubscriptionInfoVO activeSubscription) {
        //0免费
        //1永久
        //2订阅
        //3订阅过期
        if (activeSubscription.getPlatform().equals(0)) {
            return "0";
        } else if (activeSubscription.getPlatform().equals(1)) {
            return "1";
        } else if (activeSubscription.getPlatform().equals(2)&&activeSubscription.getStatus().equals(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value())) {
            return "2";
        } else if (activeSubscription.getPlatform().equals(2)&&activeSubscription.getStatus().equals(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_EXPIRED.value())) {
            return "3";
        }
        return "0";
    }
    private String convertStatusForCard(UserSubscriptionInfoVO activeSubscription) {
        //0免费
        //1永久
        //2订阅
        //3订阅中但是取消了续订
        if (activeSubscription.getPlatform().equals(0)) {
            return "0";
        } else if (activeSubscription.getPlatform().equals(1)) {
            return "1";
        } else if (activeSubscription.getPlatform().equals(2)
                &&activeSubscription.getStatus().equals(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value())
        &&activeSubscription.getPayType().equals(1)) {
            return "2";
        } else if (activeSubscription.getPlatform().equals(2)&&
                activeSubscription.getStatus().equals(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value())
                &&activeSubscription.getPayType().equals(0)) {
            return "3";
        }
        return "0";
    }

    private String getIsShowGetAiButton(UserSubscriptionInfoVO activeSubscription, List<UserSubscriptionInfoVO> subscriptionInfoList, String email, String isFreeAccount, String isHaveAIDiscount) {
        //有一年AI可以领但还没领的用户
        if(isHaveAIDiscount.equals("1")){
            return "3";
        }
        List<LicenseCodeResult> licenseCodes = PDFReaderUtils.getLicenseByEmail(pdfReaderUrl,email);
          //当前只有一个永久会员（没有订阅会员）并且没有买过序列码
        if (activeSubscription.getPlatform().equals(1) && licenseCodes.isEmpty()) {
              String result = "1";
              for (UserSubscriptionInfoVO infoVO : subscriptionInfoList) {
                  if (infoVO.getPlatform().equals(2)) {
                      result = "0";
                      break;
                  }
              }
              return result;
          }
        //注册邮箱下有且仅购买过1个license，且账号下没有Pro会员权益
        if(licenseCodes.size()==1){
            //有且仅购买过1个license，且账号下没有永久Pro会员权益
            String result = "2";
            for (UserSubscriptionInfoVO infoVO : subscriptionInfoList) {
                if (infoVO.getPlatform().equals(1)) {
                    result = "0";
                    break;
                }
            }
            return result;
        }
        //如果用户是免费用户
        if(isFreeAccount.equals("1")){
            //展示0元领取1年AI 按钮，但是是跳转到price页面
            return "1";
        }
        return "0";
    }

    public String getUserLicenseStatus(String deviceSign,String email,String userId,String model,String isFreeAccount,List<String> appTransactionIds,String appBundleId,Boolean isHaveActiveSubscription) {
        //官网
        if (model.equals(ModelEnum.PC_OFFICIAL.getEnglishDescription())||model.equals(ModelEnum.MOBILE_OFFICIAL.getEnglishDescription())){
            //该邮箱没有购买过
            List<LicenseCodeResult> licenseCodes = PDFReaderUtils.getLicenseByEmail(pdfReaderUrl,email);
            if(CollectionUtils.isEmpty(licenseCodes)){
                List<OrdersVO> list = orderApi.getOrderListByStatus(3,userId).getResult();
                if (CollectionUtils.isEmpty(list)&&isFreeAccount.equals("1")){
                    //如果是没有过购买的新用户账号跳转pricing页面
                    //用户有没有过购买的订单
                    return "2";
                }
                //有过购买的老用户登录成功
                return "1";
            }else {
                //如果用户买了不只一个序列码，登录成功
                if(licenseCodes.size()>1){
                    //如果用户买了不只一个序列码，登录成功
                    return "1";
                }
                //如果用户只购买了一个，则弹出权益转移弹窗
                return "3";
            }
        }
        //dmg或者exe
        if (model.contains("DMG")||model.contains("EXE")) {
            List<LicenseCodeResult> licenseCodes = getLicenseByDevice(deviceSign);
            if (CollectionUtils.isEmpty(licenseCodes)) {
                //如果本机没有绑定序列码
                List<OrdersVO> list = orderApi.getOrderListByStatus(3,userId).getResult();
                if (CollectionUtils.isEmpty(list)&&isFreeAccount.equals("1")){
                    //如果是没有过购买的新用户账号跳转比较表页面
                    return "2";
                }
                //登录成功
                return "1";
            }else{
               LicenseCodeResult licenseCodeResult = licenseCodes.get(0);
               //批量购买,或者不是这个邮箱买的,或者是vpp的序列码不绑定
               if(licenseCodes.size()>1||!licenseCodeResult.getEmail().equals(email)){
                   return "1";
               }
                //如果用户只购买了一个，则弹出权益转移弹窗
                return "3";
            }
        }
        //ios免费版和mac免费版
        if (model.equals(ModelEnum.MAC_APP_STORE_FREE.getEnglishDescription())||model.equals(ModelEnum.IOS_APP_STORE_FREE.getEnglishDescription())){
            //app上如果没有付费权益
            if(CollectionUtils.isEmpty(appTransactionIds)){
                List<OrdersVO> list = orderApi.getOrderListByStatus(3,userId).getResult();
                if (CollectionUtils.isEmpty(list)&&isFreeAccount.equals("1")){
                    //如果是没有过购买的新用户账号跳转pricing页面
                    //用户有没有过购买的订单
                    return "2";
                }
                //有过购买的老用户登录成功
                return "1";
            }else {
                boolean isBind = orderApi.bindAppleTransaction(appTransactionIds,userId,appBundleId,isHaveActiveSubscription).getResult();
                if(isBind){
                    EmailSendBO bo = new EmailSendBO();
                    bo.setUseEvent(EmailCodeTypeEnum.TRANSFER_SUCCESS_APP.value());
                    //设置title
                    Map<String,String> titleMap = new HashMap<>();
                    //设置内容
                    Map<String,String> contentMap = new HashMap<>();
                    contentMap.put("@email@",email);
                    bo.setSendTitleContent(titleMap);
                    //设置内容
                    bo.setSendContent(contentMap);
                    bo.setToEmail(email);
                    emailApi.sendEmail(bo);
                    return "3";
                }else {
                    return "1";
                }
            }
        }
        return "1";
    }

    public List<LicenseCodeResult> getLicenseByDevice(String deviceSign) {
       return PDFReaderUtils.getLicenseByDevice(pdfReaderUrl,deviceSign);
    }

    private void handleMaxDevice(Principal principal, String userId, List<LoginDevice> devices) {
        // 获取所有设备的 token，并存储到 map 中
        Map<String, String> deviceTokenMap = devices.stream()
                .collect(Collectors.toMap(
                        device -> device.getAppId() + CommonConstant.STRING_SIGN_COLON + device.getUniqueSn(),
                        device -> redisUtils.hget(device.getAppId() + AuthConstant.DEVICE_LOGIN_TOKEN_KEY,
                                userId + CommonConstant.STRING_SIGN_COLON + device.getUniqueSn())
                ));

        // 过滤掉 null 的 token
        List<String> validTokenList = deviceTokenMap.values().stream()
                .filter(Objects::nonNull)
                .collect(Collectors.toList());

        String currentToken = getToken(principal);

        if (validTokenList.contains(currentToken)) {
            // 当前设备的 token 存在，登出其他设备
            devices.stream()
                    .filter(device -> {
                        String token = deviceTokenMap.get(device.getAppId() + CommonConstant.STRING_SIGN_COLON + device.getUniqueSn());
                        return token != null && !currentToken.equals(token); // 过滤 null 和不同的 token
                    })
                    .forEach(device -> {
                        String token = deviceTokenMap.get(device.getAppId() + CommonConstant.STRING_SIGN_COLON + device.getUniqueSn());
                        if (token != null) {
                            authService.logout(token, userId, device.getUniqueSn(), device.getAppId(), true);
                        }
                    });
        } else {
            // 当前设备不在 validTokenList 中，保留最近登录设备
            LoginDevice latestDevice = devices.stream()
                    .max(Comparator.comparing(LoginDevice::getCreatedAt))
                    .orElseThrow(() -> new IllegalStateException("No device found"));

            devices.stream()
                    .filter(device -> !device.equals(latestDevice))
                    .forEach(device -> {
                        String token = deviceTokenMap.get(device.getAppId() + CommonConstant.STRING_SIGN_COLON + device.getUniqueSn());
                        if (token != null) {
                            authService.logout(token, userId, device.getUniqueSn(), device.getAppId(), true);
                        }
                    });
        }
    }

    /**
     * 序列码权益转移接口 (DMG EXE 官网)
     * @param principal principal
     * @return Boolean
     */
    @PostMapping("/codeTransfer")
    public ResultMap<Boolean> transfer(Principal principal){
        String userId = getUserId(principal);
        UserInfoVO user = userApi.getMemberInfoById(userId).getResult();
        //序列码转移直接插入就行
        List<LicenseCodeResult> licenseCodes = PDFReaderUtils.getLicenseByEmail(pdfReaderUrl,user.getEmail());
        if (CollectionUtils.isNotEmpty(licenseCodes)){
            String productCode = CodeConvertEnum.getNewCodeByOldCode(licenseCodes.get(0).getCode());
            ProductVO productVO = productApi.getProductByCode(productCode).getResult();
            //插入订阅
            UserSubscriptionInfo newInfo = new UserSubscriptionInfo().withId(CommonUtils.generateId())
                    //status 已注册
                    .withUserId(user.getId()).withStatus(1).withEndDate(CommonUtils.getDateByCycle(new Date(),100)).withAppId(CommonConstant.PDF_READER_PRO_APP_ID).withPlatform(productVO.getPlatform()).withPayType(PayTypeEnum.NO_AUTO.value()).withProductId(productVO.getId());
            userSubscriptionInfoApi.insert(newInfo);
            UserVO userVO = new UserVO();
            userVO.setId(user.getId());
            userVO.setAccountType(AccountTypeEnum.PAID_ACCOUNT.value());
            userApi.updateUser(userVO);
            PDFReaderUtils.invalid(pdfReaderUrl, licenseCodes.get(0).getCdkey());
            RightBindLogDTO rightBindLogDTO = RightBindLogDTO.builder().id(CommonUtils.generateId())
                    .cdkey(licenseCodes.get(0).getCdkey())
                    .licenseProductCode(licenseCodes.get(0).getCode())
                    .productId(productVO.getId())
                    .type("1")
                    .userId(user.getId())
                    .build();
            rightBindLogApi.save(rightBindLogDTO);
            EmailSendBO bo = new EmailSendBO();
            bo.setUseEvent(EmailCodeTypeEnum.TRANSFER_SUCCESS_WEB.value());
            //设置title
            Map<String,String> titleMap = new HashMap<>();
            //设置内容
            Map<String,String> contentMap = new HashMap<>();
            contentMap.put("@email@",user.getEmail());
            bo.setSendTitleContent(titleMap);
            //设置内容
            bo.setSendContent(contentMap);
            bo.setToEmail(user.getEmail());
            emailApi.sendEmail(bo);
        }
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

    /**
     * 获取用户当前的会员情况（用于用户注销时提醒用户）
     *
     * @return List
     */
    @GetMapping("/rightList")
    public ResultMap<List<SimpleUserSubscriptionInfoVO>> rightList(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            {
                String userId = ((CustomUserDetails) ((OAuth2Authentication) principal).getPrincipal()).getId();
                // 获取用户的基本信息
                UserInfoVO user = userApi.getMemberInfoById(userId).getResult();
                AtomicBoolean isUpgrade = new AtomicBoolean(false);
                List<SimpleUserSubscriptionInfoVO> resultList = user.getSubscriptionInfoList().stream().map(item -> {
                    if(item.getCode().contains("upgrade")&&item.getCode().contains("subscription")){
                        isUpgrade.set(true);
                    }
                    item.setPlatforms(convertToLowerCase(item.getPlatforms()));
                    // 没有购买显示当前时间
                    if (ObjectUtils.isEmpty(item.getEndDate())) {
                        item.setEndDate(new Date());
                    }
                    if (ObjectUtils.isEmpty(item.getPoint())) {
                        item.setPoint(0);
                    }
                    //默认不是ai
                    item.setIsAi(StringUtils.isEmpty(item.getIsAi()) ? "0" : item.getIsAi());
                    SimpleUserSubscriptionInfoVO vo = new SimpleUserSubscriptionInfoVO();
                    BeanUtils.copyProperties(item, vo);
                    return vo;
                }).collect(Collectors.toList());
                if (isUpgrade.get()) {
                    resultList.removeIf(info -> Objects.equals(info.getPlatforms(), "3")); // 通过 removeIf 来移除 level 为 0 的记录
                }
                if (resultList.size() > 1) {
                    resultList.removeIf(info -> StringUtils.isNotEmpty(info.getLevels())&&Objects.equals(info.getLevels(), "1")); // 通过 removeIf 来移除 level 为 0 的记录
                }
                return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, resultList);
            }
        }
        throw new BackendRuntimeException("net error");
    }

    /**
     * 登出(登出设备)
     *
     * @param deviceSign 设备号
     * @param appId      appId （PDF Reader Pro 为1）
     * @return Boolean
     */
    @PostMapping("/logout")
    public ResultMap<Boolean> logout(@RequestParam(required = false) String deviceSign, @RequestParam String appId, HttpServletRequest request, Principal principal) {
        String token = getToken(principal);
        String userId = getUserId(principal);
        authService.logout(token, userId, deviceSign, appId, true);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

    /**
     * 用户手动注销
     *
     * @param code  邮箱验证码
     * @param appId appId （PDF Reader Pro 为1）
     * @return Boolean
     */
    @PostMapping("/logOffForUser")
    public ResultMap<Map<String, String>> logOffForUser(@RequestParam String code, @RequestParam String appId, Principal principal) {
        String userId = getUserId(principal);
        userApi.logOffForUser(code, userId, appId);
        UserVO userVO = userApi.getById(userId).getResult();
        if (!appId.equals(CommonConstant.PDF_READER_PRO_APP_ID)) {
            authService.logOutAll(userVO);
        }
        LocalDateTime now = LocalDateTime.now();

        // 计算7天后的时间
        LocalDateTime sevenDaysLater = now.plusDays(7);
        // 定义日期格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        // 格式化当前时间和7天后的时间
        String currentTimeFormatted = now.format(formatter);
        String sevenDaysLaterFormatted = sevenDaysLater.format(formatter);
        Map<String, String> result = new HashMap<>();
        result.put("currentTime", currentTimeFormatted);
        result.put("logOffTime", sevenDaysLaterFormatted);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, result);
    }

    @PostMapping("/logOutAllSystem")
    public ResultMap<Boolean> logOutAllSystem(@RequestBody LogOutSystemDTO logOutSystemDTO) {
        UserVO userVO = userApi.getById(logOutSystemDTO.getUserId()).getResult();
        authService.logOutAll(userVO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

    @PostMapping("/logOutSystem")
    public ResultMap<Boolean> logOutSystem(@RequestBody LogOutSystemDTO logOutSystemDTO) {
        authService.logOutDevice(logOutSystemDTO.getAppId(), logOutSystemDTO.getUserId(), logOutSystemDTO.getUniqueSn());
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

    /**
     * 撤销注销申请
     *
     * @return Boolean
     */
    @PostMapping("/revokeCancel")
    public ResultMap<Boolean> revokeCancel(Principal principal) {
        UserVO userVO = userApi.getById(getUserId(principal)).getResult();
        UserVO updateVO = new UserVO();
        updateVO.setId(userVO.getId());
        updateVO.setUpdatedAt(new Date());
        updateVO.setValidFlag(PDFOfficeUserStatusEnum.VALID.value().toString());
        userApi.updateUser(updateVO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

    /**
     * 赠送7天试用
     *
     * @param appId appid 1
     * @return String
     */
    @PostMapping("/getTrail")
    public ResultMap<String> getTrail(@RequestParam String appId, Principal principal) {
        CreateSubscriptionDTO dto = new CreateSubscriptionDTO();
        dto.setStartDate(LocalDateTime.now());
        dto.setEndDate(LocalDateTime.now().plusDays(7));
        dto.setProductId("23");
        String userId = getUserId(principal);
        dto.setUserId(userId);
        dto.setAppId(appId);
        dto.setPayment(PaymentMethodEnum.NO_NEED_TO_PAY.getValue());
        UserInfoVO user = userApi.getMemberInfoById(userId).getResult();
        dto.setEmail(user.getEmail());
        dto.setSubscriptionType(SubscriptionTypeEnum.TRAIL.value());
        subscriptionApi.createSubscription(dto);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, CommonConstant.RESULT_SUCCESS);
    }

    @PostMapping("/getInviteNum")
    public ResultMap<Integer> getInviteNum( Principal principal) {
        String userId = getUserId(principal);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, userApi.getInviteNum(userId).getResult());
    }

    //收集退订原因
    @PostMapping("/cancelReason")
    public ResultMap<String> cancelReason(@RequestBody CancelReasonDTO cancelReasonDTO, Principal principal) {
        String userId = getUserId(principal);
        UserVO vo = userApi.getById(userId).getResult();
        EmailSendBO bo = new EmailSendBO();
        bo.setUseEvent("member_cancel_reason");
        //设置title
        Map<String,String> titleMap = new HashMap<>();
        titleMap.put("@model@",cancelReasonDTO.getModel());
        titleMap.put("@title@", FeedbackType.fromCode(cancelReasonDTO.getTitle()).getDescription());
        titleMap.put("@email@",vo.getEmail());
        //设置内容
        Map<String,String> contentMap = new HashMap<>();
        contentMap.put("@content@",StringUtils.isEmpty(cancelReasonDTO.getContent())?"":cancelReasonDTO.getContent());
        bo.setSendTitleContent(titleMap);
        //设置内容
        bo.setSendContent(contentMap);
        bo.setToEmail("support@pdfreaderpro.com");
        emailApi.sendEmail(bo);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS);
    }

    /**
     * app端到web端获取web端新token
     * @param principal principal
     * @return TokenVO
     */
    @PostMapping("/convertWebToken")
    public ResultMap<TokenVO> convertWebToken(Principal principal) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, authService.convertWebToken(getUserId(principal)));
    }

    private String getUserId(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            return ((CustomUserDetails) (((OAuth2Authentication) principal).getPrincipal())).getId();
        }
        throw new BackendRuntimeException(CommonConstant.EXCEPTION_MSG_LOGIN_USER_NOT_FOUND);
    }

    private String getToken(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            return ((OAuth2AuthenticationDetails) (((OAuth2Authentication) principal).getDetails())).getTokenValue();
        }
        throw new BackendRuntimeException(CommonConstant.EXCEPTION_MSG_LOGIN_USER_NOT_FOUND);
    }
}
