package cn.kdan.cloud.pdf.office.sso.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.LoginDeviceApi;
import cn.kdan.cloud.pdf.office.api.account.feign.OauthClientDetailsApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserSubscriptionInfoApi;
import cn.kdan.cloud.pdf.office.api.account.vo.LoginDevice;
import cn.kdan.cloud.pdf.office.api.account.vo.OauthClientDetails;
import cn.kdan.cloud.pdf.office.api.account.vo.UserSubscriptionInfo;
import cn.kdan.cloud.pdf.office.api.email.bo.EmailSendBO;
import cn.kdan.cloud.pdf.office.api.email.feign.EmailApi;
import cn.kdan.cloud.pdf.office.api.product.enums.PlatformEnum;
import cn.kdan.cloud.pdf.office.api.product.feign.LoginLogApi;
import cn.kdan.cloud.pdf.office.api.product.feign.PrizeApi;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.LoginInVO;
import cn.kdan.cloud.pdf.office.api.product.vo.PrizeVO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.dto.ResetPasswordDTO;
import cn.kdan.cloud.pdf.office.common.dto.UserRegisterDTO;
import cn.kdan.cloud.pdf.office.common.dto.UserResetPasswordDTO;
import cn.kdan.cloud.pdf.office.common.dto.ValidUserDTO;
import cn.kdan.cloud.pdf.office.common.enums.*;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PayTypeEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.RedisUtils;
import cn.kdan.cloud.pdf.office.common.vo.TokenVO;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.sso.constant.AuthConstant;
import cn.kdan.cloud.pdf.office.sso.enums.VerifyTypeEnum;
import cn.kdan.cloud.pdf.office.sso.pojo.LoginParam;
import cn.kdan.cloud.pdf.office.sso.service.AuthService;
import cn.kdan.cloud.pdf.office.sso.utils.RSAUtils;
import cn.kdan.cloud.pdf.office.sso.utils.TokenUtils;
import com.alibaba.nacos.common.utils.CollectionUtils;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.context.annotation.Lazy;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.common.OAuth2RefreshToken;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.authentication.OAuth2AuthenticationDetails;
import org.springframework.security.oauth2.provider.endpoint.AuthorizationEndpoint;
import org.springframework.security.oauth2.provider.endpoint.TokenEndpoint;
import org.springframework.security.oauth2.provider.token.TokenStore;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.bind.support.SimpleSessionStatus;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.view.RedirectView;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.security.Principal;
import java.time.LocalDateTime;
import java.time.Month;
import java.time.ZoneOffset;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

@Service
@RefreshScope
@RequiredArgsConstructor
@Transactional
public class AuthServiceImpl implements AuthService {

    private final Logger logger = LoggerFactory.getLogger(AuthServiceImpl.class);

    @Autowired
    private RedisUtils<String, String> redisUtils;

    @Autowired
    private UserApi userApi;
    @Autowired
    private ProductApi productApi;
    @Autowired
    private LoginLogApi loginLogApi;

    @Autowired
    private UserSubscriptionInfoApi userSubscriptionInfoApi;

    @Autowired
    private LoginDeviceApi loginDeviceApi;
    @Autowired
    private OauthClientDetailsApi oauthClientDetailsApi;

    @Autowired
    private AuthorizationEndpoint authorizationEndpoint;

    @Autowired
    private TokenEndpoint tokenEndpoint;
    @Autowired
    private EmailApi emailApi;
    @Autowired
    private PrizeApi prizeApi;


    @Value("${security.oauth2.client.user-authorization-uri}")
    private String authorizeUrl;

    @Value("${security.oauth2.client.access-token-uri}")
    private String tokenUri;
    @Value("${security.oauth2.client.client-id}")
    private String clientId;

    @Override
    public String getAuthorizeCode(String userId,String clientId, HttpServletRequest request) {
        //获取用户的基本信息
        Principal principal = request.getUserPrincipal();
        Map<String, Object> model = new HashMap<>();
        Map<String,String> params = new HashMap<>();
        params.put("response_type","code");
        params.put("client_id",clientId);
        params.put("redirect_uri",AuthConstant.DEFAULT_REDIRECT_URI);
        SessionStatus status = new SimpleSessionStatus();
        ModelAndView authorizeMV = authorizationEndpoint.authorize(model, params, status, principal);
        RedirectView view = (RedirectView) authorizeMV.getView();
        if(view != null && StringUtils.isNotEmpty(view.getUrl())) {
            return view.getUrl().substring(view.getUrl().indexOf("code=")+5);
        } else {
            return null;
        }
    }

    @Override
    public TokenVO getToken(String code, String clientId, HttpServletRequest request) {
        Map<String,String> params = new HashMap<>();
        params.put("grant_type","authorization_code");
        params.put("code",code);
        params.put("client_id",clientId);
        //获取用户服务节点信息
        params.put("redirect_uri",AuthConstant.DEFAULT_REDIRECT_URI);
        params.put("client_secret","");
        User principal = new User(clientId,"", Collections.emptySet());
        UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(principal, null, Collections.emptySet());
        try {
            ResponseEntity<OAuth2AccessToken> oAuth2AccessTokenResponseEntity = tokenEndpoint.postAccessToken(authenticationToken, params);
            return convert(oAuth2AccessTokenResponseEntity.getBody());
        } catch (HttpRequestMethodNotSupportedException e) {
            logger.error(e.getMessage(),e);
        }
        return null;
    }

    @Override
    public TokenVO emailLogin(String email, String password, String appId, String platformType, String deviceSign, String model,HttpServletRequest request) {
        //如果是管理平台就去查管理平台的用户表
        //统一会员系统不根据appid去查用户了
        UserVO userVO = userApi.getByAppAccount(email, platformType).getResult();
        //检查用户存在
        checkUser(userVO);
        checkPassword(password, userVO.getDigestPassword());
        //检查邮件验证码
        // checkEmailCodeValid(EmailCodeTypeEnum.LOGIN,email,code);
        // 检查设备是否达到上限
        checkLoginDeviceNum(userVO.getId(), deviceSign, appId);
        TokenVO vo = getTokenByUser(password, userVO);
        Integer platform = "mac".equalsIgnoreCase(model)?1:2;
        UserSubscriptionInfo userSubscriptionInfo = userSubscriptionInfoApi.getByUserIdAndPlatform(userVO.getId(), platform).getResult();
        if(ObjectUtils.isEmpty(userSubscriptionInfo)){
            ProductVO productVO = productApi.getProductByAppIdAndModel(appId, platform).getResult();
            UserSubscriptionInfo newInfo = new UserSubscriptionInfo().withId(CommonUtils.generateId())
                    //status 已注册
                    .withUserId(userVO.getId()).withStatus(0).withAppId(appId).withPlatform(platform).withPayType(PayTypeEnum.NO_AUTO.value()).withProductId(productVO.getId().toString());
            userSubscriptionInfoApi.insert(newInfo);
        }
        //关联设备登录
        relateTokenAndDevice(vo.getAccess_token(), userVO.getId(), deviceSign, appId, model);
        return vo;
    }

    private void checkProDeviceNum(String userId, String deviceSign, String appId,String model) {
        //查询用户对于某个app会员，以及会员支持的设备数
        List<UserSubscriptionInfoVO> subscriptionInfoList = userSubscriptionInfoApi.getByUserIdAndAppId(userId, appId).getResult();
        if(CollectionUtils.isEmpty(subscriptionInfoList)&&CommonConstant.PDF_READER_PRO_APP_ID.equals(appId)){
            //默认给一个免费版的"订阅"
            UserSubscriptionInfo newInfo = new UserSubscriptionInfo().withId(CommonUtils.generateId())
                    //status 已注册
                    .withUserId(userId).withStatus(0).withAppId(appId).withPlatform(PDFReaderProPlatformEnum.FREE.getCode()).withPayType(PayTypeEnum.NO_AUTO.value()).withProductId("SP2024101400001");
            userSubscriptionInfoApi.insert(newInfo);
            subscriptionInfoList = userSubscriptionInfoApi.getByUserIdAndAppId(userId, appId).getResult();
        }
        UserSubscriptionInfoVO activeSubscription = findActiveSubscription(subscriptionInfoList, null);
        //根据userId查询如果list>2，并且设备号不属于其中的某一个
        List<LoginDevice> list = loginDeviceApi.getByUserId(userId, appId).getResult();
        //验证token有效性，如果token已经过期则删除对应登录设备信息
        list.forEach(item->{
            String token = redisUtils.hget(appId + AuthConstant.DEVICE_LOGIN_TOKEN_KEY, userId + CommonConstant.STRING_SIGN_COLON + item.getUniqueSn());
            OAuth2AccessToken oAuth2AccessToken = tokenStore.readAccessToken(token);
            //如果token过期了，删除设备登录记录
            if(ObjectUtils.isEmpty(oAuth2AccessToken)||oAuth2AccessToken.isExpired()){
                loginDeviceApi.delete(item.getUniqueSn(),userId,appId,true);
            }
        });
        //查询经过处理后的设备记录
        List<LoginDevice> finalList = loginDeviceApi.getByUserId(userId, appId).getResult();
        List<String> snList = finalList.stream().map(LoginDevice::getUniqueSn).collect(Collectors.toList());
        Integer deviceNum = activeSubscription.getMaxDeviceNum();
//        List<String> platforms = Arrays.stream(activeSubscription.getPlatforms().split(","))
//                .map(String::toLowerCase) // 转换为小写
//                .collect(Collectors.toList());
        //如果登录的事当前平台
        if(CollectionUtils.isNotEmpty(finalList)&&finalList.size() >= deviceNum&&!snList.contains(deviceSign)){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_DEVICE_NUM_MAX);
        }
    }

    @Override
    public Integer getMaxDeviceName(String email, String appId) {
        UserVO userVO = userApi.getByAppAccount(email, "0").getResult();
        List<UserSubscriptionInfoVO> subscriptionInfoList = userSubscriptionInfoApi.getByUserIdAndAppId(userVO.getId(), appId).getResult();
        UserSubscriptionInfoVO activeSubscription = findActiveSubscription(subscriptionInfoList, null);
        return activeSubscription.getMaxDeviceNum();
    }


    //获取用户登录设备列表
    @Override
    public List<LoginDevice> getLoginDeviceList(String email, String appId) {
        UserVO userVO = userApi.getByEmail(email).getResult();
        if(ObjectUtils.isEmpty(userVO)){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_EMAIL_NOT_REGISTER);
        }
        return loginDeviceApi.getByUserId(userVO.getId(), appId).getResult();
    }

    @Override
    public UserSubscriptionInfoVO findActiveSubscription(List<UserSubscriptionInfoVO> subscriptionInfoList, String model) {
        return CommonUtils.findActiveSubscription(subscriptionInfoList,model); // 返回结果，如果没有找到则返回null
    }

    @Override
    public Boolean judgeActiveSubscription(List<UserSubscriptionInfoVO> subscriptionInfoList) {
        // 过滤掉isAi为true的记录
        List<UserSubscriptionInfoVO> filteredList = subscriptionInfoList.stream()
                .filter(info -> info.getStatus() == 1)                          // 筛选 status = 1
                .filter(info -> info.getPlatform()==2 && info.getAppId().equals("1"))               // 筛选 paymentModel = "1"// 筛选 isAi 为空
                .collect(Collectors.toList());
        if (CollectionUtils.isEmpty(filteredList)){
            return false;
        }else {
            return true;
        }
    }

    @Override
    public int getAiPoints(List<UserSubscriptionInfoVO> subscriptionInfoList) {
        int aiPoints = 0; // 初始化AI点数

        for (UserSubscriptionInfoVO info : subscriptionInfoList) {
            // 检查isAi是否为true，且points为null时视为0
            if (ValidStatusEnum.VALID.value().equals(info.getIsAi())) {
                int points = info.getPoint() != null ? info.getPoint() : 0; // 如果points为null则设为0
                aiPoints += points; // 累加有效的points
            }
        }

        return aiPoints; // 返回总的AI点数
    }
    @Override
    public UserSubscriptionInfoVO getActiveAi(List<UserSubscriptionInfoVO> subscriptionInfoList) {
        for (UserSubscriptionInfoVO info : subscriptionInfoList) {
            // 检查isAi是否为true，且points为null时视为0
            if (ValidStatusEnum.VALID.value().equals(info.getIsAi())) {
               return info;
            }
        }
        return new UserSubscriptionInfoVO();
    }

    // 根据levels字符串获取对应的优先级
    private int getLevelPriority(UserSubscriptionInfoVO info) {
        // 假设 levels 是以逗号分隔的字符串
        String[] levels = info.getLevels().split(",");
        int maxPriority = Integer.MIN_VALUE;

        for (String level : levels) {
            int lvl = Integer.parseInt(level.trim());
            maxPriority = Math.max(maxPriority, lvl);
        }

        // 定义优先级
        switch (maxPriority) {
            case 3: return 3; // 高级版
            case 2: return 2; // 标准版
            case 1: return 1; // 免费版
            default: return 0; // 未知或无效等级
        }
    }


    private void checkPassword (String userPassword,String realPassword){
        if(StringUtils.isEmpty(realPassword)){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_PASSWORD_ERROR);
        }
        realPassword =  new RSAUtils().decrypt(realPassword);
        if(!userPassword.equals(realPassword)){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_PASSWORD_ERROR);
        }
    }

    @Override
    public TokenVO emailRegister(UserRegisterDTO userRegisterDTO) {
        ValidUserDTO dto = new ValidUserDTO();
        dto.setValidType("2");
        dto.setUsername(userRegisterDTO.getUsername());
        dto.setPlatformType(userRegisterDTO.getPlatformType());
        dto.setPassword(userRegisterDTO.getPassword());
        String model = userRegisterDTO.getModel();
        userRegisterDTO.setModel(model.replaceFirst(".", String.valueOf(Character.toUpperCase(model.charAt(0)))));
        validUserRegisterParam(dto);
        ResultMap<Boolean> resultMap = userApi.register(userRegisterDTO);
        if(resultMap.getCode() == CommonConstant.SUCCESS){
            //如果是管理平台就去查管理平台的用户表
            UserVO userVO = userApi.getByAppAccount(userRegisterDTO.getUsername(),
                    userRegisterDTO.getPlatformType()).getResult();
            //生成token
            TokenVO vo = getTokenByUser(CommonUtils.generateDateId(), userVO);
            //关联设备登录
            relateTokenAndDevice(vo.getAccess_token(), userVO.getId(), userRegisterDTO.getDeviceSign(), userRegisterDTO.getAppId(), userRegisterDTO.getModel());
            Integer platform = "mac".equalsIgnoreCase(userRegisterDTO.getModel())? PlatformEnum.MAC.getValue():PlatformEnum.WINDOWS.getValue();
            ProductVO productVO = productApi.getProductByAppIdAndModel(userRegisterDTO.getAppId(), platform).getResult();
            UserSubscriptionInfo newInfo = new UserSubscriptionInfo().withId(CommonUtils.generateId())
                    //status 已注册
                    .withUserId(userVO.getId()).withStatus(0).withAppId(userRegisterDTO.getAppId()).withPlatform(platform).withPayType(PayTypeEnum.NO_AUTO.value()).withProductId(productVO.getId().toString());
            userSubscriptionInfoApi.insert(newInfo);

            return vo;
        }
        throw new BackendRuntimeException(ExceptionEnum.EMAIL_REGISTER_ERROR);
    }

    @Override
    public void resetPassword(UserResetPasswordDTO userResetPasswordDTO) {
        //检查验证码
        if(!userResetPasswordDTO.getFirstPassword().equals(userResetPasswordDTO.getSecondPassword())){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_PASSWORD_NOT_INCONSISTENT);
        }
        //检查密码长度
        if(userResetPasswordDTO.getFirstPassword().length()<6){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_PASSWORD_SIZE_MIN);
        }
        if(userResetPasswordDTO.getFirstPassword().length()>24){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_PASSWORD_SIZE_MAX);
        }
        checkEmailCodeValid(EmailCodeTypeEnum.USER_RESET_PASSWORD,userResetPasswordDTO.getAccount(),userResetPasswordDTO.getVerifyCode(), userResetPasswordDTO.getAppId());
        UserVO userVO = userApi.getByAppAccount(userResetPasswordDTO.getAccount(), userResetPasswordDTO.getPlatformType()).getResult();
        //修改密码
        userVO.setUpdatedAt(new Date());
        userVO.setDigestPassword(new RSAUtils().encrypt(userResetPasswordDTO.getFirstPassword()));
        userApi.updateUser(userVO);
        logOutAll(userVO);

    }

    @Override
    public void memberResetPassword(ResetPasswordDTO resetPasswordDTO) {
        //检查密码长度
        if(resetPasswordDTO.getPassword().length()<6){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_PASSWORD_SIZE_MIN);
        }
        if(resetPasswordDTO.getPassword().length()>24){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_PASSWORD_SIZE_MAX);
        }
        checkEmailCodeValid(EmailCodeTypeEnum.MEMBER_RESET_PASSWORD,resetPasswordDTO.getEmail(),resetPasswordDTO.getVerifyCode(), resetPasswordDTO.getAppId());
        UserVO userVO = userApi.getByAppAccount(resetPasswordDTO.getEmail(), "0").getResult();
        //修改密码
        userVO.setUpdatedAt(new Date());
        userVO.setDigestPassword(new RSAUtils().encrypt(resetPasswordDTO.getPassword()));
        userApi.updateUser(userVO);
        logOutAll(userVO);
        EmailSendBO bo = new EmailSendBO();
        bo.setToEmail(resetPasswordDTO.getEmail());
        //设置事件
        bo.setUseEvent(EmailCodeTypeEnum.MEMBER_UPDATE_PASSWORD_SUCCESS.value());
        //设置title
        Map<String,String> titleMap = new HashMap<>();
        //设置内容
        Map<String,String> contentMap = new HashMap<>();
        // 获取当前的时间，设置为 GMT 时区
        ZonedDateTime now = ZonedDateTime.now(ZoneOffset.UTC);
        // 定义目标日期时间格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy/MM/dd HH:mm '(GMT)'");
        // 格式化当前时间
        String formattedTime = now.format(formatter);
        contentMap.put("@time@",formattedTime);
        contentMap.put("@email@",resetPasswordDTO.getEmail());
        bo.setSendTitleContent(titleMap);
        //设置内容
        bo.setSendContent(contentMap);
        emailApi.sendEmail(bo);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public TokenVO memberLogin(LoginParam loginParam) {
        //查询用户信息
        UserVO userVO = userApi.getByAppAccount(loginParam.getEmail(),"0").getResult();
        //如果检查邮件验证码
        if(!StringUtils.isEmpty(loginParam.getCode())){
            //检查用户存在,如果不存在的话对用户进行注册(如果是验证码注册则对用户自动注册，密码为空)
            checkEmailCodeValid(EmailCodeTypeEnum.MEMBER_LOGIN,loginParam.getEmail(),loginParam.getCode(),loginParam.getAppId());
            userVO = checkUserAndAutoRegister(userVO,loginParam);
        }
        if(!StringUtils.isEmpty(loginParam.getPassword())){
            checkUser(userVO);
            checkPassword(loginParam.getPassword(), userVO.getDigestPassword());
        }
        // 检查设备是否达到上限(app端登录才算)
        if(StringUtils.isNotEmpty(loginParam.getModel())){
            checkProDeviceNum(userVO.getId(), loginParam.getDeviceSign(), loginParam.getAppId(),loginParam.getModel());
        }
        handOldUser(loginParam, userVO);
        TokenVO vo = getTokenByUser(!StringUtils.isEmpty(loginParam.getPassword())?loginParam.getPassword():loginParam.getCode(), userVO);
        // 关联设备和token app端登录才算
        if(StringUtils.isNotEmpty(loginParam.getModel())){
            relateTDevice(vo.getAccess_token(), userVO.getId(), loginParam);
        }else{
            recordToken(vo.getAccess_token(), userVO.getId(), loginParam);
        }
        //记录登录历史设备
        recordHistoryDevices(loginParam, userVO);
        return vo;
    }

    @Override
    public TokenVO convertWebToken(String userId) {
        UserVO userVO = userApi.getById(userId).getResult();
        TokenVO vo = getTokenByUser(userId, userVO);
        LoginParam loginParam = new LoginParam();
        loginParam.setAppId("1");
        recordToken(vo.getAccess_token(), userVO.getId(), loginParam);
        return getTokenByUser(userId, userVO);
    }

    private void recordHistoryDevices(LoginParam loginParam, UserVO userVO) {
        LoginInVO loginInVO = new LoginInVO();
        BeanUtils.copyProperties(loginParam,loginInVO);
        loginInVO.setUserId(userVO.getId());
        loginInVO.setType("1");
        loginInVO.setModel(loginParam.getModelDetail());
        loginInVO.setUniqueSn(loginParam.getDeviceSign());
        loginLogApi.insert(loginInVO);
    }

    private void handOldUser(LoginParam loginParam, UserVO userVO) {
        List<UserSubscriptionInfoVO> subscriptionInfoList = userSubscriptionInfoApi.getByUserIdAndAppId(userVO.getId(), loginParam.getAppId()).getResult();
        if(CollectionUtils.isEmpty(subscriptionInfoList)&&CommonConstant.PDF_READER_PRO_APP_ID.equals(loginParam.getAppId())){
            //默认给一个免费版的"订阅"
            UserSubscriptionInfo newInfo = new UserSubscriptionInfo().withId(CommonUtils.generateId())
                    //status 已注册
                    .withUserId(userVO.getId()).withStatus(0).withAppId(loginParam.getAppId()).withPlatform(PDFReaderProPlatformEnum.FREE.getCode()).withPayType(PayTypeEnum.NO_AUTO.value()).withProductId("SP2024101400001");
            userSubscriptionInfoApi.insert(newInfo);
            if(StringUtils.isNotEmpty(loginParam.getInviteUserId())){
                processInvite(loginParam.getInviteUserId());
            }
        }
    }

    @Override
    public void logoutDevice(String deviceId) {
        LoginDevice device = loginDeviceApi.getById(deviceId).getResult();
        if(ObjectUtils.isEmpty(device)){
            return;
        }
        String earliestLoginDeviceToken = redisUtils.hget(device.getAppId() + AuthConstant.DEVICE_LOGIN_TOKEN_KEY,
                device.getUserId() + CommonConstant.STRING_SIGN_COLON + device.getUniqueSn());
        //3.登出设备
        logout( earliestLoginDeviceToken, device.getUserId(), device.getUniqueSn(), device.getAppId(), true);

    }

    @Override
    public void validEmail(String email) {
        UserVO user = userApi.getByAppAccount(email, "0").getResult();
        //邮箱是否被注册
        if (ObjectUtils.isEmpty(user)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_EMAIL_NOT_REGISTER);
        }
    }

    private UserVO checkUserAndAutoRegister(UserVO user, LoginParam loginParam) {
        if(!ObjectUtils.isEmpty(user)){
            return user;
        }
        ValidUserDTO dto = new ValidUserDTO();
        dto.setValidType("2");
        dto.setUsername(loginParam.getEmail());
        dto.setPlatformType(ValidStatusEnum.VALID.value());
        UserRegisterDTO userRegisterDTO = new UserRegisterDTO();
        userRegisterDTO.setUsername(loginParam.getEmail());
        userRegisterDTO.setAppId(loginParam.getAppId());
        String model = StringUtils.isEmpty(loginParam.getModelDetail())?"PC_OFFICIAL":loginParam.getModelDetail();
        userRegisterDTO.setModel(ModelEnum.fromEnglishString(model).getEnglishDescription());
        userRegisterDTO.setPlatformType("0");
        if(StringUtils.isNotEmpty(loginParam.getInviteUserId())){
            userRegisterDTO.setInviteUserId(loginParam.getInviteUserId());
        }
        validUserRegisterParam(dto);
        ResultMap<Boolean> resultMap = userApi.register(userRegisterDTO);
        if(resultMap.getCode() == CommonConstant.SUCCESS){
            //如果是管理平台就去查管理平台的用户表
            UserVO userVO = userApi.getByAppAccount(userRegisterDTO.getUsername(),
                    userRegisterDTO.getPlatformType()).getResult();
            //默认给一个免费版的"订阅"
            ProductVO productVO = productApi.getProduct("SP2024101400001").getResult();
            UserSubscriptionInfo newInfo = new UserSubscriptionInfo().withId(CommonUtils.generateId())
                    //status 已注册
                    .withUserId(userVO.getId()).withStatus(0).withAppId(userRegisterDTO.getAppId()).withPlatform(PDFReaderProPlatformEnum.FREE.getCode()).withPayType(PayTypeEnum.NO_AUTO.value()).withProductId(productVO.getId().toString());
            userSubscriptionInfoApi.insert(newInfo);
            if(StringUtils.isNotEmpty(loginParam.getInviteUserId())){
                processInvite(loginParam.getInviteUserId());
            }
            //注册给用户发送邮件
            sendRegisterEmail(userVO.getEmail());
            return userVO;
        }
        throw new BackendRuntimeException(ExceptionEnum.EMAIL_REGISTER_ERROR);
    }

    /**
     * 注册发送邮件
     * @param
     */
    public void sendRegisterEmail(String email) {
        EmailSendBO emailSendBO = new EmailSendBO();
        //设置事件
        emailSendBO.setUseEvent(EmailCodeTypeEnum.MEMBER_REGISTER_SUCCESS.value());
        //设置title
        Map<String, String> titleMap = new HashMap<>();
        //设置内容
        Map<String, String> contentMap = new HashMap<>();
        emailSendBO.setSendTitleContent(titleMap);
        //设置内容
        emailSendBO.setSendContent(contentMap);
        emailSendBO.setToEmail(email);
        emailApi.sendEmail(emailSendBO);
    }


    public void processInvite(String inviteUserId) {
        Integer count = userApi.getInviteNum(inviteUserId).getResult();
        // 仅在达到特定阈值时发放奖励
        if (count == 15) {
            givePrize(inviteUserId,10,70);
        } else if (count == 10) {
            givePrize(inviteUserId,8,50);
        } else if (count == 5) {
            givePrize(inviteUserId,3,20);
        }
    }

    private void givePrize(String inviteUserId,Integer usdPrice,Integer cnyPrice) {
        // 处理优惠券奖品的逻辑
        // 记录优惠券到奖品表
        PrizeVO prizeVO = new PrizeVO();
        prizeVO.setUserId(inviteUserId);
        //全平台高级版可用
       // prizeVO.setProductId(productApi.getProductByCode("advanced-annual-subscription").getResult().getId());
        prizeVO.setProductId(null);
        prizeVO.setType(PrizeTypeEnum.COUPON.getCode());
        prizeVO.setCode(getUpperCaseSubstringFromUUID(8));
        prizeVO.setStartDate(new Date());
        LocalDateTime localDateTime = LocalDateTime.of(2024, Month.DECEMBER, 31, 23, 59, 59);

        // 将LocalDateTime转换为Date
        Date date = Date.from(localDateTime.atZone(java.time.ZoneId.systemDefault()).toInstant());

        prizeVO.setEndDate(date);
        //1是黑五2是老带新
        prizeVO.setActivityType("2");
        prizeVO.setUsdPrice(BigDecimal.valueOf(usdPrice));
        prizeVO.setCnyPrice(BigDecimal.valueOf(cnyPrice));
        //优惠券可用
        prizeVO.setValidFlag(1);
        prizeApi.insert(prizeVO);
    }

    public static String getUpperCaseSubstringFromUUID(int length) {
        // 生成 UUID
        String uuid = UUID.randomUUID().toString();

        // 获取从第 4 个字符开始的子字符串
        // 注意，Java 字符串是从索引 0 开始，因此要从索引 3 开始
        String substring = uuid.substring(3, Math.min(3 + length, uuid.length()));

        // 转换为大写
        return substring.toUpperCase();
    }

    @Override
    public void logOutAll(UserVO userVO) {
        List<LoginDevice> list = loginDeviceApi.getByUserId(userVO.getId(),null).getResult();
        list.forEach( item ->{
                    String token = redisUtils.hget(item.getAppId() + AuthConstant.DEVICE_LOGIN_TOKEN_KEY,
                            userVO.getId() + CommonConstant.STRING_SIGN_COLON + item.getUniqueSn());
                    logout(token, userVO.getId(),item.getUniqueSn(), item.getAppId(),true);
                }
        );
        //退出官网的token
        String key = "1" + AuthConstant.WEB_LOGIN_TOKEN_KEY + ":" + userVO.getId();
        List<String> tokens = redisUtils.lrange(key, 0, -1);
        // 判断 token 列表是否存在
        if (tokens != null && !tokens.isEmpty()) {
            for (String token : tokens) {
                // 这里可以进行你想要的操作，比如检查 token 是否过期
                logout(token, userVO.getId(),null, null,null);
            }
        }
    }

    @Override
    public void logOutDevice(String appId, String userId, String uniqueSn) {
        String token = redisUtils.hget(appId + AuthConstant.DEVICE_LOGIN_TOKEN_KEY,
                userId + CommonConstant.STRING_SIGN_COLON + uniqueSn);
        logout(token, userId,uniqueSn, appId, true);
    }


    @Override
    public void validUserRegisterParam(ValidUserDTO param) {
        //验证邮箱格式
        String emailRegex = CommonConstant.emailRegex;
        if (!param.getUsername().matches(emailRegex)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_EMAIL_INVALID);
        }
        UserVO user = userApi.getByAppAccount(param.getUsername(), param.getPlatformType()).getResult();
        //邮箱是否被注册
        if (!ObjectUtils.isEmpty(user)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_EMAIL_EXIST);
        }
        if(StringUtils.isNotEmpty(param.getPassword())&&param.getPassword().length()<6){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_PASSWORD_SIZE_MIN);
        }
        if(StringUtils.isNotEmpty(param.getPassword())&&param.getPassword().length()>24){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_PASSWORD_SIZE_MAX);
        }
        if(StringUtils.isNotEmpty(param.getPassword())&&param.getPassword().contains(" ")){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_PASSWORD_CONTAIN_EMPTY);
        }
    }

    @Override
    public void validUserLoginParam(ValidUserDTO param) {
        UserVO user =userApi.getByAppAccount(param.getUsername(), param.getPlatformType()).getResult();
        //邮箱是否被注册
        if (ObjectUtils.isEmpty(userApi.getByAppAccount(param.getUsername(), param.getPlatformType()).getResult())) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_EMAIL_NOT_REGISTER);
        }
        //验证邮箱格式
        String emailRegex = CommonConstant.emailRegex;
        if (!param.getUsername().matches(emailRegex)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_EMAIL_INVALID);
        }
        if(user.getValidFlag().equals(PDFOfficeUserStatusEnum.LOGGED_OUT_ING.value().toString())){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_EMAIL_LOGOUT_ING);
        }
        //停用中的用户无法登陆
        if(user.getValidFlag().equals(PDFOfficeUserStatusEnum.STOP_USE.value().toString())){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_EMAIL_LOGOUT_ING);
        }
    }

    @Override
    public boolean getVerifyCode(EmailCodeTypeEnum action, VerifyTypeEnum type, String receiver, String appId) {
        boolean flag = false;
        if (redisUtils.hexists(CommonConstant.VERIFY_CODE_TIME_OUT + action.value() + CommonConstant.STRING_SIGN_COLON + appId, receiver.toLowerCase())) {
            throw new BackendRuntimeException(ExceptionEnum.VERIFY_CODE_SEND_TOO_QUICKLY);
        }
        String code = CommonUtils.generateVerifyCode();
        switch (type) {
            //发送手机验证码
            case PHONE:
                //忘记密码时手机验证 发送手机短信
                break;
            //发送邮件
            case EMAIL:
                //创建账号
                EmailSendBO bo = new EmailSendBO();
                bo.setToEmail(receiver);
                handleEmailSendBO(action,bo,code);
                if(emailApi.sendEmail(bo).getCode() == HttpStatus.OK.value()){
                    flag = true;
                }
                break;
            default:
                break;
        }
        //存入缓存
        if (flag) {
            redisUtils.hset(CommonConstant.EMAIL_VERIFY_CODE_KEY + action.value() + CommonConstant.STRING_SIGN_COLON + appId, receiver.toLowerCase(), code, AuthConstant.VERIFY_CODE_KEY_EXPIRE_TIME);
            //再次发送计时
            redisUtils.hset(CommonConstant.VERIFY_CODE_TIME_OUT + action.value() + CommonConstant.STRING_SIGN_COLON + appId , receiver.toLowerCase(), "1", AuthConstant.VERIFY_CODE_KEY_RESEND_TIME);
        }
        return flag;
    }

    void handleEmailSendBO (EmailCodeTypeEnum action,EmailSendBO bo,String code) {
        //设置事件
        bo.setUseEvent(action.value());
        //设置title
        Map<String,String> titleMap = new HashMap<>();
        titleMap.put("@code@",code);
        //设置内容
        Map<String,String> contentMap = new HashMap<>();
        contentMap.put("@code@",code);
        contentMap.put("@email@",bo.getToEmail());
        bo.setSendTitleContent(titleMap);
        //设置内容
        bo.setSendContent(contentMap);
    }

    @Resource
    @Lazy
    private TokenStore tokenStore;

    @Override
    public void logout(String token, String userId, String deviceSign, String appId, Boolean isDelete) {
        //官网没有设备id哦
        if(StringUtils.isNotEmpty(deviceSign)){
            deleteTokenAndDevice(userId, deviceSign, appId, isDelete);
        }
        OAuth2AccessToken oAuth2AccessToken = tokenStore.readAccessToken(token);
        if (oAuth2AccessToken != null) {
            OAuth2RefreshToken oAuth2RefreshToken = oAuth2AccessToken.getRefreshToken();
            //从tokenStore中移除token
            tokenStore.removeAccessToken(oAuth2AccessToken);
            tokenStore.removeRefreshToken(oAuth2RefreshToken);
            tokenStore.removeAccessTokenUsingRefreshToken(oAuth2RefreshToken);
        }
    }



    /**
     * 判断用户是否存在
     *
     * @param userVO
     */
    private void checkUser(UserVO userVO) {
        if (ObjectUtils.isEmpty(userVO)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_EMAIL_NOT_REGISTER);
        }
        //注销中的用户无法登陆
        if("16".equals(userVO.getAppId())&&userVO.getValidFlag().equals(PDFOfficeUserStatusEnum.LOGGED_OUT_ING.value().toString())){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_EMAIL_LOGOUT_ING);
        }
        //停用中的用户无法登陆
        if(userVO.getValidFlag().equals(PDFOfficeUserStatusEnum.STOP_USE.value().toString())){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_EMAIL_LOGOUT_ING);
        }
    }

    /**
     * 检查设备是否达到当前app上限
     * @param userId 用户id
     * @param deviceSign 设备
     */
    private void checkLoginDeviceNum(String userId, String deviceSign, String appId) {
        //根据userId查询如果list>2，并且设备号不属于其中的某一个
        List<LoginDevice> list = loginDeviceApi.getByUserId(userId, appId).getResult();
        //验证token有效性，如果token已经过期则删除对应登录设备信息
        list.forEach(item->{
            String token = redisUtils.hget(appId + AuthConstant.DEVICE_LOGIN_TOKEN_KEY, userId + CommonConstant.STRING_SIGN_COLON + deviceSign);
            OAuth2AccessToken oAuth2AccessToken = tokenStore.readAccessToken(token);
            //如果token过期了，删除设备登录记录
            if(ObjectUtils.isEmpty(oAuth2AccessToken)||oAuth2AccessToken.isExpired()){
               loginDeviceApi.delete(deviceSign,userId,appId,true);
            }
        });
        //查询经过处理后的设备记录
        List<LoginDevice> finalList = loginDeviceApi.getByUserId(userId, appId).getResult();
        List<String> snList = finalList.stream().map(LoginDevice::getUniqueSn).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(finalList)&&finalList.size() >= 2&&!snList.contains(deviceSign)) {
            //登出用户登录的最早的设备
            logOutOfTheOldestDevice(userId, appId, finalList);
        }
    }


    @Override
    public void checkDeviceStatus (String deviceSign,String appId) {
        LoginDevice loginDevice = loginDeviceApi.getByDeviceSignAndAppId(deviceSign,null,appId).getResult();
        if (!ObjectUtils.isEmpty(loginDevice)&&loginDevice.getValidFlag().equals(ValidStatusEnum.INVALID.value())){
            //无效的设备记录，则是被自动登出最早设备登出的设备，抛出异常
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_THE_ACCOUNT_IS_ALREADY_LOGGED_IN_ON_OTHER_DEVICES);
        }
    }



    /**
     * 登出用户登录的最早的设备
     * @param userId 用户id
     * @param appId appid
     * @param finalList 用户登录设备列表
     */
    private void logOutOfTheOldestDevice(String userId, String appId, List<LoginDevice> finalList) {
        //1.得到用户最早的登录设备记录
        Optional<LoginDevice> earliestLoginDevice = finalList.stream()
                .min(Comparator.comparing(LoginDevice::getCreatedAt));
        //2.获取该设备对应token
        String earliestLoginDeviceToken = redisUtils.hget(appId + AuthConstant.DEVICE_LOGIN_TOKEN_KEY,
                userId + CommonConstant.STRING_SIGN_COLON + earliestLoginDevice.get().getUniqueSn());
        //3.登出设备
        logout( earliestLoginDeviceToken, userId, earliestLoginDevice.get().getUniqueSn(), appId, false);
    }

    /**
     * 校验邮件验证码
     *
     * @param type    邮件验证码类型
     * @param account 用户
     * @param code    验证码
     */
    @Override
    public void checkEmailCodeValid(EmailCodeTypeEnum type, String account, String code, String appId) {
        //获取用户存在redis中的登录邮箱验证码
        String captchaCode = redisUtils.hget(CommonConstant.EMAIL_VERIFY_CODE_KEY + type.value() + CommonConstant.STRING_SIGN_COLON + appId, account.toLowerCase());
        if(StringUtils.isNotEmpty(code) && code.equals(captchaCode)){
            if(!type.value().equals("member_login")){
                redisUtils.hdel(CommonConstant.EMAIL_VERIFY_CODE_KEY + type.value() + CommonConstant.STRING_SIGN_COLON + appId, account.toLowerCase());
            }
            //验证通过删除
        }else{
            throw new BackendRuntimeException(ExceptionEnum.EMAIL_VERIFY_CODE_KEY_ERROR);
        }
    }

    /**
     * 判断邮件验证码是否存在
     *
     * @param type 邮件验证码类型 login  user_register
     * @param account 用户
     * @param code 验证码
     */
    @Override
    public void isEmailCodeValid(EmailCodeTypeEnum type, String account, String code, String appId) {
        //获取用户存在redis中的登录邮箱验证码
        String captchaCode = redisUtils.hget(CommonConstant.EMAIL_VERIFY_CODE_KEY + type.value() + CommonConstant.STRING_SIGN_COLON + appId, account.toLowerCase());
        if(StringUtils.isNotEmpty(code) && !code.equals(captchaCode)){
            throw new BackendRuntimeException(ExceptionEnum.EMAIL_VERIFY_CODE_KEY_ERROR);
        }
    }

    private TokenVO getTokenByUser(String code, UserVO userVO) {
        Set<SimpleGrantedAuthority> authorities = new LinkedHashSet<>();
        String account = userVO.getId();
        CustomUserDetails principal = new CustomUserDetails(userVO.getId(),
                account,
                code,
                true,
                true,
                true,
                true,
                authorities);
        UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(principal, null, Collections.emptySet());
        OauthClientDetails clientDetails = oauthClientDetailsApi.getByClientId(clientId).getResult();
        if (clientDetails == null) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_PLEASE_ADD_AUTH_CONFIG);
        }
        String authCode = TokenUtils.getAuthorizeCode(clientDetails.getClientId(), authenticationToken);
        OAuth2AccessToken token = TokenUtils.getToken(authCode, clientId, clientDetails.getClientSecret());
        return TokenUtils.convert(token);
    }


    /**
     * 将设备的token存在redis ，登出时根据对应的规则去删除
     *
     * @param token token
     * @param userId userId
     * @param deviceSign deviceSign
     * @param appId appId
     * @param model model
     */
    private void relateTokenAndDevice(String token, String userId, String deviceSign, String appId, String model) {
        LoginDevice loginDevice = loginDeviceApi.getByDeviceSignAndAppId(deviceSign,userId,appId).getResult();
        if(ObjectUtils.isEmpty(loginDevice)){
            //如果账号在该设备没有登录过
            LoginDevice loginDeviceBo = new LoginDevice();
            loginDeviceBo.setId(CommonUtils.generateId());
            loginDeviceBo.setCreatedAt(new Date());
            loginDeviceBo.setUpdatedAt(new Date());
            loginDeviceBo.setUniqueSn(deviceSign);
            loginDeviceBo.setUserId(userId);
            loginDeviceBo.setAppId(appId);
            loginDeviceBo.setModel(model);
            //todo
            if(StringUtils.isNotEmpty(model)&&model.contains("Mac")&& "16".equals(appId)){
                loginDeviceBo.setProductId("21");
            }
            if(StringUtils.isNotEmpty(model)&&model.contains("windows")&& "16".equals(appId)){
                loginDeviceBo.setProductId("22");
            }
            loginDeviceApi.create(loginDeviceBo);
        }
        redisUtils.hset(appId + AuthConstant.DEVICE_LOGIN_TOKEN_KEY, userId + CommonConstant.STRING_SIGN_COLON + deviceSign, token);
    }

    /**
     * 新会员系统用，登录时管理设备
     * @param token
     * @param userId
     * @param loginParam
     */
    private void relateTDevice(String token, String userId, LoginParam loginParam) {
        LoginDevice loginDevice = loginDeviceApi.getByDeviceSignAndAppId(loginParam.getDeviceSign(),userId,loginParam.getAppId()).getResult();
        if(ObjectUtils.isEmpty(loginDevice)){
            //如果账号在该设备没有登录过
            LoginDevice loginDeviceBo = new LoginDevice();
            loginDeviceBo.setId(CommonUtils.generateId());
            loginDeviceBo.setCreatedAt(new Date());
            loginDeviceBo.setUpdatedAt(new Date());
            loginDeviceBo.setUniqueSn(loginParam.getDeviceSign());
            loginDeviceBo.setUserId(userId);
            loginDeviceBo.setAppId(loginParam.getAppId());
            //ios 安卓 mac windows
            loginDeviceBo.setModel(loginParam.getModel());
            //设备名称
            loginDeviceBo.setDeviceName(loginParam.getDeviceName());
            loginDeviceApi.create(loginDeviceBo);
        }
        redisUtils.hset(loginParam.getAppId() + AuthConstant.DEVICE_LOGIN_TOKEN_KEY, userId + CommonConstant.STRING_SIGN_COLON + loginParam.getDeviceSign(), token);
    }

    private void recordToken(String token, String userId, LoginParam loginParam) {
        String key = loginParam.getAppId() + AuthConstant.WEB_LOGIN_TOKEN_KEY+ ":" + userId;

        List<String> tokens = redisUtils.lrange(key, 0, -1);
        if(CollectionUtils.isNotEmpty(tokens)){
            List<String> validTokens = new ArrayList<>();
            for (String item : tokens) {
                OAuth2AccessToken oAuth2AccessToken = tokenStore.readAccessToken(item);
                //如果token过期了，删除设备登录记录
                if(!ObjectUtils.isEmpty(oAuth2AccessToken)&&!oAuth2AccessToken.isExpired()){
                    validTokens.add(item); // 仅保留有效的 token
                }
            }
            redisUtils.del(key);
            redisUtils.lSet(key, validTokens);
        }
        redisUtils.rpush(key, token);
    }


    private void deleteTokenAndDevice(String userId, String deviceSign, String appId, Boolean isDelete) {
        // 登录设备表删除数据关联用户
        loginDeviceApi.delete(deviceSign, userId, appId,isDelete);
        redisUtils.hdel(appId + AuthConstant.DEVICE_LOGIN_TOKEN_KEY, userId + CommonConstant.STRING_SIGN_COLON + deviceSign);
    }

    private TokenVO convert(OAuth2AccessToken token) {
        if (token == null) {
            return null;
        }
        TokenVO tokenVO = new TokenVO();
        tokenVO.setAccess_token(token.getValue());
        tokenVO.setExpires_in(token.getExpiresIn()+"");
        tokenVO.setToken_type(token.getTokenType());
        tokenVO.setRefresh_token(token.getRefreshToken().getValue());
        return tokenVO;
    }

    @Override
    public void refreshToken(Principal principal) {
        String token = ((OAuth2AuthenticationDetails)((OAuth2Authentication) principal).getDetails()).getTokenValue();
        Set<String> keySet = new HashSet<>();
        keySet.add(AuthConstant.ACCESS+token);
        keySet.add(AuthConstant.AUTH+token);
        keySet.add(AuthConstant.ACCESS_TO_REFRESH + token);
        long time = 30 * 24 * 60 * 60L;
        keySet.forEach(key -> redisUtils.expire(key, time));
    }


}
